package gcr.mmm.ui;

import edu.berkeley.guir.prefuse.Display;
import edu.berkeley.guir.prefuse.ItemRegistry;
import edu.berkeley.guir.prefuse.action.filter.GraphFilter;
import edu.berkeley.guir.prefuse.activity.ActionList;
import edu.berkeley.guir.prefuse.graph.DefaultNode;
import edu.berkeley.guir.prefuse.graph.io.XMLGraphReader;
import gcr.mmm2.tag.TaggableMedia;

import java.awt.GridLayout;
import java.awt.Panel;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * @author Benjamin Hill
 * 
 * MMM2 Project BSD Licence
 */
public class AMPLayout extends DefaultLayout implements ChangeListener {

    final static int[] MEDIA_TYPES = { TaggableMedia.CONTEXT_CELL,
            TaggableMedia.CONTEXT_PEOPLE_NEARBY,
            TaggableMedia.CONTEXT_TIME_DAYOFWEEK,
            TaggableMedia.CONTEXT_TIME_HOUR, TaggableMedia.COMMUNITY_SHARES,
            TaggableMedia.CONTEXT_PHOTOGRAPHER };

    // Create at http://mmm.sims.berkeley.edu/export/tag.jsp
    private static final String XML_GRAPH = "etc/test3.xml";

    /**
     * Prefuse based display using photo nearness
     */
    public AMPLayout() {
        super("AMP");
        try {
            this.init();
        } catch (IOException e) {
            e.printStackTrace();
        }
        System.out.println("Finished init.");
    }

    private void init() throws IOException {

        // Load the XML file
        XMLGraphReader gr = new XMLGraphReader();
        gr.setNodeType(DefaultNode.class);
        this.graph = gr.loadGraph(XML_GRAPH);
        
        // Construct lists and registry
        this.registry = new ItemRegistry(this.graph);
        this.filter = new ActionList(this.registry);
        this.filter.add(new GraphFilter());
        
        Display display = setPrefuseDefaults();

        // UI Setup
        getContentPane().setLayout(
                new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
        Panel sliders = new Panel();
        sliders.setLayout(new GridLayout(0, 3));
        
        // For each slidable type
        List sliderList = new LinkedList();
        for (int i = 0; i < MEDIA_TYPES.length; i++) {
            final MediaSlider sf = new MediaSlider(MEDIA_TYPES[i], graph, this);
            sliders.add(sf.getPanel());
            sliderList.add(sf);
        }
        
        this.filter.add(new EdgeFilterAction(sliderList));

        getContentPane().add(sliders);
        getContentPane().add(display);
        pack();
        setVisible(true);
        
        this.filter.runNow(); // filter Re-run by changing sliders
        this.forces.runNow(); // forces re-runs every 20ms, indefinitely

    }

    /**
     * @param argv
     */
    public static void main(String[] argv) {
        new AMPLayout();
    }
    
    /**
     * We don't care which slider triggered it, we just want to launch a refresh. 
     * @param e
     * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
     */
    public void stateChanged(ChangeEvent e) {
        if (!((JSlider) e.getSource()).getValueIsAdjusting()) {
            this.filter.runNow();
        }
    }

}