/*
 * Created on Nov 7, 2005
 *
 */
package gcr.mmm.ui;

import javax.swing.JFrame;

import edu.berkeley.guir.prefuse.Display;
import edu.berkeley.guir.prefuse.ItemRegistry;
import edu.berkeley.guir.prefuse.action.RepaintAction;
import edu.berkeley.guir.prefuse.activity.ActionList;
import edu.berkeley.guir.prefuse.graph.Graph;
import edu.berkeley.guir.prefuse.render.DefaultEdgeRenderer;
import edu.berkeley.guir.prefuse.render.DefaultRendererFactory;
import edu.berkeley.guir.prefuse.render.ShapeRenderer;
import edu.berkeley.guir.prefuse.render.TextImageItemRenderer;
import edu.berkeley.guir.prefuse.util.StringAbbreviator;
import edu.berkeley.guir.prefusex.controls.DragControl;
import edu.berkeley.guir.prefusex.force.DragForce;
import edu.berkeley.guir.prefusex.force.ForceSimulator;
import edu.berkeley.guir.prefusex.force.NBodyForce;
import edu.berkeley.guir.prefusex.force.SpringForce;
import edu.berkeley.guir.prefusex.force.WallForce;
import edu.berkeley.guir.prefusex.layout.ForceDirectedLayout;

public class DefaultLayout extends JFrame {

    private static final long serialVersionUID = 4190324746536725147L;

    ItemRegistry registry;

    Graph graph;

    ActionList filter, forces;
    
    private final static int APP_WIDTH = 800;

    private final static int APP_HEIGHT = 600;
    
    public DefaultLayout(String title) {
        super(title);
    }

    /**
     * From the Prefuse FDL demo
     */
    Display setPrefuseDefaults() {
        
        // set up a renderer such that nodes show text labels
        TextImageItemRenderer nodeRenderer = new TextImageItemRenderer();
        nodeRenderer.setRenderType(ShapeRenderer.RENDER_TYPE_NONE);
        //nodeRenderer.setMaxTextWidth(1);
        nodeRenderer.setAbbrevType(StringAbbreviator.TRUNCATE);
        
        DefaultEdgeRenderer edgeRenderer = new DefaultEdgeRenderer();
        edgeRenderer.setRenderType(ShapeRenderer.RENDER_TYPE_NONE);
        
        // create a new renderer factory
        this.registry.setRendererFactory(new DefaultRendererFactory(
                nodeRenderer, edgeRenderer, null));

        // create a new display component to show the data
        Display display = new Display(this.registry);
        display.setSize(APP_WIDTH, APP_HEIGHT);

        // let users drag nodes around on screen
        boolean repaint = false;
        boolean fixOnMouseOver = true;
        display.addControlListener(new DragControl(repaint, fixOnMouseOver));
        display.addControlListener(new HoverZoomControl());
        
        // run this action list for an infinite duration (duration value
        // equals -1), re-executing the action list every 20 milliseconds
        this.forces = new ActionList(this.registry, -1, 20);

        //ForceDirectedLayout fdl = new ForceDirectedLayout(new MyForceSimulator(), true);
        this.forces.add(new DynamicForceDirectedLayout(true));
        

        this.forces.add(new RepaintAction());

        // set up this JFrame
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        return display;
    }
    
    static final class MyForceSimulator extends ForceSimulator {
        MyForceSimulator() {
            this.addForce(new NBodyForce());
            this.addForce(new SpringForce());
            this.addForce(new DragForce());
            // TODO Why is it larger than this?
            
            this.addForce(new WallForce(-30f, -1,-1,APP_WIDTH+1, -1));
            this.addForce(new WallForce(-30f, -1,-1,-1,APP_HEIGHT+1));
            this.addForce(new WallForce(-30f, 0,APP_HEIGHT+1,APP_WIDTH+1, APP_HEIGHT+1));
            this.addForce(new WallForce(-30f, APP_WIDTH+1,-1,APP_WIDTH+1, APP_HEIGHT+1));
            
        }
    }

}
