/*
 * Created on Nov 16, 2005
 *
 */
package gcr.mmm.ui;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

import edu.berkeley.guir.prefuse.EdgeItem;
import edu.berkeley.guir.prefuse.ItemRegistry;
import edu.berkeley.guir.prefuse.action.AbstractAction;
import edu.berkeley.guir.prefuse.graph.Node;

public class EdgeFilterAction extends AbstractAction {
    private MediaSlider[] mediaSliders;

    private final HashMap community = new HashMap();
    
    public EdgeFilterAction(List sliders) {
        this.mediaSliders = (MediaSlider[])sliders.toArray(new MediaSlider[0]);
    }

    public void run(ItemRegistry reg, double frac) {
        System.out.println("Starting EdgeFilterAction");
        community.clear();
        
        // Cache all current slider values.
        System.out.println("... caching");
        final int[] currentValues = new int[this.mediaSliders.length];
        for (int i = 0; i < this.mediaSliders.length; i++) {
            currentValues[i] = this.mediaSliders[i].getValue();
        }
        
        // Get all edges, even the hidden ones.
        System.out.println("... iterating");
        final Iterator edges = reg.getItems(ItemRegistry.DEFAULT_EDGE_CLASS, false);
        
        Node n1, n2;
        int currentGroup = 0;
        
        
        while (edges.hasNext()) {
            final EdgeItem edge = (EdgeItem) edges.next();
            double sum = 0;
            for (int i = 0; i < this.mediaSliders.length; i++) {
                if (currentValues[i] == 0)
                    continue;

                int type = this.mediaSliders[i].getType();
                int edgeTypeValue = Integer.parseInt(edge.getAttribute(""
                        + type));
                // sum += (edgeTypeValue/this.mediaSliders[i].getMaxValue()) *
                // currentValues[i];
                sum += edgeTypeValue * currentValues[i];
            }

            if (sum > 50) {
                edge.setVisible(true);
                edge.setAttribute(DynamicForceDirectedLayout.ATTR_SPRING, ""+(sum/100));
                setCommunity(currentGroup, edge);
                
                //System.out.println("+");
            } else {
                edge.setVisible(false);
            }
        }
        System.out.println("\nFinished EdgeFilterAction");
    }

    /**
     * @param currentGroup
     * @param edge
     */
    private void setCommunity(int currentGroup, final EdgeItem edge) {
        Node n1;
        Node n2;
        n1 = edge.getFirstNode();
        n2 = edge.getSecondNode();

        boolean contains1 = this.community.containsKey(n1);
        boolean contains2 = this.community.containsKey(n2);

        if(!contains1 && !contains2) {
            // Create a new community
            String newcom = ""+currentGroup++;
            this.community.put(n1, newcom);
            n1.setAttribute("label", newcom);
            this.community.put(n2, newcom);
            n2.setAttribute("label", newcom);
        } else if(contains1 && !contains2) {
            // Expand a community
            this.community.put(n2, this.community.get(n1));
            n2.setAttribute("label", (String)this.community.get(n1));
        } else if(!contains1 && contains2) {
            // Expand a community
            this.community.put(n1, this.community.get(n2));
            n1.setAttribute("label", (String)this.community.get(n2));
        } else if(contains1 && contains2) {
            String n1c = (String)this.community.get(n1);
            String n2c = (String)this.community.get(n2);
            if(!n1c.equals(n2c)) {
                n2c.replaceAll(n2c, n1c);
                n2.setAttribute("label", (String)this.community.get(n1));
            }
        }
    }

}
