/*
 * Created on Aug 10, 2004
 *
 */
package gcr.mmm2.model;

import gcr.mmm2.rdb.RDBConnection;
import gcr.mmm2.rdb.ResultSetWrapper;
import gcr.rdf.MMMModelFactory;
import gcr.mmm2.util.StringUtils;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;
import java.util.List;

/**
 * @author Simon King
 * 
 */
public class AlbumImpl implements IAlbum {
    private String m_caption;

    private Date m_createDate;

    private int m_id;

    private int m_ownerID;

    private String m_title;

    private int m_visibility;

    protected AlbumImpl(int id) throws ObjectNotFoundException {
        this.m_id = id;
        String query = "select title, caption, created_by, visibility, date_added from album "
                + "where id=" + this.m_id + " and date_inactive is null";
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        try {
            if (rs.next()) {
                this.m_title = StringUtils.URLDecode(rs.getString("title") );
                this.m_caption = StringUtils.URLDecode(rs.getString("caption") );
                this.m_ownerID = rs.getInt("created_by");
                this.m_visibility = rs.getInt("visibility");
                this.m_createDate = rs.getDate("date_added");
            } else {
                throw new ObjectNotFoundException("Unable to find album(id)="
                        + id);
            }
        } catch (SQLException e) {
            e.printStackTrace();
            throw new ObjectNotFoundException("Unable to find album(id)=" + id);
        } finally {
            rsw.closeAll();
        }
    }

    public void addComment(String comment, IUser u) {
        if (comment != null) {
            int id = RDBConnection.getNextVal("comment_id_seq");
            String query = "insert into media_comment (id, media_id, comment, date_added, created_by) "
                + "values (?, ?, ?, now(), ?)";
            Object[] params = {new Integer(id), new Integer(m_id), StringUtils.URLEncode(comment), new Integer(u.getID() )};
            RDBConnection.executePrepared(query, params);
        }
    }

    // TODO -- think about storing album contents in a cache -- lotsa DB hits
    // could get slow
    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IAlbum#addPhoto(gcr.mmm2.model.IPhoto, int)
     */
    public void addPhoto(IPhoto photo, int sortRank, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify album id " + m_id + " owner by "
                    + this.m_ownerID);
        }
        if (!containsPhoto(photo)) {
            int id = RDBConnection.getNextVal("album_contents_id_seq");
            String query = "insert into album_contents (id, album_id, photo_id, sort, created_by, date_added) "
                    + "values ("
                    + id
                    + ", "
                    + this.m_id
                    + ", "
                    + photo.getID()
                    + ", " + sortRank + ", " + owner.getID() + ", now())";
            RDBConnection.execute(query);
        }
    }

    public boolean containsPhoto(IPhoto photo) {
        boolean rtnVal = false;
        String query = "select photo_id from album_contents where album_id="
                + this.m_id + " and photo_id=" + photo.getID()
                + " and date_inactive is null";
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        try {
            if (rs.next()) {
                rtnVal = true;
            } else {
                rtnVal = false;
            }
        } catch (SQLException e) {
            e.printStackTrace();
            rtnVal = false;
        } finally {
            rsw.closeAll();
        }
        return rtnVal;
    }

    public void delete(IUser owner) throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify album id " + this.m_id + " owner by "
                    + this.m_ownerID);
        }

        String query = "update album set date_inactive=now() where id="
                + this.getID();
        RDBConnection.execute(query);
        query = "update share_album set date_inactive=now() where album_id="
                + this.getID();
        RDBConnection.execute(query);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IAlbum#getCaption()
     */
    public String getCaption() {
        return this.m_caption;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IAlbum#getCreateTime()
     */
    public Date getCreateTime() {
        return this.m_createDate;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IAlbum#getID()
     */
    public int getID() {
        return this.m_id;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IAlbum#getOwner()
     */
    public IUser getOwner() {
        return UserManager.getByID(this.m_ownerID);
    }

    public String getTitle() {
        return this.m_title;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IAlbum#getVisibility()
     */
    public int getVisibility() {
        return this.m_visibility;
    }

    private boolean hasPermission(IUser owner) {
        try {
            return (owner.getID() == this.m_ownerID || UserManager
                    .isAdmin(owner));
        } catch (Exception e) {
            return false;
        }
    }

    public List listComments() {
        String query = "select id, created_by, comment, date_added from media_comment where "
                + "media_id=" + this.m_id + " and date_inactive is null";
        return MediaManager.listComments(query);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IAlbum#listPhotos()
     */
    public List listPhotos() {
        return PhotoFactory.listPhotosInAlbum(this);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IAlbum#movePhoto(gcr.mmm2.model.IPhoto, int)
     */
    public void movePhoto(IPhoto photo, int newSortRank, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify album id " + this.m_id + " owner by "
                    + this.m_ownerID);
        }
        if (containsPhoto(photo)) {
            String query = "update album_contents set sort=" + newSortRank
                    + " where album_id=" + this.m_id + " and photo_id="
                    + photo.getID();
            RDBConnection.execute(query);
        }
    }

    public void removeComment(int id, IUser owner) throws PermissionException {
        if (!hasPermission(owner)) {
            // now check if owner owns the comment itself
            if (owner.getID() != MediaManager.getCommentByID(id).getOwner()
                    .getID()) {
                throw new PermissionException("User " + owner.getID()
                        + " tried to remove comment id " + id
                        + " for album id " + this.m_id + " owner by "
                        + this.m_ownerID);
            }
        }
        MediaManager.deleteComment(id);
    }

    public void removePhoto(IPhoto photo, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify album id " + this.m_id + " owner by "
                    + this.m_ownerID);
        }
        if (containsPhoto(photo)) {
            String query = "update album_contents set date_inactive=now() "
                    + "where album_id=" + this.m_id + " and photo_id="
                    + photo.getID();
            RDBConnection.execute(query);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IAlbum#setCaption(java.lang.String)
     *      @SuppressWarnings("unqualified-field-access")
     */
    public void setCaption(String caption, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify album id " + this.m_id + " owner by "
                    + this.m_ownerID);
        }
        updateValue("caption", caption);
        m_caption = caption;
    }

    public void setTitle(String title, IUser owner) throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify album id " + this.m_id + " owner by "
                    + this.m_ownerID);
        }
        updateValue("title", title);
        this.m_title = title;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IAlbum#setVisibility(int)
     */
    public void setVisibility(int visibility, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify album id " + this.m_id + " owner by "
                    + this.m_ownerID);
        }
        updateValue("visibility", visibility);
        this.m_visibility = visibility;
    }

    public String toString() {
        return getCaption();
    }

    private void updateValue(String colName, String newVal) {
        String query = "update album set " + colName + "=? where id=?";
        Object[] params = { StringUtils.URLEncode(newVal), new Integer(m_id) };
        RDBConnection.executePrepared(query, params);
    }

    private void updateValue(String colName, int newVal) {
        String query = "update album set " + colName + "=? where id=?";
        Object[] params = { new Integer(newVal), new Integer(m_id) };
        RDBConnection.executePrepared(query, params);
    }

    public String getURI() {
        return MMMModelFactory.BASE_URI + "album/" + this.m_id;
    }

}
