/*
 * Created on Oct 20, 2004
 */
package gcr.mmm2.model;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * Caches recently accessed information
 * 
 * @author Jeff Towle,
 */
public class CacheManager {

    static private final Map userIDMap = Collections
            .synchronizedMap(new HashMap());

    static private final Map userNameMap = Collections
            .synchronizedMap(new HashMap());

    static private final Map photoMap = Collections
            .synchronizedMap(new HashMap());

    static private final Map contactMap = Collections
            .synchronizedMap(new HashMap());

    static private final Map personMap = Collections
            .synchronizedMap(new HashMap());

    static private final Map albumMap = Collections
            .synchronizedMap(new HashMap());

    static private final Map blogMap = Collections
            .synchronizedMap(new HashMap());

    public CacheManager() {
        // do nothing
    }

    /**
     * @param id
     * @return
     */
    static protected IUser getUserByID(int id) throws ObjectNotFoundException{
        if (!userIDMap.containsKey(new Integer(id))) {
                IUser user = new UserImpl(id);
                cacheUser(user);
                return user;           
        }
        return (IUser) userIDMap.get(new Integer(id));
    }

    /**
     * @param username
     * @return
     */
    static protected IUser getUserByName(String username) {
        if (!userNameMap.containsKey(username)) {
            try {
                IUser user = new UserImpl(username);
                cacheUser(user);
                return user;
            } catch (ObjectNotFoundException e) {
                e.printStackTrace();
                return null;
            }
        }
        return (IUser) userNameMap.get(username);
    }

    private static void cacheUser(IUser user) {
        userIDMap.put(new Integer(user.getID()), user);
        userNameMap.put(user.getUsername(), user);
    }

    /**
     * @param id
     * @return
     */
    static protected IPhoto getPhoto(int id) {
        Integer i = new Integer(id);
        if (!photoMap.containsKey(i)) {
            try {
                IPhoto photo = new PhotoImpl(id);
                photoMap.put(i, photo);
                return photo;
            } catch (ObjectNotFoundException e) {
                e.printStackTrace();
                return null;
            }
        }
        return (IPhoto) photoMap.get(i);
    }

    /**
     * @param id
     * @return
     */
    static protected IContact getContact(int id) {
        Integer i = new Integer(id);
        if (!contactMap.containsKey(i)) {
            try {
                IContact contact = new ContactImpl(id);
                contactMap.put(i, contact);
                return contact;
            } catch (ObjectNotFoundException e) {
                e.printStackTrace();
                return null;
            }
        }
        return (IContact) contactMap.get(i);
    }

    /**
     * @param id
     * @return
     */
    static protected IPerson getPerson(int id) {
        Integer i = new Integer(id);
        if (!personMap.containsKey(i)) {
            try {
                IPerson person = new PersonImpl(id);
                personMap.put(i, person);
                return person;
            } catch (ObjectNotFoundException e) {
                e.printStackTrace();
                return null;
            }
        }
        return (IPerson) personMap.get(i);
    }

    /**
     * @param id
     * @return
     */
    static protected IAlbum getAlbum(int id) {
        Integer i = new Integer(id);
        if (!albumMap.containsKey(i)) {
            try {
                IAlbum album = new AlbumImpl(id);
                albumMap.put(i, album);
                return album;
            } catch (ObjectNotFoundException e) {
                e.printStackTrace();
                return null;
            }
        }
        return (IAlbum) albumMap.get(i);
    }

    /**
     * @param id
     * @return
     */
    static protected Blog getBlog(int id) {
        Integer i = new Integer(id);
        if (!blogMap.containsKey(i)) {
            try {
                Blog blog = new Blog(id);
                blogMap.put(i, blog);
                return blog;
            } catch (ObjectNotFoundException e) {
                e.printStackTrace();
                return null;
            }
        }
        return (Blog) blogMap.get(i);
    }

    /**
     * 
     */
    static public synchronized void flushCache() {
        userIDMap.clear();
        userNameMap.clear();
        photoMap.clear();
        contactMap.clear();
        personMap.clear();
    }

}