/*
 * Created on Aug 11, 2004
 *
 */
package gcr.mmm2.model;

import gcr.mmm2.rdb.RDBConnection;
import gcr.mmm2.rdb.ResultSetWrapper;
import gcr.mmm2.util.StringUtils;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author Simon King
 * 
 */
public class ContactListImpl implements IContactList {
    private int m_id;

    private int m_ownerID;

    private Date m_createDate;

    private String m_name;

    protected ContactListImpl(int id) throws ObjectNotFoundException {
        m_id = id;
        String query = "select name, created_by, date_added from contact_list where id="
                + m_id;
        Object[] params = {new Integer(m_id)};
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        try {
            if (rs.next()) {
                m_ownerID = rs.getInt("created_by");
                m_name = rs.getString("name");
                m_createDate = rs.getDate("date_added");
            } else {
                throw new ObjectNotFoundException(
                        "Unable to find contactlist(id)=" + m_id);
            }
        } catch (SQLException e) {
            e.printStackTrace();
            throw new ObjectNotFoundException("Unable to find contactlist(id)="
                    + m_id);
        } finally {
            rsw.closeAll();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IContactList#getID()
     */
    public int getID() {
        return m_id;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IContactList#getOwner()
     */
    public IUser getOwner() {
        return UserManager.getByID(m_ownerID);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IContactList#listContacts()
     */
    public List listContacts() {
        // String query = "select distinct member_id from group_membership where
        // "+
        // "group_id="+m_id+" and date_inactive is null";
        String query = "select distinct c.id from contact c, group_membership g where c.person_id=g.member_id and c.created_by=g.created_by "
                + "and group_id="
                + m_id
                + " and g.date_inactive is null and c.date_inactive is null";
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        ArrayList l = new ArrayList();
        try {
            while (rs.next()) {
                int id = rs.getInt(1);
                IContact c = ContactManager.getContactByID(id);
                if (c != null) {
                    l.add(c);
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return l;
    }

    public boolean containsContact(IContact contact) {
        boolean rtnVal = false;
        // String query = "select member_id from group_membership where
        // group_id="+m_id+
        // " and member_id="+contact.getID()+" and date_inactive is null";
        String query = "select distinct c.id from contact c, group_membership g where c.person_id=g.member_id and c.created_by=g.created_by "
                + "and group_id="
                + m_id
                + " and member_id="
                + contact.getID()
                + " and g.date_inactive is null and c.date_inactive is null";

        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        try {
            if (rs.next()) {
                rtnVal = true;
            } else {
                rtnVal = false;
            }
        } catch (SQLException e) {
            e.printStackTrace();
            rtnVal = false;
        } finally {
            rsw.closeAll();
        }
        return rtnVal;
    }

    private boolean hasPermission(IUser owner) {
        try {
            return (owner.getID() == m_ownerID || UserManager.isAdmin(owner));
        } catch (Exception e) {
            return false;
        }
    }

    public String getName() {
        return m_name;
    }

    public void setName(String name, IUser owner) throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify contact_list id " + m_id + " owner by "
                    + m_ownerID);
        }
        if (name == null || name.length() < 0) {
            return;
        }
        String query = "update contact_list set name=? where id=?";
        Object[] params = { StringUtils.URLEncode(name), new Integer(m_id) };
        RDBConnection.executePrepared(query, params);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IContactList#addContact(gcr.mmm2.model.IContact)
     */
    public void addContact(IContact contact, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify contact_list id " + m_id + " owner by "
                    + m_ownerID);
        }
        int id = RDBConnection.getNextVal("group_mem_id_seq");
        if (!containsContact(contact)) {
            String query = "insert into group_membership (id, group_id, member_id, date_added, created_by) "
                    + "values (?, ?, ?, now(), ?)";
            Object[] params = {new Integer(id), new Integer(m_id), new Integer(contact.getID() ), new Integer(m_ownerID)};
            RDBConnection.executePrepared(query, params);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IContactList#removeContact(gcr.mmm2.model.IContact)
     */
    public void removeContact(IContact contact, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify contact_list id " + m_id + " owner by "
                    + m_ownerID);
        }
        if (containsContact(contact)) {
            String query = "update group_membership set date_inactive=now() where group_id=? and member_id=? and date_inactive is null";
            Object[] params = {new Integer(m_id), new Integer(contact.getID() ) };
            RDBConnection.executePrepared(query, params);
        }
    }

    public void delete(IUser owner) throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify contact_list id " + m_id + " owner by "
                    + m_ownerID);
        }
        ContactManager.deleteContactList(this);
    }

    public String toString() {
        return getName();
    }

}
