/*
 * Created on Aug 23, 2004
 *
 */
package gcr.mmm2.model;

import gcr.mmm2.rdb.RDBConnection;
import gcr.mmm2.rdb.ResultSetWrapper;
//import gcr.mmm2.rdf.Person;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.logging.Logger;

/**
 * @author simon
 * @author jeff
 * 
 */
public class ContextImpl implements IContext {
    final Logger log = Logger.getLogger("sharingActivity");
    
    private int m_id;

    private String m_network, m_lac, m_cellID;

    private String m_btMac, m_btName;

    private float m_latitude, m_longitude;

    private String m_location_label;

    private Date m_sampleDate;

    private int m_creatorID;

    protected ContextImpl(int id) throws ObjectNotFoundException {
        this.m_id = id;
        String query = "select network, lac, cell_id, gps_lat, gps_long, location_label, sample_time, user_id "
                + "from context where id=" + this.m_id;
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        try {
            if (rs.next()) {
                this.m_network = rs.getString("network");
                this.m_lac = rs.getString("lac");
                this.m_cellID = rs.getString("cell_id");
                this.m_latitude = rs.getFloat("gps_lat");
                this.m_longitude = rs.getFloat("gps_long");
                this.m_location_label = rs.getString("location_label");
                if (this.m_location_label == null)
                    this.m_location_label = "";
                this.m_creatorID = rs.getInt("user_id");
                this.m_sampleDate = rs.getDate("sample_time");
            } else {
                throw new ObjectNotFoundException("Unable to find context(id)="
                        + id);
            }
        } catch (SQLException e) {
            e.printStackTrace();
            throw new ObjectNotFoundException("Unable to find context(id)="
                    + id);
        } finally {
            rsw.closeAll();
        }

    }

    public int getID() {
        return this.m_id;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IContext#setCellLocation(java.lang.String,
     *      java.lang.String, java.lang.String)
     */
    public void setCellLocation(String cellId, String lac, String network) {
        // TODO deal with nulls
        String query = "update context set cell_id='" + cellId + "', lac='"
                + lac + "', network='" + network + "' where id=" + m_id;
        RDBConnection.execute(query);
        this.m_cellID = cellId;
        this.m_lac = lac;
        this.m_network = network;
    }

    public String getNetwork() {
        return this.m_network;
    }

    public String getLac() {
        return this.m_lac;
    }

    public String getCellId() {
        return this.m_cellID;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IContext
     */
    public void setGpsLocation(String gps_lat, String gps_long,
            String gps_speed, String gps_direction) {
        // TODO deal with nulls
        String query = "update context set gps_lat=" + gps_lat + ", gps_long="
                + gps_long + ", gps_speed=" + gps_speed + ", gps_direction="
                + gps_direction + " where id=" + m_id;
        RDBConnection.execute(query);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IContext#addBluetoothDevice(java.lang.String,
     *      java.lang.String)
     */
    public void addBluetoothDevice(String btMac, String btName) {
        // TODO deal with nulls
        int id = RDBConnection.getNextVal("ctxt_bt_id_seq");
        String query = "insert into context_bluetooth (id, context_id, bt_mac, bt_alias) "
                + "values ("
                + id
                + ", "
                + m_id
                + ", '"
                + btMac
                + "', '"
                + btName + "')";
        this.log.info("Adding Bluetooth Device: "+ query);
        
        RDBConnection.execute(query);
        m_btMac = btMac;
        m_btName = btName;
    }

    /**
     * @param locName
     * @see gcr.mmm2.model.IContext#setLocationLabel(java.lang.String)
     */
    public void setLocationLabel(String locName) {
        String query = "update context set location_label='" + locName
                + "' where id=" + this.m_id;

        if (locName == null || "null".equals(locName))
            query = "update context set location_label=null where id="
                    + this.m_id;

        RDBConnection.execute(query);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IContext#getGpsLocationInfo()
     */
    public List getGpsLocationInfo() {
        String query = "select gps_lat, gps_long " + "from context where id="
                + m_id;

        String gps_lat = null;
        String gps_long = null;
        ArrayList l = new ArrayList();
        ResultSetWrapper rsw = null;

        try {
            rsw = RDBConnection.executeQuery(query);
            ResultSet rs = rsw.getResultSet();

            if (rs.next()) {
                gps_lat = String.valueOf(rs.getFloat("gps_lat"));
                gps_long = String.valueOf(rs.getFloat("gps_long"));
                l.add(gps_lat);
                l.add(gps_long);
            }

            rsw.closeAll();

            query = "select city, state, zip from (select city, state, zip, "
                    + "point_distance(coordinates, '" + gps_long + ", "
                    + gps_lat + "') as dist " + "from gps2zip "
                    + "order by dist " + "limit 1) as tbl;";

            rsw = RDBConnection.executeQuery(query);
            rs = rsw.getResultSet();

            String city = null;
            String state = null;
            String zip = null;

            if (rs.next()) {
                city = rs.getString("city");
                state = rs.getString("state");
                zip = String.valueOf(rs.getInt("zip"));
                l.add(city);
                l.add(state);
                l.add(zip);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }

        return l;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IContext#getGpsLatitude()
     */
    public String getGpsLatitude() {
        String query = "select gps_lat " + "from context where id=" + m_id;
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        String latitude = "";
        try {
            if (rs.next()) {
                latitude = String.valueOf(rs.getFloat("gps_lat"));
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return latitude;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IContext#getGpsLongitude()
     */
    public String getGpsLongitude() {
        String query = "select gps_long " + "from context where id=" + m_id;
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        String longitude = "";
        try {
            if (rs.next()) {
                longitude = String.valueOf(rs.getFloat("gps_long"));
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return longitude;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IContext#getLocationLabel()
     */
    public String getLocationLabel() {
        // TODO: how to handle multiple location labels for same
        // network/lac/cellid
        return m_location_label;
    }

    // return list of String
    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IContext#listLocationLabelGuesses(int)
     */
    public List listLocationLabelGuesses(int numGuesses) {
        // TODO do it
        return null;
    }

    /**
     * @return list of IPerson
     * @see gcr.mmm2.model.IContext#listNearbyBluetoothPeople()
     */
    public List listNearbyBluetoothPeople() {
        // TODO this
        String query = "select distinct pb.person_id from person_bluetooth pb, context_bluetooth cb "
                + "where pb.bt_mac=cb.bt_mac and cb.context_id=" + this.m_id;
        return PersonManager.listUsersOrPersons(query);
    }
    
    public List listNearbyBluetoothPeopleWithMac() {
    	 // TODO this
        String query = "select distinct pb.person_id, cb.bt_mac from person_bluetooth pb, context_bluetooth cb "
                + "where pb.bt_mac=cb.bt_mac and cb.context_id=" + this.m_id;
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        ArrayList l = new ArrayList();
        try {
            while (rs.next()) {
            	   IPerson person = PersonManager.getByID(rs.getInt("person_id"));
                String[] bt = {person.toString(),
                        rs.getString("bt_mac") };
                l.add(bt);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return l;
    
    }

    /**
     * @return list of String[2]
     * @see gcr.mmm2.model.IContext#listUnknownBluetoothDevices()
     */
    public List listUnknownBluetoothDevices() {
        String query = "select bt_mac, bt_alias from context_bluetooth where context_id="
                + this.m_id
                + "and bt_mac not in (select bt_mac from person_bluetooth where date_inactive is null)";
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        ArrayList l = new ArrayList();
        try {
            while (rs.next()) {
                String[] bt = { rs.getString("bt_mac"),
                        rs.getString("bt_alias") };
                l.add(bt);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return l;
    }

}
