/*
 * Created on Aug 10, 2004
 *
 */
package gcr.mmm2.model;

import gcr.mmm2.rdb.RDBConnection;
import gcr.mmm2.rdb.ResultSetWrapper;
import gcr.mmm2.server.servlets.JpegRotator;
import gcr.mmm2.util.MMM2Properties;
import gcr.mmm2.util.StringUtils;
import gcr.rdf.MMMModelFactory;

import java.io.File;
import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Catch-all for utility fucnctions. Should eventually be broken down and
 * incorporated into PhotoImpl, etc.
 * 
 * @author Simon King, many others.
 */
public class MediaManager {
    public static final int ADD = 1;

    public static final int ALL = 8;

    // Ownership Filter
    public static final int ALL_VISIBLE = 0;

    public static final int DELETE = 7;

    private static Pattern filenamePattern = Pattern
            .compile("^c?((\\d+)_)*((\\d+)[W|M].+\\.(jpg|amr|3gp)$)");

    public static final int IMAGE = 0;

    public static final int LAST_MONTH = 5;

    public static final int MOVIE = 1;

    // Time Periods
    public static final int NEWEST = -1;

    // Media Types
    public static final int NON_MEDIA = -1;

    public static final String ORIG_IMAGE_DIR = MMM2Properties
            .getProperty("ORIG_IMAGE_DIR");

    public static final int OWNED_PHOTOS = 1;

    public static final int PREV_14 = 3;

    public static final int PREV_7 = 2;

    public static final int PUBLIC = 11;

    public static final int PUBLIC_PHOTOS = 3;

    public static final int RECEIVED_PHOTOS = 2;

    public static final int RESTRICT = 2;

    public static final int ROTATE_180 = 6;

    public static final int ROTATE_LEFT_90 = 4;

    public static final int ROTATE_RIGHT_90 = 5;

    public static final SimpleDateFormat sdf = new SimpleDateFormat(
            "yyyyMMdd'T'HHmmss");

    // Command Menu Actions
    public static final int SHARE = 0;

    public static final int SHOEBOX = 7;

    // Sort Order
    public static final int SORT_DATE_ASC = 1;

    public static final int SORT_DATE_DESC = 2;

    public static final int SOUND = 2;

    public static final int THIS_MONTH = 4;

    public static final int THIS_YEAR = 6;

    public static final String THUMB_IMAGE_DIR = MMM2Properties
            .getProperty("THUMB_IMAGE_DIR");

    public static final int THUMB_SIZE_LARGE = 10;

    public static final int THUMB_SIZE_MEDIUM = 9;

    public static final int THUMB_SIZE_SMALL = 8;

    public static final int TODAY = 1;

    public static final int UNRESTRICT = 3;

    public static final int UNRESTRICTED_OWNED_PHOTOS = 3;

    public static void createRDFAlbumEntry(int id) {
        String newURI = MMMModelFactory.BASE_URI + "album/" + id;
        // ResourceManipulator.createIndividual(newURI,
        // ResourceManipulator.PERSON_ID_CLASS_URI);
    }

    public static void createRDFPhotoEntry(int id) {
        String newURI = MMMModelFactory.BASE_URI + "photo/" + id;
        // ResourceManipulator.createIndividual(newURI,
        // ResourceManipulator.PERSON_ID_CLASS_URI);
    }

    protected static void deleteComment(int id) {
        String query = "update media_comment set date_inactive=now() where id="
                + id;
        RDBConnection.execute(query);
    }

    public static Comment getCommentByID(int id) {
        String query = "select comment, date_added, created_by from media_comment where id="
                + id;
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        Comment c = null;
        try {
            if (rs.next()) {
                String comment = rs.getString("comment");
                Date d = rs.getDate("date_added");
                IUser u = UserManager.getByID(rs.getInt("created_by"));
                c = new Comment(id, comment, d, u);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return c;
    }

    public static String getFormattedNow() {
        Date now = new Date();
        return sdf.format(now);
    }

    public static String getImagePlaceholder(int width, int height, boolean crop)
            throws IOException {
        return getImagePlaceholder(width, height, MMM2Properties
                .getProperty("PLACEHOLDER_FILE"), crop);
    }

    public static String getImagePlaceholder(int width, int height,
            String placeholderFileName, boolean crop) {
        // if width==0, return original
        
        String cropChar = "";
        if(crop)
            cropChar="c";

        String dir = MMM2Properties.getProperty("PLACEHOLDER_DIR");
        String fnameOrig = dir + "/" + placeholderFileName;

        File f = null;

        if (width > 0 && height > 0) { // looking for a thumbnail/scaled
            // version
            String fname = dir + "/" + cropChar + width + "_" + height + "_"
                    + placeholderFileName;
            f = new File(fname);
            if (f.exists()) {// thumbnail exists
                return fname;
            }

            // create a new scaled image file
            File fOrig = new File(fnameOrig);
            if (fOrig.exists()) { // create the scaled one
                if(crop)
                    JpegRotator.doCropResize(fnameOrig,fname,width,height);
                else
                    JpegRotator.doResize(fnameOrig, fname, width, height);
                return fname;

            } // couldn't find the original
            return null;

        }
        // looking for original
        String fname = fnameOrig;
        f = new File(fname);
        if (f.exists()) {
            return fname;
        }

        return null;

    }

    public static String getPhotoURL(IPhoto photo, String base, String code,
            int size) {
        if (base == null) {
            base = ".";
        }
        if (code == null) {
            code = "mmmpicture";
        }
        String sizeString = "";
        if (size > 0) {
            sizeString = "" + size + "_";
        }
        return base + "/images/" + code + "/" + sizeString
                + photo.getJPEGFilename();
    }

    public static String getPhotoURL(IPhoto photo, String base, String code,
            int width, int height) {
        if (base == null) {
            base = ".";
        }
        if (code == null) {
            code = "mmmpicture";
        }
        String sizeString = "";
        if (width > 0 && height > 0) {
            sizeString = "" + width + "_" + height + "_";
        }
        return base + "/images/" + code + "/" + sizeString
                + photo.getJPEGFilename();
    }
    
    public static String getPhotoURL(IPhoto photo, String base, String code,
            int width, int height, boolean crop) {
        String cropChar = "";
        if(crop)
            cropChar = "c";
        if (base == null) {
            base = ".";
        }
        if (code == null) {
            code = "mmmpicture";
        }
        String sizeString = "";
        if (width > 0 && height > 0) {
            sizeString = cropChar + width + "_" + height + "_";
        }
        return base + "/images/" + code + "/" + sizeString
                + photo.getJPEGFilename();
    }

    public static final String getPrivacyString(int level) {
        switch (level) {
        case IPhoto.UNRESTRICTED_PRIVACY:
            return "unrestricted";
        case IPhoto.RESTRICTED_PRIVACY:
            return "restricted";
        case IPhoto.PUBLIC_PRIVACY:
            return "public";
        default:
            return "unknown";
        }
    }

    /**
     * Gets the string for the filename that is wanted. May be a filename with a
     * nnn_ on the first. If so it will generate a thumbnail that will fit in a
     * square box of this size. May also be nnn_nnn_ at the first which would
     * generate a thumbnail that is going to fit in a rectangular box of the
     * specified width and height.
     * If has c (for crop) nnn_nnn such as cnnn_ or cnnn_ it will crop the photo
     * and fill up the specified space instead of sizing inside the box
     * 
     * @param filename
     * @return
     * @throws StringFormatException
     * @throws IOException
     */
    public static String getRealPath(String filename)
            throws StringFormatException, IOException {
        // strip off leading c for cropping
        String cFN = filename; // filename with any c removed from first 
        boolean crop = false;
        if(filename.length()>0&&filename.charAt(0)=='c'){
            cFN = filename.substring(1);
            crop = true;
        }
//      this matcher matches the filename format
        Matcher m = filenamePattern.matcher(cFN);
        int widthSize = -1;
        int heightSize = -1;
        int userid = -1;
        String rtn = "no match";
        String fnameOrig = "";
        if (m.matches()) {
            try {
                // set height and width the same for now
                heightSize = Integer.parseInt(m.group(2));
                widthSize = heightSize;
                // try getting other dimension if it exists before this match
                if (m.start(2) > 0 && cFN.indexOf("_") < m.start(2)
                        && cFN.indexOf("_") > 0) {
                    widthSize = Integer.parseInt(cFN.substring(0, cFN
                            .indexOf("_")));
                }
            } catch (NumberFormatException ignored) {
            }
            // get filename and user id
            fnameOrig = m.group(3);
            userid = Integer.parseInt(m.group(4));
            rtn = "MM - widthSize=" + widthSize + ", heightSize=" + heightSize
                    + " userid=" + userid + ", original=" + fnameOrig;
            System.out.println(rtn);
        }

        // if couldn't get userID or original filename then blow up
        if ((userid < 0) || (fnameOrig == null)) {
            throw new StringFormatException(filename
                    + " is not a valid filename format:" + userid + ","
                    + fnameOrig);
        }

        String dir = null;
        File f = null;
        String fname = null;

        if (heightSize > 0) { // we're looking for a thumbnail/scaled version
            dir = THUMB_IMAGE_DIR + "/" + userid;
            fname = dir + "/" + filename;
            f = new File(fname);
            // if the file already exists then just return it
            if (f.exists()) {
                return fname;
            }
            // otherwise we're creating a new scaled image file
            // check to make sure the directory is there
            File thumbDir = new File(dir);
            if (thumbDir.exists()) {
                if (!thumbDir.isDirectory()) {
                    // a file exists instead of a directory here--bad
                    throw new IOException("Fatal error in getRealPath: : '" + dir
                            + "' is a file, not a directory.");
                }
            } else {
                if (!thumbDir.mkdirs()) {
                    throw new IOException("Unable to create directory '" + dir
                            + "'");
                }
            }
            // get user's specific image folder
            dir = ORIG_IMAGE_DIR + "/" + userid;
            // get the full non-thumbnailed image
            fnameOrig = dir + "/" + fnameOrig;
            File fOrig = new File(fnameOrig);

            if (fOrig.exists() && fOrig.length() > 2) { // create the scaled one

                // Check for audio, video -bhill
                if (fnameOrig.endsWith("amr"))
                    return getImagePlaceholder(widthSize, heightSize,
                            MMM2Properties.getProperty("PLACEHOLDER_AUDIO"),crop);
                if (fnameOrig.endsWith("3gp"))
                    return getImagePlaceholder(widthSize, heightSize,
                            MMM2Properties.getProperty("PLACEHOLDER_VIDEO"),crop);
                if(crop)
                    JpegRotator.doCropResize(fnameOrig,fname,widthSize, heightSize);
                else
                    JpegRotator.doResize(fnameOrig, fname, widthSize, heightSize);

                return fname;
            }
            // couldn't find the original
            if (widthSize == heightSize) {
                // send a "hold your horses" for a square box
                return getImagePlaceholder(widthSize, widthSize,crop);
            }
            // send a "hold your horses" for rectangle
            return getImagePlaceholder(widthSize, heightSize,crop);
        }
        // looking for original
        dir = ORIG_IMAGE_DIR + "/" + userid;
        fname = dir + "/" + filename;
        System.out.println("\t" + fname);
        f = new File(fname);
        if (f.exists()) {
            return fname;
        }
        // no image exists that they wanted (full size)
        return getImagePlaceholder(0, 0, crop);
    } // end get real path

    static String getSQLDate(Date theDate) {

        /*
         * Initialize a Calendar, set to date (today)
         */
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(theDate);

        /*
         * Set the Date String
         */
        String the_date = "'" + calendar.get(Calendar.YEAR) + "-"
                + (calendar.get(Calendar.MONTH) + 1) + "-"
                + calendar.get(Calendar.DAY_OF_MONTH) + "'";
        return the_date;
    }

    static String getSQLEndDate(int timeID) {

        // NOTE: None of this is used because all timeIDs presently end at now()
        // This is for future use

        /*
         * Initialize a Calendar, set to current date (today)
         */
        Calendar calendar = new GregorianCalendar();
        java.util.Date date = new java.util.Date();
        calendar.setTime(date);

        /*
         * Calculate the Start Date based on Time Range
         */
        switch (timeID) {
        case TODAY:
            break;
        case PREV_7:
            calendar.add(Calendar.DATE, -7);
            break;
        case PREV_14:
            calendar.add(Calendar.DATE, -14);
            break;
        case THIS_MONTH:
            calendar.set(Calendar.DATE, 1);
            break;
        case THIS_YEAR:
            calendar.set(Calendar.MONTH, 0);
            calendar.set(Calendar.DATE, 1);
            break;
        default:
            calendar.set(Calendar.MONTH, 0);
            calendar.set(Calendar.DATE, 1);
            calendar.set(Calendar.YEAR, 1970);
            break;
        }

        /*
         * Set the Date Range Strings
         */
        String end_date = "now()";

        return end_date;
    }

    static String getSQLStartDate(int timeID) {

        /*
         * Initialize a Calendar, set to current date (today)
         */
        Calendar calendar = new GregorianCalendar();
        java.util.Date date = new java.util.Date();
        calendar.setTime(date);

        /*
         * Calculate the Start Date based on Time Range
         */
        switch (timeID) {
        case TODAY:
            break;
        case PREV_7:
            calendar.add(Calendar.DATE, -7);
            break;
        case PREV_14:
            calendar.add(Calendar.DATE, -14);
            break;
        case THIS_MONTH:
            calendar.set(Calendar.DATE, 1);
            break;
        case THIS_YEAR:
            calendar.set(Calendar.MONTH, 0);
            calendar.set(Calendar.DATE, 1);
            break;
        default:
            calendar.set(Calendar.MONTH, 0);
            calendar.set(Calendar.DATE, 1);
            calendar.set(Calendar.YEAR, 1970);
            break;
        }

        /*
         * Set the Date Range Strings
         */
        // String end_date = "now()";
        String start_date = "'" + calendar.get(Calendar.YEAR) + "-"
                + (calendar.get(Calendar.MONTH) + 1) + "-"
                + calendar.get(Calendar.DAY_OF_MONTH) + "'";
        return start_date;
    }

    // returns a list of Comment
    protected static List listComments(String query) {
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        ArrayList l = new ArrayList();
        try {
            while (rs.next()) {
                int id = rs.getInt("id");
                String comment = StringUtils.URLDecode(rs.getString("comment") );
                Date d = rs.getDate("date_added");
                IUser u = UserManager.getByID(rs.getInt("created_by"));
                Comment c = new Comment(id, comment, d, u);
                if (c != null) {
                    l.add(c);
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return l;
    }

    // return n newest photos created by owner
    public static List listNewestOwnedPhotos(int n, IUser owner) {
        String query = "select id, case capture_date is null when true then date_added else capture_date end "
                + "as date_taken from photo where date_inactive is null "
                + "and created_by="
                + owner.getID()
                + " order by date_taken desc limit " + n;
        return PhotoFactory.listPhotos(query);
    }

    public static List listOwnedPhotosInCellLocation(String network,
            String lac, String cellID, IUser owner, int sortOrder) {
        String query = "select p.id, case p.capture_date is null when true then p.date_added else p.capture_date end as the_date "
                + "from context c, photo p where c.id=p.context_id and p.date_inactive is null and p.created_by="
                + owner.getID();
        if (network != null)
            query += " and c.network='" + network + "'";
        if (lac != null)
            query += " and c.lac='" + lac + "'";
        if (cellID != null)
            query += " and c.cell_id='" + cellID + "'";
        if (sortOrder == SORT_DATE_ASC) {
            query = query + " order by the_date asc";
        } else if (sortOrder == SORT_DATE_DESC) {
            query = query + " order by the_date desc";
        }
        return PhotoFactory.listPhotos(query);
    }

    public static List listOwnedUnrestrictedPhotosByTime(IUser user, int timeID) {
        return listOwnedUnrestrictedPhotosByTime(user, timeID, SORT_DATE_DESC);
    }

    public static List listOwnedUnrestrictedPhotosByTime(IUser user,
            int timeID, int sortOrder) {
        /*
         * Set the Date Range Strings
         */
        String start_date = getSQLStartDate(timeID);
        String end_date = getSQLEndDate(timeID);

        String query = "select id, case capture_date is null when true then date_added else"
                + " capture_date end as the_date from photo where created_by="
                + user.getID()
                + " and (photo_privacy=0 or photo_privacy is null)"
                + " and date_inactive is null";

        /*
         * Sorting the Results (By Date)
         */
        if (sortOrder == SORT_DATE_DESC) {
            query = "select id, max(the_date) as the_date from (" + query
                    + ") q group by id having max(the_date) between "
                    + start_date + " and " + end_date
                    + " order by the_date desc";
        } else if (sortOrder == SORT_DATE_ASC) {
            query = "select id, min(the_date) as the_date from (" + query
                    + ") q group by id having min(the_date) between "
                    + start_date + " and " + end_date
                    + " order by the_date asc";
        } else {
            query = "select distinct id from (" + query
                    + ") q where the_date between " + start_date + " and "
                    + end_date;
        }

        return PhotoFactory.listPhotos(query);
    }

    // recipient is share recip, photo is contained in returned albums
    // returned albums either shared with or owned by recipient
    // TODO -- this doesn't yet consider visibility (public albums)
    public static List listSharedAlbums(IUser recipient, IPhoto photo) {
        return listSharedAlbums(recipient, photo, SORT_DATE_DESC);
    }

    public static List listSharedAlbums(IUser recipient, IPhoto photo,
            int sortOrder) {
        String query = "as the_date from share_recipient sr, album a, share_album sa, album_contents ac "
                + "where sr.share_id=sa.share_id and ac.album_id=sa.album_id and a.id=ac.album_id "
                + "and ac.date_inactive is null and a.date_inactive is null and sr.date_inactive is null "
                + "and sa.date_inactive is null and (sr.recipient="
                + recipient.getID()
                + " or a.created_by="
                + recipient.getID()
                + ") "
                + "and ac.photo_id="
                + photo.getID()
                + " group by the_id";
        if (sortOrder == SORT_DATE_DESC) {
            query = "select a.id as the_id, max(ac.date_added) " + query
                    + " group by the_id order by the_date desc";
        } else if (sortOrder == SORT_DATE_ASC) {
            query = "select a.id as the_id, min(ac.date_added) " + query
                    + " group by the_id order by the_date asc";
        }
        return AlbumFactory.listAlbums(query);
    }

    public static List listUngroupedPhotos(IUser user) {
        return listUngroupedPhotos(user, SORT_DATE_DESC);
    }

    // lists all owned photos not in any album
    public static List listUngroupedPhotos(IUser user, int sortOrder) {
        String query = "select id, case capture_date is null when true then date_added else capture_date end as the_date "
                + "from photo where date_inactive is null and created_by="
                + user.getID()
                + " and id not in "
                + "(select photo_id from album_contents ac, album a where a.id=ac.album_id "
                + "and a.date_inactive is null and ac.date_inactive is null and a.created_by="
                + user.getID() + ")";
        if (sortOrder == SORT_DATE_ASC) {
            query = query + " order by the_date asc";
        } else if (sortOrder == SORT_DATE_DESC) {
            query = query + " order by the_date desc";
        }
        return PhotoFactory.listPhotos(query);
    }

    public static String translateFilePath(String base, String imageFile) {
        return base + "/images/mmmpicture/" + imageFile;
    }
}
