/*
 * Created on Oct 21, 2004
 *
 */
package gcr.mmm2.model;

import gcr.rdf.AnnotationWrapper;
import gcr.rdf.PropertyDescriptor;

import java.util.Comparator;

/**
 * This class contains method that return various comparators
 * Use these for sorting collections that you may come across
 * in your trek through MMM
 * 
 * @author Jeff Towle
 *
 */
public class ObjectSorter {
	
	public static final int ALPHABETICAL = 0;
	public static final int REVERSE_ALPHA = 1;

	/**
	 * @param sortOrder
	 * @return
	 */
	public static Comparator personByFirstName(int sortOrder){
		Comparator comp = null;
		switch(sortOrder){
		
			case ObjectSorter.ALPHABETICAL:
				comp = new Comparator(){
				    public int compare(Object a, Object b){
				        return ((IPerson)a).getFirstName().
							compareToIgnoreCase( ((IPerson)b).getFirstName() );
				     }
				};
				break;
				
			case ObjectSorter.REVERSE_ALPHA:
				comp = new Comparator(){
				    public int compare(Object a, Object b){
				        return ((IPerson)b).getFirstName().
							compareToIgnoreCase( ((IPerson)a).getFirstName() );
				    }
				};
				break;
		}
		return comp;
	}
	
	/**
	 * @param sortOrder
	 * @return
	 */
	public static Comparator personByLastName(int sortOrder){
		Comparator comp = null;
		switch(sortOrder){
		
			case ObjectSorter.ALPHABETICAL:
				comp = new Comparator(){
				    public int compare(Object a, Object b){
				        return ((IPerson)a).getLastName().
							compareToIgnoreCase( ((IPerson)b).getLastName() );
				     }
				};
				break;
				
			case ObjectSorter.REVERSE_ALPHA:
				comp = new Comparator(){
				    public int compare(Object a, Object b){
				        return ((IPerson)b).getLastName().
							compareToIgnoreCase( ((IPerson)a).getLastName() );
				    }
				};
				break;
		}
		return comp;
	}
	
	public static Comparator propertyDescriptor(){
		Comparator comp=new Comparator(){
			public int compare(Object o1, Object o2) throws ClassCastException {
				PropertyDescriptor pd1=(PropertyDescriptor)o1;
				PropertyDescriptor pd2=(PropertyDescriptor)o2;
				if (pd1.getSortOrder()==pd2.getSortOrder()) {
					return 0;
				}
				if (pd1.getSortOrder()<pd2.getSortOrder()) {
					return -1;
				}
				return 1;
			}
		};
		return comp;
	}
	
	public static Comparator annotationWrapperByDate(){
		Comparator comp=new Comparator(){
			public int compare(Object o1, Object o2) throws ClassCastException {
				AnnotationWrapper aw1=(AnnotationWrapper)o1;
				AnnotationWrapper aw2=(AnnotationWrapper)o2;
				return -aw1.getTimestamp().compareTo(aw2.getTimestamp());
			}
		};
		return comp;
	}
	
	// TODO -- this not done
	public static Comparator annotationWrapperByType(){
		Comparator comp=new Comparator(){
			public int compare(Object o1, Object o2) throws ClassCastException {
				AnnotationWrapper aw1=(AnnotationWrapper)o1;
				AnnotationWrapper aw2=(AnnotationWrapper)o2;
				return aw1.getClassLabel().compareTo(aw2.getClassLabel());
			}
		};
		return comp;
	}
}
