/*
 * Created on Aug 10, 2004
 *
 */
package gcr.mmm2.model;

import gcr.mmm2.rdb.RDBConnection;
import gcr.mmm2.rdb.ResultSetWrapper;
import gcr.rdf.MMMModelFactory;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

/**
 * Methods relating to managing people
 * 
 * @author Simon King
 * 
 */
public class PersonManager {

    /**
     * @param id
     * @return
     */
    public static IPerson getByID(int id) {
        return CacheManager.getPerson(id);
    }

    public static void createRDFPersonEntry(int id) {
        String newURI = MMMModelFactory.BASE_URI + "person/" + id;
        // ResourceManipulator.createIndividual(newURI,
        // ResourceManipulator.PERSON_ID_CLASS_URI);
    }

    /**
     * @param firstName
     * @param lastName
     * @param creatingUser
     * @return
     */
    public static IPerson createPerson(String firstName, String lastName,
            IUser creatingUser) {
        int creatorID = 0;
        if (creatingUser != null) {
            creatorID = creatingUser.getID();
        }
        try {
            int id = RDBConnection.getNextVal("consumer_id_seq");
            String query = "insert into person (id, first, last, date_added, created_by) "
                    + "values ("
                    + id
                    + ", '"
                    + firstName
                    + "', '"
                    + lastName
                    + "', now(), " + creatorID + ")";
            RDBConnection.execute(query);
            createRDFPersonEntry(id);
            return new PersonImpl(id);
        } catch (ObjectNotFoundException e) {
            e.printStackTrace();
            return null;
        }
    }

    // return persons matching first and last (either can be null)
    /**
     * @param firstName
     * @param lastName
     * @return
     */
    public static List listPersons(String firstName, String lastName) {
    		int flag = 0;
    		if ((firstName == null) || (firstName == ""))
    			flag += 1;
    			
		if ((lastName == null) || (lastName==""))
    			flag +=1;
    		
    		if (flag==2)
    			return new ArrayList();
    	
        String query = "select id from person where id>=10";
        if (firstName != null){
            query = query + " and upper(first) like '"
                    + firstName.toUpperCase() + "%'";
        }
        if (lastName != null) {
            query = query + " and upper(last) like '" + lastName.toUpperCase()
                    + "%'";
        }
        return listPersons(query);
    }

    // returns a list of all contacts union all users
    public static List listAvailablePersons(IUser u) {
        String query = "select distinct id from "
                + "(select person_id as id from contact where created_by="
                + u.getID() + " and date_inactive is null union "
                + "select id from mmm_user where id>=10) q";
        List l = listPersons(query);
        Collections.sort(l, ObjectSorter
                .personByFirstName(ObjectSorter.ALPHABETICAL));
        Collections.sort(l, ObjectSorter
                .personByLastName(ObjectSorter.ALPHABETICAL));
        return l;
    }

    public static List listAllPersons() {
        String query = "select distinct id from person where id>=10";
        List l = listPersons(query);
        Collections.sort(l, ObjectSorter
                .personByFirstName(ObjectSorter.ALPHABETICAL));
        Collections.sort(l, ObjectSorter
                .personByLastName(ObjectSorter.ALPHABETICAL));
        return l;
    }

    /**
     * @param query
     * @return
     */
    protected static List listPersons(String query) {
    		ArrayList l = new ArrayList();
    		if (query == "")
        		return l;
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        try {
            while (rs.next()) {
                int id = rs.getInt(1);
                IPerson p = getByID(id);
                if (p != null) {
                    l.add(p);
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return l;
    }

    // returns a list of EmailAddress
    /**
     * @param query
     * @return
     */
    protected static List listEmails(String query) {
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        ArrayList l = new ArrayList();
        EmailAddress email = null;
        try {
            while (rs.next()) {
                String addr = rs.getString("email_address");
                int id = rs.getInt("id");
                int owner = rs.getInt("person_id");
                email = new EmailAddress(id, addr, PersonManager.getByID(owner));
                if (email != null) {
                    l.add(email);
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return l;
    }

    /**
     * @param person
     * @param btMac
     * @param creator
     */
    public static void associateBTDevice(IPerson person, String btMac,
            IUser creator) { // creator is guy who created association
        // btMac should be lowercase hex, no punctuation -- TODO validate string
        // format?
        // TODO make this more elegant -- deactivate and reinsert isn't so
        // pretty
        int id = RDBConnection.getNextVal("person_bt_id_seq");
        String query = "update person_bluetooth set date_inactive=now() where bt_mac='"
                + btMac + "'";
        RDBConnection.execute(query);
        query = "insert into person_bluetooth (id, person_id, bt_mac, created_by, date_added) "
                + "values ("
                + id
                + ", "
                + person.getID()
                + ", '"
                + btMac
                + "', " + creator.getID() + ", now())";
        RDBConnection.execute(query);
    }

    // returns a list of Bluetooth MAC addresses
    protected static List listBTDevices(IPerson person) {

        String query = "select bt_mac, person_id from person_bluetooth where person_id="
                + person.getID() + " and date_inactive is null";

        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        ArrayList l = new ArrayList();
        try {
            while (rs.next()) {
                String addr = rs.getString("bt_mac");
                if (addr != null) {
                    l.add(addr);
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return l;
    }

    public static void removeBluetoothMAC(IPerson person, String btMAC) {
        String query = "update person_bluetooth set date_inactive=now() where bt_mac='"
                + btMAC + "' and person_id=" + person.getID();
        RDBConnection.execute(query);
    }

    /**
     * @param person
     * @param firstName
     */
    public static void setFirstName(IPerson person, String firstName) {
        String query = "update person set first='" + firstName + "' where id='"
                + person.getID() + "'";
        RDBConnection.execute(query);
    }

    /**
     * @param person
     * @param lastName
     */
    public static void setLastName(IPerson person, String lastName) {
        String query = "update person set last='" + lastName + "' where id='"
                + person.getID() + "'";
        RDBConnection.execute(query);
    }

    /**
     * @param query
     * @return
     */
    public static List listUsersOrPersons(String query) {
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        ArrayList l = new ArrayList();
        try {
            while (rs.next()) {
                int id = rs.getInt(1);
                // try to get user record
                IPerson p = UserManager.getByID(id);

                if (p == null) {
                    p = PersonManager.getByID(id);
                }

                if (p != null) {
                    l.add(p);
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return l;
    }

    public static List listPhotoAnnotations(int photoID) {
        String query = "select person_id, left_coordinate, top_coordinate, width, height from person_in_photo "
                + "where photo_id = " + photoID;

        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        ArrayList l = new ArrayList();
        try {
            while (rs.next()) {
                int id = rs.getInt(1);
                IPerson p = null; // UserManager.getByID(id);

                if (p == null) {
                    p = PersonManager.getByID(id);
                }
                if (p != null) {
                    AnnotationImpl a = new AnnotationImpl(photoID, p, rs
                            .getInt("left_coordinate"), rs
                            .getInt("top_coordinate"), rs.getInt("width"), rs
                            .getInt("height"));
                    l.add(a);
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return l;
    }

    public static void removeAllAnnotations(int photoID) {

        String query = "delete from person_in_photo where photo_id=" + photoID;
        RDBConnection.execute(query);

    }

    public static void addAnnotation(int photoID, String firstName,
            String lastName, int xcoord, int ycoord, int width, int height,
            IUser creatingUser) {

        System.out.println("About to Add Annotation: " + photoID + " "
                + firstName + " " + xcoord);

        // Check if person exist
        List personsList = listPersons(firstName, lastName);
        Iterator persons = personsList.iterator();
        IPerson person = null;

        if (persons.hasNext()) {
            // Person exists
            person = (IPerson) persons.next();
        } else {
            // Creating a new person
            person = createPerson(firstName, lastName, creatingUser);
        }

        int id = RDBConnection.getNextVal("consumer_id_seq");
        String query = "insert into person_in_photo (id, photo_id, person_id, left_coordinate, top_coordinate, width, height) "
                + "values ("
                + id
                + ", "
                + photoID
                + ", "
                + person.getID()
                + ", "
                + xcoord
                + ", "
                + ycoord
                + ", "
                + width
                + ", "
                + height
                + ")";
        RDBConnection.execute(query);
        System.out.println("Added Annotation: " + photoID + " "
                + person.toString() + " " + xcoord);

    }

}
