/*
 * Created on Jul 19, 2005
 *
 */
package gcr.mmm2.model;

import gcr.mmm2.rdb.RDBConnection;
import gcr.mmm2.rdb.ResultSetWrapper;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Breaking up the MediaManager class into factory chunks.
 * 
 * @author Benjamin
 * 
 */
public class PhotoFactory {

    public static IPhoto createPhoto(String JPEGfilename, IUser user) {
        int id = RDBConnection.getNextVal("media_id_seq");
        try {
            String query = "insert into photo (id, image_file, created_by, date_added) "
                    + "values ("
                    + id
                    + ", '"
                    + JPEGfilename
                    + "', "
                    + user.getID() + ", now())";
            RDBConnection.execute(query);
            MediaManager.createRDFPhotoEntry(id);
            return new PhotoImpl(id);
        } catch (ObjectNotFoundException e) {
            e.printStackTrace();
            return null;
        }
    }

    public static IPhoto createPhoto(String JPEGfilename, IUser user,
            Date captureDate) {
        int id = RDBConnection.getNextVal("media_id_seq");
        try {
            String query = "insert into photo (id, image_file, created_by, capture_date, date_added) "
                    + "values ("
                    + id
                    + ", '"
                    + JPEGfilename
                    + "', "
                    + user.getID() + ", '" + captureDate + "', now())";
            RDBConnection.execute(query);
            MediaManager.createRDFPhotoEntry(id);
            return new PhotoImpl(id);
        } catch (ObjectNotFoundException e) {
            e.printStackTrace();
            return null;
        }
    }

    public static List listAllOwnedPhotos(IUser user) {
        return listAllOwnedPhotos(user, MediaManager.SORT_DATE_DESC);
    }

    // lists all photos owned by user whether in albums or not
    public static List listAllOwnedPhotos(IUser user, int sortOrder) {
        String query = "select id, case capture_date is null when true then date_added else capture_date end as the_date "
                + "from photo where created_by="
                + user.getID()
                + " and date_inactive is null";
        if (sortOrder == MediaManager.SORT_DATE_ASC) {
            query = query + " order by the_date asc";
        } else if (sortOrder == MediaManager.SORT_DATE_DESC) {
            query = query + " order by the_date desc";
        }
        return PhotoFactory.listPhotos(query);
    }

    public static List listAllOwnedPhotosByTime(IUser user, int timeID) {
        return listAllOwnedPhotosByTime(user, timeID,
                MediaManager.SORT_DATE_DESC);
    }
    
    public static int countAllOwnedPhotosByTime(IUser user, int timeID) {
        return countRows(getQueryListAllOwnedPhotosByTime(user, timeID,
                MediaManager.SORT_DATE_DESC));
    }
    
    public static String getQueryListAllOwnedPhotosByTime(IUser user, int timeID,
            int sortOrder)
    {
        /*
         * Set the Date Range Strings
         */
        String start_date = MediaManager.getSQLStartDate(timeID);
        String end_date = MediaManager.getSQLEndDate(timeID);

        String query = "select id, case capture_date is null when true then date_added else"
                + " capture_date end as the_date from photo where created_by="
                + user.getID() + " and date_inactive is null";

        /*
         * Sorting the Results (By Date)
         */
        if (sortOrder == MediaManager.SORT_DATE_DESC) {
            query = "select id, max(the_date) as the_date from (" + query
                    + ") q group by id having max(the_date) between "
                    + start_date + " and " + end_date
                    + " order by the_date desc";
        } else if (sortOrder == MediaManager.SORT_DATE_ASC) {
            query = "select id, min(the_date) as the_date from (" + query
                    + ") q group by id having min(the_date) between "
                    + start_date + " and " + end_date
                    + " order by the_date asc";
        } else {
            query = "select distinct id from (" + query
                    + ") q where the_date between " + start_date + " and "
                    + end_date;
        }
        return query;
    }

    // lists all photos owned by user whether in albums or not
    public static List listAllOwnedPhotosByTime(IUser user, int timeID,
            int sortOrder) {
        return PhotoFactory.listPhotos(
                getQueryListAllOwnedPhotosByTime(
                        user,timeID,sortOrder));
    }

    // lists all photos
    public static List listAllPhotosByTime(Date startDate, Date endDate) {

        /*
         * Set the Date Range Strings
         */
        String start_date = MediaManager.getSQLDate(startDate);
        String end_date = MediaManager.getSQLDate(endDate);

        /*
         * String query = "select id, case capture_date is null when true then
         * date_added else" + " capture_date end as the_date from photo";
         */

        /*
         * query = "select id, max(the_date) as the_date from (" + query + ") q
         * group by id having max(the_date) between " + start_date + " and " +
         * end_date + " order by the_date asc";
         */

        /*
         * query = "select distinct id from (" + query + ") q where the_date
         * between " + start_date + " and " + end_date;
         */
        String query = "SELECT id, date_added FROM photo "
                + "WHERE date_added BETWEEN " + start_date + " and " + end_date
                + " order by date_added ASC";

        return PhotoFactory.listPhotos(query);
    }
    
    public static String getQueryListAllPublicPhotosByTime( int timeID) {
        /*
         * Set the Date Range Strings
         */
        String start_date = MediaManager.getSQLStartDate(timeID);
        String end_date = MediaManager.getSQLEndDate(timeID);

        String query = "select id, case capture_date is null when true then date_added else capture_date end as the_date "
            + "from photo where date_inactive is null"
            + " and photo_privacy=2" 
            + " and date_added BETWEEN " + start_date + " and " + end_date
            + " order by the_date desc";
        return query;
    }

    // lists all public photos
    public static List listAllPublicPhotosByTime( int timeID) {
	    return PhotoFactory.listPhotos(
                getQueryListAllPublicPhotosByTime(timeID));
    }
    
    public static int countAllPublicPhotosByTime( int timeID) {
        return countRows(getQueryListAllPublicPhotosByTime(timeID));
    }

    // lists all public photos filtered by user
    public static List listAllPublicPhotosByTimeAndUser( int timeID, int userID) {

        /*
         * Set the Date Range Strings
         */
        String start_date = MediaManager.getSQLStartDate(timeID);
        String end_date = MediaManager.getSQLEndDate(timeID);

        String query = "select id, case capture_date is null when true then date_added else capture_date end as the_date "
            + "from photo where date_inactive is null"
            + " and photo_privacy=2" 
            + " and date_added BETWEEN " + start_date + " and " + end_date
            + " and created_by = " + userID
            + " order by the_date desc";
	    return PhotoFactory.listPhotos(query);

    }

    // returns a list of random photos 
    // (does not include photos set as restricted or video clips)
    public static List listRandomPhotosByTime( int timeID, int numPhotos, int unique, IUser user) {

        /*
         * Set the Date Range Strings
         */
        String start_date = MediaManager.getSQLStartDate(timeID);
        String end_date = MediaManager.getSQLEndDate(timeID);

        String query = "select id, random()"
            + " from photo where date_inactive is null"
            + " and (photo_privacy is null or photo_privacy = 0 or photo_privacy = 2)" 
            + " and date_added BETWEEN " + start_date + " and " + end_date
            + " and image_file LIKE '%jpg'";
        	if (unique == 1) {
        		query += " and id not in"
        		+ " (select c.photo_id"
        		+ " from album a, album_contents c"
        		+ " where a.id = c.album_id"
        		+ " and c.created_by = " + user.getID()
        		+ " and a.date_inactive is null)";
        	}
        	query += " order by 2"
            + " limit " + numPhotos;
        	System.out.println("QUERY: " + query);
        
	    return PhotoFactory.listPhotos(query);

    }

   public static List listAllReceivedPhotosByTime(IUser user, int timeID) {
        return listAllReceivedPhotosByTime(user, timeID,
                MediaManager.SORT_DATE_DESC);
    }
   
   public static int countAllReceivedPhotosByTime(IUser user, int timeID) {
       return countRows(getQueryListAllReceivedPhotosByTime(user, timeID,
               MediaManager.SORT_DATE_DESC));
   }
   
   public static String getQueryListAllReceivedPhotosByTime(IUser user, int timeID,
           int sortOrder) {
       
       /*
        * Set the Date Range Strings
        */
       String start_date = MediaManager.getSQLStartDate(timeID);
       String end_date = MediaManager.getSQLEndDate(timeID);

       /*
        * Form the Query
        */
       String query = "select id, case capture_date is null when true then date_added else capture_date end as the_date "
               + "from photo where visibility=3 and date_inactive is null";

       query += " union select sp.photo_id as id, sr.date_added as the_date from share_recipient sr, share_photo sp, photo p "
               + "where sr.share_id=sp.share_id and sp.photo_id=p.id and sr.recipient="
               + user.getID()
               + " and "
               + "sr.date_inactive is null and sp.date_inactive is null and p.date_inactive is null";

       query += " union select ac.photo_id as id, sr.date_added as the_date from share_recipient sr, share_album sa, album_contents ac, photo p "
               + "where ac.photo_id=p.id and sr.share_id=sa.share_id and ac.album_id=sa.album_id and ac.date_inactive is null "
               + "and p.date_inactive is null and sr.date_inactive is null and sa.date_inactive is null and sr.recipient="
               + user.getID();

       /*
        * Sorting the Results (By Date)
        */
       if (sortOrder == MediaManager.SORT_DATE_DESC) {
           query = "select id, max(the_date) as the_date from (" + query
                   + ") q group by id having max(the_date) between "
                   + start_date + " and " + end_date
                   + " order by the_date desc";
       } else if (sortOrder == MediaManager.SORT_DATE_ASC) {
           query = "select id, min(the_date) as the_date from (" + query
                   + ") q group by id having min(the_date) between "
                   + start_date + " and " + end_date
                   + " order by the_date asc";
       } else {
           query = "select distinct id from (" + query
                   + ") q where the_date between " + start_date + " and "
                   + end_date;
       }
       
       return query;
       
   }

    public static List listAllReceivedPhotosByTime(IUser user, int timeID,
            int sortOrder) {
        
        return PhotoFactory.listPhotos(
                getQueryListAllReceivedPhotosByTime(user,
                        timeID,sortOrder));
    }

    public static List listAllVisiblePhotos(IUser user) {
        return listAllVisiblePhotos(user, MediaManager.SORT_DATE_DESC);
    }

    // lists all photos a user has access to (via shares, public, etc.) whether
    // in albums or not
    // doesn't yet think about groups -- only shares
    public static List listAllVisiblePhotos(IUser user, int sortOrder) {
        String query = "select id, case capture_date is null when true then date_added else capture_date end as the_date "
                + "from photo where (created_by="
                + user.getID()
                + " or visibility=3) and date_inactive is null "
                + "union select sp.photo_id as id, sr.date_added as the_date from share_recipient sr, share_photo sp "
                + "where sr.share_id=sp.share_id and sr.recipient="
                + user.getID()
                + " and sr.date_inactive is null and sp.date_inactive is null "
                + "union select ac.photo_id as id, sr.date_added as the_date from share_recipient sr, share_album sa, album_contents ac "
                + "where sr.share_id=sa.share_id and ac.album_id=sa.album_id and ac.date_inactive is null "
                + "and sr.date_inactive is null and sa.date_inactive is null and sr.recipient="
                + user.getID();
        if (sortOrder == MediaManager.SORT_DATE_ASC) {
            query = "select id, max(the_date) as the_date from (" + query
                    + ") q group by id order by the_date desc";
        } else if (sortOrder == MediaManager.SORT_DATE_DESC) {
            query = "select id, min(the_date) as the_date from (" + query
                    + ") q group by id order by the_date asc";
        } else {
            query = "select distinct id from (" + query + ") q";
        }
        return PhotoFactory.listPhotos(query);
    }
    
    //List all the photos visible by a user with a given bluetooth mac address in context	
    public static List listAllVisiblePhotosByBluetoohMac(IUser user, int sortOrder, String bt_mac) {
    String query = "select photo.id, case capture_date is null when true then date_added else capture_date end as the_date" 
    				+" from photo, context_bluetooth " 
    				+"where (created_by="
    				+user.getID()
    				+" or visibility=3  or photo_privacy=2) and" 
    				+" photo.context_id = context_bluetooth.context_id and date_inactive is null and bt_mac = '"
    				+ bt_mac
    				+ "' union " 
    				+"select sp.photo_id as id, sr.date_added as the_date from share_recipient sr, share_photo sp, photo p, context_bluetooth cb "
                 +"where sr.share_id=sp.share_id and sr.recipient= " 
                 + user.getID() 
                 +" and sp.photo_id = p.id and p.context_id = cb.context_id and cb.bt_mac = '"
                 + bt_mac
                 + "' and sr.date_inactive is null and sp.date_inactive is null "
                 + " union " 
                 + "select ac.photo_id as id, sr.date_added as the_date from share_recipient sr, share_album sa, album_contents ac, photo p, context_bluetooth cb " 
                 + "where sr.share_id=sa.share_id and ac.album_id=sa.album_id and ac.date_inactive is null "
                 + "and sr.date_inactive is null and sa.date_inactive is null and sr.recipient=" 
                 + user.getID() 
                 +" and ac.photo_id = p.id AND  p.context_id = cb.context_id and cb.bt_mac = '"
                 + bt_mac	+"'";
    	
        if (sortOrder == MediaManager.SORT_DATE_ASC) {
            query = "select id, max(the_date) as the_date from (" + query
                    + ") q group by id order by the_date desc";
        } else if (sortOrder == MediaManager.SORT_DATE_DESC) {
            query = "select id, min(the_date) as the_date from (" + query
                    + ") q group by id order by the_date asc";
        } else {
            query = "select distinct id from (" + query + ") q";
        }
        System.out.println(query);
        return PhotoFactory.listPhotos(query);
    }
  

    public static List listAllVisiblePhotosByTime(IUser user, int timeID) {
        return listAllVisiblePhotosByTime(user, timeID,
                MediaManager.SORT_DATE_DESC);
    }
    
    public static int countAllVisiblePhotosByTime(IUser user, int timeID) {
        return countRows(getQueryListAllVisiblePhotosByTime(user, timeID,
                MediaManager.SORT_DATE_DESC));
    }

    public static String getQueryListAllVisiblePhotosByTime(IUser user, int timeID,
            int sortOrder)
    {
        /*
         * Bookend the time period
         */
        String start_date = MediaManager.getSQLStartDate(timeID);
        String end_date = MediaManager.getSQLEndDate(timeID);

        /*
         * Form the Query
         */
        String query = "select id, case capture_date is null when true then date_added else capture_date end as the_date "
                + "from photo where (created_by="
                + user.getID()
                + " or visibility=3) and date_inactive is null";

        query += " union select sp.photo_id as id, sr.date_added as the_date from share_recipient sr, share_photo sp, photo p "
                + "where sr.share_id=sp.share_id and sp.photo_id=p.id and sr.recipient="
                + user.getID()
                + " and sr.date_inactive is null "
                + "and sp.date_inactive is null and p.date_inactive is null ";

        query += " union select ac.photo_id as id, sr.date_added as the_date from share_recipient sr, share_album sa, album_contents ac, photo p "
                + "where ac.photo_id=p.id and sr.share_id=sa.share_id and ac.album_id=sa.album_id and p.date_inactive is null and ac.date_inactive is null "
                + "and sr.date_inactive is null and sa.date_inactive is null and sr.recipient="
                + user.getID();

        /*
         * Sorting the Results (By Date)
         */
        if (sortOrder == MediaManager.SORT_DATE_DESC) {
            query = "select id, max(the_date) as the_date from (" + query
                    + ") q group by id having max(the_date) between "
                    + start_date + " and " + end_date
                    + " order by the_date desc";
        } else if (sortOrder == MediaManager.SORT_DATE_ASC) {
            query = "select id, min(the_date) as the_date from (" + query
                    + ") q group by id having min(the_date) between "
                    + start_date + " and " + end_date
                    + " order by the_date asc";
        } else {
            query = "select distinct id from (" + query
                    + ") q where the_date between " + start_date + " and "
                    + end_date;
        }
        return query;
    }
    public static List listAllVisiblePhotosByTime(IUser user, int timeID,
            int sortOrder) {
        return PhotoFactory.listPhotos(
                getQueryListAllVisiblePhotosByTime(user,timeID,sortOrder));
    }

    public static List listAllVisiblePhotosInCellLocation(String network,
            String lac, String cellID, IUser user, int sortOrder) {
        // TODO: add photo.date_inactive is null
        String query = "select id, case capture_date is null when true then date_added else capture_date end as the_date "
                + "from photo where (created_by="
                + user.getID()
                + " or visibility=3) and date_inactive is null "
                + "union select sp.photo_id as id, sr.date_added as the_date from share_recipient sr, share_photo sp, photo p "
                + "where sr.share_id=sp.share_id and sp.photo_id=p.id and sr.recipient="
                + user.getID()
                + " and p.date_inactive is null"
                + " and sr.date_inactive is null and sp.date_inactive is null "
                + "union select ac.photo_id as id, sr.date_added as the_date from share_recipient sr, share_album sa, album_contents ac, photo p "
                + "where sr.share_id=sa.share_id and ac.photo_id=p.id and ac.album_id=sa.album_id and ac.date_inactive is null "
                + "and p.date_inactive is null and sr.date_inactive is null and sa.date_inactive is null and sr.recipient="
                + user.getID();

        query = "select p.id as id, qq.the_date as the_date from photo p, context c, ("
                + query + ") qq where p.id=qq.id and c.id=p.context_id";
        if (network != null)
            query += " and c.network='" + network + "'";
        if (lac != null)
            query += " and c.lac='" + lac + "'";
        if (cellID != null)
            query += " and c.cell_id='" + cellID + "'";

        if (sortOrder == MediaManager.SORT_DATE_ASC) {
            query = "select id, max(the_date) as the_date from (" + query
                    + ") q group by id order by the_date desc";
        } else if (sortOrder == MediaManager.SORT_DATE_DESC) {
            query = "select id, min(the_date) as the_date from (" + query
                    + ") q group by id order by the_date asc";
        } else {
            query = "select distinct p.id from (" + query + ") q";
        }

        return PhotoFactory.listPhotos(query);
    }

    public static IPhoto getPhotoByFilename(String filename) {
        String query = "select id from photo where image_file='" + filename
                + "' and date_inactive is null";
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        IPhoto p = null;
        try {
            if (rs.next()) {
                int id = rs.getInt("id");
                p = new PhotoImpl(id);
            }
        } catch (Exception e) {
            e.printStackTrace();
    
        } finally {
            rsw.closeAll();
        }
        return p;
    }

    public static IPhoto getPhotoByID(int id) {
        return CacheManager.getPhoto(id);
    }

    protected static List listPhotos(String query) {
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        
        /* testing code to get row count  
        try{
            rs.last();
            System.out.println("This resultset count is " + rs.getRow());
            rs.beforeFirst();
        }catch(SQLException se)
        {}
        */
        
        ArrayList l = new ArrayList();
        try {
            while (rs.next()) {
                int id = rs.getInt(1);
                IPhoto p = getPhotoByID(id);
                if (p != null) {
                    l.add(p);
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return l;
    }
    
    public static int countRows(String query) {
        
        int myCount = 0;
        
        query = "SELECT COUNT(*) AS nummy FROM (" + query  +") as foo";
        
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        
        try {
            while (rs.next())
            {
                    myCount = rs.getInt("nummy");
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        // System.out.println("MAGIC COUNT=" + myCount);
        return myCount;
    }

    public static List listPhotosInAlbum(IAlbum album) {
        return listPhotosInAlbum(album, MediaManager.SORT_DATE_ASC);
    }

    public static List listPhotosInAlbum(IAlbum album, int sortOrder) {
        String query = "select p.id, case p.capture_date is null when true then p.date_added else p.capture_date end as the_date "
                + "from album_contents ac, photo p where ac.photo_id=p.id and p.date_inactive is null and ac.date_inactive is null "
                + "and album_id=" + album.getID();
        if (sortOrder == MediaManager.SORT_DATE_ASC) {
            query = query + " order by the_date asc";
        } else if (sortOrder == MediaManager.SORT_DATE_DESC) {
            query = query + " order by the_date desc";
        }
        return listPhotos(query);
    }

}
