/*
 * Created on Aug 10, 2004
 */
package gcr.mmm2.model;

import gcr.mmm2.rdb.RDBConnection;
import gcr.mmm2.rdb.ResultSetWrapper;
import gcr.mmm2.server.mail.NotificationMailer;
import gcr.mmm2.util.MMM2Properties;
import gcr.mmm2.util.StringUtils;
import gcr.rdf.MMMModelFactory;

import java.io.File;
import java.io.FilenameFilter;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;
import java.sql.Time;
import java.util.List;

/**
 * Represents information for a photo object.
 * 
 * @author Simon King
 */
public class PhotoImpl implements IPhoto {
    private int m_id;

    private String m_JPEGImageFile;

    private String m_ThumbImageFile;

    private String m_BMPImageFile;

    private String m_caption;

    private String m_title;

    private int m_ownerID;

    private int m_photographerID, m_contextID;

    private int m_visibility;

    private Date m_captureDate;

    private Date m_uploadDate;

    private String m_place, m_tag;

    private int m_restriction;

    /**
     * don't normally use this, but needed for special routines
     */
    public PhotoImpl() {

    }

    public PhotoImpl(int id, boolean loadDeleted)
            throws ObjectNotFoundException {
        m_id = id;
        // String query = "select tag, place, context_id, image_file, title,
        // caption, created_by, photographer, visibility, capture_date, "+
        // "date_added, image_thumb_file, image_bitmap_file from photo where
        // id="+m_id+" and date_inactive is null";
        String query = "select p.tag, p.place, p.context_id, p.image_file, p.title, p.caption, p.created_by, p.photographer, p.visibility, p.capture_date,"
                + "p.date_added, p.image_thumb_file, p.image_bitmap_file, p.date_inactive, p.photo_privacy, (select privacy_level from photo_privacy where id=p.photo_privacy) as privacy_level from photo p where p.id="
                + m_id;
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        try {
            // rs.next();
            // System.out.println(rs.getDate("date_inactive"));
            // load the record if it exists and dateDeleted is null OR
            // dateDeleted is not null and we want to load deleted anyway
            if (rs.next()
                    && ((rs.getDate("date_inactive") == null) || (rs
                            .getDate("date_inactive") != null && loadDeleted == true))) {
                m_tag = StringUtils.URLDecode(rs.getString("tag"));
                m_place = StringUtils.URLDecode(rs.getString("place"));
                m_JPEGImageFile = rs.getString("image_file");
                m_ThumbImageFile = rs.getString("image_thumb_file");
                m_BMPImageFile = rs.getString("image_bitmap_file");
                m_title = StringUtils.URLDecode(rs.getString("title"));
                this.m_caption = StringUtils.URLDecode(rs.getString("caption"));
                m_ownerID = rs.getInt("created_by");
                m_photographerID = rs.getInt("photographer");
                m_visibility = rs.getInt("visibility");
                m_contextID = rs.getInt("context_id");
                m_captureDate = rs.getDate("capture_date");
                m_uploadDate = rs.getDate("date_added");
                m_restriction = rs.getInt("photo_privacy");

                // The time does not seem to set properly, so this works around
                Time captureTime = rs.getTime("capture_date");
                if (captureTime != null) {
                    m_captureDate.setTime(captureTime.getTime());
                }
            } else {
                throw new ObjectNotFoundException("Unable to find photo(id)="
                        + id);
            }
        } catch (SQLException e) {
            e.printStackTrace();
            throw new ObjectNotFoundException("Unable to find photo(id)=" + id);
        } finally {
            rsw.closeAll();
        }
    }

    public PhotoImpl(int id) throws ObjectNotFoundException {
        this(id, false);
    }

    public String getURL(String base, String code, int size) {
        return MediaManager.getPhotoURL(this, base, code, size);
    }

    public String getURL(String base, String code, int width, int height) {
        return MediaManager.getPhotoURL(this, base, code, width, height);
    }

    public String getURL(String base, String code, int width, int height,
            boolean crop) {
        return MediaManager.getPhotoURL(this, base, code, width, height, crop);
    }

    public String getTag() {
        return m_tag;
    }

    public void setTag(String tag, IUser owner) throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify photo id " + m_id + " owner by "
                    + m_ownerID);
        }
        updateValue("tag", tag);
        m_tag = tag;
    }

    // place is temporary
    public String getPlace() {
        return m_place;
    }

    public void setPlace(String place, IUser owner) throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify photo id " + m_id + " owner by "
                    + m_ownerID);
        }
        updateValue("place", place);
        m_place = place;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPhoto#getID()
     */
    public int getID() {
        return m_id;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPhoto#getJPEGFilename()
     */
    public String getJPEGFilename() {
        return m_JPEGImageFile;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPhoto#getJPEGThumbFilename()
     */
    public String getJPEGThumbFilename() {
        return m_ThumbImageFile;
    }

    private boolean hasPermission(IUser owner) {
        try {
            return (owner.getID() == m_ownerID || UserManager.isAdmin(owner));
        } catch (Exception e) {
            return false;
        }
    }

    public IPerson getPhotographer() {
        return PersonManager.getByID(m_photographerID);
    }

    public void setPhotographer(IPerson photographer, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify photo id " + m_id + " owner by "
                    + m_ownerID);
        }
        if (photographer != null) {
            updateValue("photographer", photographer.getID());
            m_photographerID = photographer.getID();
        }
    }

    private void updateValue(String colName, String newVal) {
        String query = "update photo set " + colName + "=? where id=?";
        Object[] params = { StringUtils.URLEncode(newVal), new Integer(m_id) };
        RDBConnection.executePrepared(query, params);
    }

    private void updateValue(String colName, int newVal) {
        String query = "update photo set " + colName + "=? where id=?";
        Object[] params = { new Integer(newVal), new Integer(m_id) };
        RDBConnection.executePrepared(query, params);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPhoto#setJPEGThumbFilename(java.lang.String)
     */
    public void setJPEGThumbFilename(String filename, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify photo id " + m_id + " owner by "
                    + m_ownerID);
        }
        updateValue("image_thumb_file", filename);
        m_ThumbImageFile = filename;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPhoto#getBMPFilename()
     */
    public String getBMPFilename() {
        return m_BMPImageFile;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPhoto#setBMPFilename(java.lang.String)
     */
    public void setBMPFilename(String filename, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify photo id " + m_id + " owner by "
                    + m_ownerID);
        }
        updateValue("image_bitmap_file", filename);
        m_BMPImageFile = filename;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPhoto#getOwner()
     */
    public IUser getOwner() {
        return UserManager.getByID(m_ownerID);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPhoto#getVisibility()
     */
    public int getVisibility() {
        return m_visibility;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPhoto#setVisibility(int)
     */
    public void setVisibility(int visibility, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify photo id " + m_id + " owner by "
                    + m_ownerID);
        }
        updateValue("visibility", visibility);
        m_visibility = visibility;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPhoto#getCaption()
     */
    public String getCaption() {
        if (m_caption == null) {
            return "";
        }
        return m_caption;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPhoto#setCaption(java.lang.String)
     */
    public void setCaption(String caption, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify photo id " + m_id + " owner by "
                    + m_ownerID);
        }
        updateValue("caption", caption);
        m_caption = caption;
    }

    public String getTitle() {
        return m_title;
    }

    public void setTitle(String title, IUser owner) throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify photo id " + m_id + " owner by "
                    + m_ownerID);
        }
        updateValue("title", title);
        m_title = title;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPhoto#listComments()
     */
    // return a List of String
    public List listComments() {
        String query = "select id, created_by, comment, date_added from media_comment where "
                + "media_id=" + m_id + " and date_inactive is null";
        return MediaManager.listComments(query);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPhoto#addComment(java.lang.String)
     */
    public void addComment(String comment, IUser u) {
        if (comment != null) {
            int id = RDBConnection.getNextVal("comment_id_seq");
            String query = "insert into media_comment (id, media_id, comment, date_added, created_by) "
                    + "values (?, ?, ?, now(), ?)";
            Object[] params = { new Integer(id), new Integer(m_id),
                    StringUtils.URLEncode(comment), new Integer(u.getID()) };
            RDBConnection.executePrepared(query, params);

            /*
             * Notify photo owner This is always delivered to users, so the
             * links lead into protected areas
             */
            if (u.getID() != this.m_ownerID) {
                comment = StringUtils.URLDecode(comment);
                EmailAddress senderEmail = u.getPrimaryEmail();
                EmailAddress recipEmail = this.getOwner().getPrimaryEmail();
                StringBuffer textPart = new StringBuffer();
                StringBuffer htmlPart = new StringBuffer();

                htmlPart.append("<html>\n<body>\n");
                htmlPart.append(u.toString() + " has left a comment:<br>\n"
                        + comment + "<br>\n<br>\n");
                htmlPart.append("<div style='clear:both;'>");

                textPart.append(u.toString() + " has left a comment:\n"
                        + comment + "\n\n");

                String serverURL = MMM2Properties.getProperty("MMM_URL");

                htmlPart
                        .append("<div style='position:relative;float:left;margin:5px;'>\n");
                htmlPart.append("<a href='" + serverURL
                        + "/photo_details.jsp?photoID=" + this.getID()
                        + "&commentIdentifier=" + id + "'>");
                htmlPart
                        .append("<img src='"
                                + this.getURL(serverURL,
                                        "comment_html_email_thumb_" + id, 200)
                                + "' style='border-color:black;border-width:1px;border-style:solid;' >");
                htmlPart.append("</a>\n");
                htmlPart.append("</div>\n");

                textPart.append(serverURL + "/photo_details.jsp?photoID="
                        + this.getID() + "&commentIdentifier=" + id + "'>");

                htmlPart.append("</div>");
                htmlPart
                        .append("<div style='clear:both;'><hr>This message brought to you by... <a href='"
                                + serverURL + "'>MMM2</a></div>\n");
                htmlPart.append("</body>\n</html>");

                textPart
                        .append("\n\n---------------------------------------------------------\n");
                textPart.append("This message brought to you by...MMM!");
                if (recipEmail != null && senderEmail != null) {
                    NotificationMailer.sendHtmlMsg(senderEmail.getAddress(),
                            recipEmail.getAddress(),
                            "MMM2 Comment Notification", textPart.toString(),
                            htmlPart.toString());
                }

            }// end if user != owner
        }// if comment !=null
    }

    public void removeComment(int id, IUser owner) throws PermissionException {
        if (!hasPermission(owner)) {
            // now check if owner owns the comment itself
            if (owner.getID() != MediaManager.getCommentByID(id).getOwner()
                    .getID()) {
                throw new PermissionException("User " + owner.getID()
                        + " tried to remove comment id " + id
                        + " for photo id " + m_id + " owner by " + m_ownerID);
            }
        }
        MediaManager.deleteComment(id);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPhoto#getCaptureTime()
     */
    public Date getCaptureDate() {
        if (m_captureDate == null) {
            return getUploadDate();
        }
        return m_captureDate;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPhoto#getUploadDate()
     */
    public Date getUploadDate() {
        return m_uploadDate;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPhoto#listAlbums()
     */
    public List listAlbums() {
        return AlbumFactory.listAlbums(this);
    }

    public void delete(IUser owner) throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify photo id " + m_id + " owner by "
                    + m_ownerID);
        }

        // Remove from RDBMS
        String query = "update photo set date_inactive=now() where id="
                + this.getID();
        RDBConnection.execute(query);
        query = "update album_contents set date_inactive=now() where photo_id="
                + this.getID();
        RDBConnection.execute(query);
        query = "update share_photo set date_inactive=now() where photo_id="
                + this.getID();
        RDBConnection.execute(query);
        this.clearCachedImages();
    }

    public void setContext(IContext context, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify photo id " + m_id + " owner by "
                    + m_ownerID);
        }
        updateValue("context_id", context.getID());
        m_contextID = context.getID();
    }

    public IContext getContext() {
    		//Check if photo has context to retrieve.
    		//If m_contextID = 0, most likely an uploaded photo.
    		if (m_contextID == 0) {
    			System.out.println("NOTE: Context unavailabe for photo: " + this.m_id +". ");
    			return null;
    		} else 
    			return ContextManager.getContextByID(m_contextID);
     }

    public String getURI() {
        return MMMModelFactory.BASE_URI + "photo/" + m_id;
    }

    public String toString() {
        return StringUtils.URLDecode(getCaption());
    }

    // Privacy Level
    public int getPrivacyLevel(IUser owner) throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify restriction.");
        }
        return m_restriction;
    }

    public void setPrivacyLevel(int level, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to set restriction.");
        }
        m_restriction = level;
        this.updateValue("photo_privacy", level);
    }

    // Returns true is restriction setting of photo is set to public
    public boolean isPublic() {
        return m_restriction == IPhoto.PUBLIC_PRIVACY;
    }

    public void unshare(IUser user) {
        String query = "UPDATE share_recipient SET date_inactive = CURRENT_TIMESTAMP"
                + " WHERE share_recipient.share_id=share_photo.share_id"
                + " AND share_recipient.recipient="
                + user.getID()
                + " AND share_photo.photo_id = "
                + this.getID()
                + " AND share_recipient.date_inactive is null";
        RDBConnection.execute(query);
        this.clearCachedImages();
    }

    public void clearCachedImages() {
        String dirName = MediaManager.THUMB_IMAGE_DIR + "/"
                + this.getOwner().getID();
        File dir = new File(dirName);
        String nameFilter = "_" + this.getJPEGFilename();
        FilenameFilter ff = new ImageFileFilter(dir, nameFilter);
        File[] fileMatches = dir.listFiles(ff);
        if (fileMatches != null) {
            for (int i = 0; i < fileMatches.length; i++) {
                System.out.println("\tremove? " + fileMatches[i].getName());
                fileMatches[i].delete();
            }
        }
    }

}
