/*
 * Created on Aug 11, 2004
 *
 */
package gcr.mmm2.model;

import gcr.mmm2.rdb.RDBConnection;
import gcr.mmm2.rdb.ResultSetWrapper;
import gcr.mmm2.server.mail.NotificationMailer;
import gcr.mmm2.util.MMM2Properties;
import gcr.mmm2.util.StringUtils;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

/**
 * This is an implementation of the share interface. It is an object that accesses the share information 
 * in the database and can modify it :was well. It allows you to add recipients, get the photos in a share,
 * add more photos/albums in a share.
 * @author Simon King
 */
public class ShareImpl implements IShare {
	private int m_id;
	private int m_ownerID;
	private String m_message;
	private Date m_createDate;
	
	protected ShareImpl (int id) throws ObjectNotFoundException {
		this.m_id=id;
		String query = "select created_by, date_added, message from share where id="+this.m_id;
		ResultSetWrapper rsw = RDBConnection.executeQuery(query);
		ResultSet rs = rsw.getResultSet();
		try {
			if (rs.next()) {
				this.m_message= StringUtils.URLDecode(rs.getString("message") );
				this.m_ownerID=rs.getInt("created_by");
				this.m_createDate=rs.getDate("date_added");
			} else {
				throw new ObjectNotFoundException("Unable to find album(id)="+id);
			}
		} catch (SQLException e) {
			e.printStackTrace();
			throw new ObjectNotFoundException("Unable to find album(id)="+id);
		} finally {
			rsw.closeAll();
		}
	}

	/* (non-Javadoc)
	 * @see gcr.mmm2.model.IShare#getID()
	 */
	public int getID() {
		return this.m_id;
	}

	/* (non-Javadoc)
	 * @see gcr.mmm2.model.IShare#getOwner()
	 */
	public IUser getOwner() {
		return UserManager.getByID(this.m_ownerID);
	}

	/* (non-Javadoc)
	 * @see gcr.mmm2.model.IShare#getMessage()
	 */
	public String getMessage() {
		return StringUtils.URLDecode(this.m_message);
	}
	
	/**
	 * @param colName
	 * @param newVal
	 */
	private void updateValue(String colName, String newVal) {
        String query = "update share set " + colName + "=? where id=?";
        Object[] params = { StringUtils.URLEncode(newVal), new Integer(m_id) };
        RDBConnection.executePrepared(query, params);
	}
	
	/**
	 * @param owner
	 * @return
	 */
	private boolean hasPermission(IUser owner){
		try {
			return (owner.getID()==this.m_ownerID || UserManager.isAdmin(owner));
		} catch (Exception e) {
			return false;
		}
	}

	/* (non-Javadoc)
	 * @see gcr.mmm2.model.IShare#setMessage(java.lang.String)
	 */
	public void setMessage(String message, IUser owner) throws PermissionException {
		if (!hasPermission(owner)){
			throw new PermissionException("User "+owner.getID()+" tried to modify share id "+this.m_id+" owned by user "+m_ownerID);
		}
		updateValue("message", message);
		this.m_message=StringUtils.URLEncode(message);
	}
	
	public void addRecipient(IContactList contactList, IUser owner) throws PermissionException {
		if (!hasPermission(owner)){
			throw new PermissionException("User "+owner.getID()+" tried to modify share id "+this.m_id+" owned by user "+this.m_ownerID);
		}
		Iterator i=contactList.listContacts().iterator();
		while (i.hasNext()) {
			IContact contact=(IContact)i.next();
			if (!containsRecipient(contact)) {
				int id=RDBConnection.getNextVal("share_recip_id_seq");
//				String query="insert into share_recipient (id, share_id, recipient, date_added, contact_list_id) values "+
//					"("+id+", "+this.m_id+", "+contact.getID()+", now(), "+contactList.getID()+")";
				String query="insert into share_recipient (id, share_id, recipient, date_added, contact_list_id) values "+
				"(?, ?, ?, now(), ?)";
				Object[] params = {new Integer(id), new Integer(this.m_id), new Integer(contact.getID() ), new Integer(contactList.getID() ) };
				RDBConnection.executePrepared(query, params);
			}
		}
	}
	
	//	 TODO -- what is going on here? -- think about contacts vs persons vs users
	public void addRecipient(IPerson person, IUser owner) throws PermissionException {
		if (!hasPermission(owner)){
			throw new PermissionException("User "+owner.getID()+" tried to modify share id "+this.m_id+" owned by user "+this.m_ownerID);
		}
		if (!containsRecipient(person)) {
			int id=RDBConnection.getNextVal("share_recip_id_seq");
//			String query="insert into share_recipient (id, share_id, recipient, date_added) values "+
//				"("+id+", "+this.m_id+", "+person.getID()+", now())";
			String query="insert into share_recipient (id, share_id, recipient, date_added) values "+
			"(?, ?, ?, now())";
			Object[] params = {new Integer(id), new Integer(this.m_id), new Integer(person.getID() ) };
			RDBConnection.executePrepared(query, params);
		}
	}

	/* (non-Javadoc)
	 * @see gcr.mmm2.model.IShare#listRecipients()
	 */
	// returns a list of person (they could be contacts or users...)
	// TODO separate out contact lists and non-list people...maybe?
	public List listRecipients() {
		String query = "select distinct recipient from share_recipient where "+
			"share_id="+m_id+" and date_inactive is null";
		//return PersonManager.listUsersOrContacts(query);
		return PersonManager.listUsersOrPersons(query);
	}

	/* (non-Javadoc)
	 * @see gcr.mmm2.model.IShare#addAlbum(gcr.mmm2.model.IAlbum)
	 */
	public void addAlbum(IAlbum album, IUser owner) throws PermissionException {
		if (!hasPermission(owner)){
			throw new PermissionException("User "+owner.getID()+" tried to modify share id "+m_id+" owned by user "+this.m_ownerID);
		}
		if (!containsAlbum(album)) {
			int id=RDBConnection.getNextVal("share_album_id_seq");
//			String query="insert into share_album (id, share_id, album_id, date_added) values "+
//				"("+id+", "+m_id+", "+album.getID()+", now())";
			String query="insert into share_album (id, share_id, album_id, date_added) values "+
			"(?, ?, ?, now())";
			Object[] params = {new Integer(id), new Integer(m_id), new Integer(album.getID() )};
			RDBConnection.executePrepared(query, params);
		}
	}

	/* (non-Javadoc)
	 * @see gcr.mmm2.model.IShare#addPhoto(gcr.mmm2.model.IPhoto)
	 */
	public void addPhoto(IPhoto photo, IUser owner) throws PermissionException {
		if (!hasPermission(owner)){
			throw new PermissionException("User "+owner.getID()+" tried to modify share id "+this.m_id+" owned by user "+this.m_ownerID);
		}
		if (!containsPhoto(photo)) {
			int id=RDBConnection.getNextVal("share_photo_id_seq");
//			String query="insert into share_photo (id, share_id, photo_id, date_added) values "+
//				"("+id+", "+this.m_id+", "+photo.getID()+", now())";
			String query="insert into share_photo (id, share_id, photo_id, date_added) values "+
			"(?, ?, ?, now())";
			Object[] params = {new Integer(id), new Integer(this.m_id), new Integer(photo.getID() )};
			RDBConnection.executePrepared(query, params);
		}
	}
	
	/**
	 * @param query
	 * @return
	 */
	private boolean contains(String query) {
		boolean rtnVal=false;
		ResultSetWrapper rsw = RDBConnection.executeQuery(query);
		ResultSet rs = rsw.getResultSet();
		try {
			if (rs.next()) {
				rtnVal=true;
			} else {
				rtnVal=false;
			}
		} catch (SQLException e) {
			e.printStackTrace();
			rtnVal=false;
		} finally {
			rsw.closeAll();
		}
		return rtnVal;
	}

	/* (non-Javadoc)
	 * @see gcr.mmm2.model.IShare#containsRecipient(gcr.mmm2.model.IPerson)
	 */
	public boolean containsRecipient(IPerson person) {
		String query = "select recipient from share_recipient where share_id="+m_id+
			" and recipient="+person.getID()+" and date_inactive is null";
		return contains(query);
	}
	
	/* (non-Javadoc)
	 * @see gcr.mmm2.model.IShare#containsPhoto(gcr.mmm2.model.IPhoto)
	 */
	public boolean containsPhoto(IPhoto photo) {
		String query="select photo_id from share_photo where share_id="+m_id+
			" and photo_id="+photo.getID()+" and date_inactive is null";
		return contains(query);
	}
	
	/* (non-Javadoc)
	 * @see gcr.mmm2.model.IShare#containsAlbum(gcr.mmm2.model.IAlbum)
	 */
	public boolean containsAlbum(IAlbum album){
		String query="select album_id from share_album where share_id="+m_id+
			" and album_id="+album.getID()+" and date_inactive is null";
		return contains(query);
	}
	
	/* (non-Javadoc)
	 * @see gcr.mmm2.model.IShare#listPhotos()
	 */
	public List listPhotos() {
		String query="select photo_id from share_photo where share_id="+m_id+" and date_inactive is null";
		return PhotoFactory.listPhotos(query);
	}
	
	/* (non-Javadoc)
	 * @see gcr.mmm2.model.IShare#listAlbums()
	 */
	public List listAlbums() {
		String query="select album_id from share_album where share_id="+m_id+" and date_inactive is null";
		return AlbumFactory.listAlbums(query);
	}
	
	/* (non-Javadoc)
	 * @see gcr.mmm2.model.IShare#notifyRecipients(boolean)
	 */
	public void notifyRecipients(boolean notifyMobile){
		
		EmailAddress senderEmail = getOwner().getPrimaryEmail();
		Iterator p_it;
		Iterator recipients;
		
		//Collect all photos, including album contents
		HashSet allPhotos = new HashSet();
		allPhotos.addAll(listPhotos());
		Iterator album_it = listAlbums().iterator();
		while(album_it.hasNext()){
			IAlbum cur_album = (IAlbum)album_it.next();
			allPhotos.addAll(cur_album.listPhotos());
		}
		
		if(this.m_message==null) this.m_message = "";
		
		if(notifyMobile){
			StringBuffer mobileMessage = new StringBuffer();
			mobileMessage.append(getOwner().toString()+" says: "+StringUtils.showSafeHTML(this.m_message)+" ");
			mobileMessage.append("I've shared with you. ");
			mobileMessage.append("Log on to mmm2!");
			//URL in message seems to cause message delivery failure
			//it's probably too long...  (>160chars)
			//mobileMessage.append("Go to http://red.sims.berkeley.edu/mmm2v1 for details.");
			
			
			//We're not sending SMSs to non-Users.
			recipients = listRecipients().iterator();
			while(recipients.hasNext()){
				IPerson cur_recipient = (IPerson)recipients.next();
				
				EmailAddress recipEmail = null;
				
				if(cur_recipient instanceof IUser) {//TODO: fix NPexecption
					System.out.println("Sending sms notification to: " + cur_recipient);
					IUser u = (IUser)cur_recipient;
					if(u.getSmsNotify())
						recipEmail = u.getSmsEmail();
				}
				
				//TODO: notify the person's contact of this IPerson
				if(recipEmail != null && senderEmail != null){
					System.out.println("sending message from "+senderEmail.getAddress() +" to " + recipEmail.getAddress());
					NotificationMailer.sendTextMsg(
							senderEmail.getAddress(), recipEmail.getAddress(), 
							"MMM2 Share Notification", mobileMessage.toString());
				}
			}
		}
		
		recipients = listRecipients().iterator();
		while(recipients.hasNext()){
			IPerson cur_recipient = (IPerson)recipients.next();
			System.out.println("Sending notification to: " + cur_recipient);
			String identifier = "sid"+this.m_id+"_pid"+cur_recipient.getID();
			EmailAddress recipEmail = null;
			if(cur_recipient instanceof IUser){//TODO: fix NP execption
				recipEmail = ((IUser)cur_recipient).getPrimaryEmail();
			}else if(cur_recipient instanceof IPerson){
				IContact c = ContactManager.getContactByPerson(cur_recipient, this.getOwner());
				if(c != null)
					recipEmail = c.getPrimaryEmail();
			}
			
			StringBuffer textPart = new StringBuffer();
			StringBuffer htmlPart = new StringBuffer();
			
			
			String shortName = getOwner().toString();
			if(shortName.indexOf(" ")>0)
			{
				shortName = shortName.substring(0,shortName.indexOf(" "));
			}
			htmlPart.append("<html>\n<body>\n");
			htmlPart.append(shortName+" says:<br>\n"+StringUtils.showSafeHTML(this.m_message)+"<br>\n<br>\n");
			htmlPart.append("In other words, you've got pictures!<br>\n<br>\n");
			htmlPart.append("<div style='clear:both;'>");
			
			textPart.append(getOwner().toString()+" says:\n"+this.m_message+"\n\n");
			textPart.append("In other words, you've got pictures!\n\n");
			
			String serverURL = MMM2Properties.getProperty("MMM_URL");
			
			p_it = allPhotos.iterator();
			while(p_it.hasNext()){
				IPhoto cur_photo = (IPhoto)p_it.next();
				htmlPart.append("<div style='position:relative;float:left;margin:5px;width:210px;'>\n");
				if(cur_recipient instanceof IUser){
					htmlPart.append("<a href='"+serverURL+"/photo_details.jsp?photoID="+cur_photo.getID()+"&shareIdentifier="+identifier+"'>");
				}else{
					htmlPart.append("<a href='"+cur_photo.getURL(serverURL, "html_email_"+identifier, 640)+"'>");
				}
				htmlPart.append("<img src='"+cur_photo.getURL(serverURL, "html_email_thumb_"+identifier, 200)+"' style='border-color:black;border-width:1px;border-style:solid;' >");
				htmlPart.append("</a><br>\n");
				htmlPart.append(cur_photo.getCaption());
				htmlPart.append("</div>\n");
				
				if(cur_recipient instanceof IUser){
					textPart.append(serverURL+"/photo_details.jsp?photoID="+cur_photo.getID()+"&shareIdentifier="+identifier);
				}else{
					textPart.append(cur_photo.getURL(serverURL, "text_email_"+identifier, 640)+"\n");
				}
				
			}
			htmlPart.append("</div>");
			htmlPart.append("<div style='clear:both;'><hr>This message brought to you by... <a href='"+serverURL+"'>MMM2</a></div>\n");
			htmlPart.append("</body>\n</html>");
			
			textPart.append("\n\n---------------------------------------------------------\n");
			textPart.append("This message brought to you by...MMM!");
			
			if(recipEmail != null && senderEmail != null){
				NotificationMailer.sendHtmlMsg(
						senderEmail.getAddress(), recipEmail.getAddress(), 
						"MMM2 Share from " + getOwner().toString(), textPart.toString(), htmlPart.toString());
			}
		}
	}
}
