/*
 * Created on Aug 11, 2004
 *
 */
package gcr.mmm2.model;

import gcr.mmm2.rdb.RDBConnection;
import gcr.mmm2.rdb.ResultSetWrapper;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Simon King
 *
 */
public class ShareManager {
	public static IShare createShare(IUser initiator) {
		int id=RDBConnection.getNextVal("share_id_seq");
		try {
			String query="insert into share (id, created_by, date_added) "+
				"values ("+id+", "+initiator.getID()+", now())";
			RDBConnection.execute(query);
			return new ShareImpl(id);
		} catch (ObjectNotFoundException e) {
			e.printStackTrace();
			return null;
		}
	}
	
	public static IShare getShareByID(int id) {
		try {
			return new ShareImpl(id);
		} catch (ObjectNotFoundException e) {
			e.printStackTrace();
			return null;
		}
	}
	
	protected static List listShares(String query) {
		ResultSetWrapper rsw = RDBConnection.executeQuery(query);
		ResultSet rs = rsw.getResultSet();
		ArrayList l = new ArrayList();
		try {
			while (rs.next()) {
				int id = rs.getInt(1);
				IShare s=getShareByID(id);
				if (s !=null) {
					l.add(s);
				}
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			rsw.closeAll();
		}
		return l;
	}
	
	// TODO add ordering to this -- either by time or by sending user
	// maybe even allow some filtering?
	public static List getReceivedShares(IUser recipient) {
		String query="select distinct share_id from share_recipient where "+
			"recipient="+recipient.getID()+" and date_inactive is null";
		return listShares(query);
	}
	
	public static List getReceivedPhotos(IUser recipient) {
		String query="select sp.photo_id from share_recipient sr, share_photo sp "+ 
			"where sr.share_id=sp.share_id and sr.recipient="+recipient.getID()+" and sr.date_inactive is null"+
			" union select ac.photo_id from share_recipient sr, share_album sa, "+
			"album_contents ac where sr.share_id=sa.share_id and ac.album_id=sa.album_id "+ 
			"and sr.recipient="+recipient.getID()+" and sr.date_inactive is null";
		return PhotoFactory.listPhotos(query);
	}
	
	// TODO -- separate out contact lists and non-list people
	// returns list of IPerson (either IContact or IUser)
	public static List getPhotoRecipients(IPhoto photo, IUser initiator){
		String query="select sr.recipient from share s, share_recipient sr, share_photo sp "+
			"where s.id=sr.share_id and s.id=sp.share_id and s.created_by="+initiator.getID()+
			"and sp.photo_id="+photo.getID()+" and sr.date_inactive is null union select sr.recipient "+
			"from share s, share_recipient sr, share_album sa, album_contents ac "+
			"where s.id=sr.share_id and s.id=sa.share_id and ac.album_id=sa.album_id "+ 
			"and s.created_by="+initiator.getID()+" and ac.photo_id="+photo.getID()+" and sr.date_inactive is null";
		//return PersonManager.listUsersOrContacts(query);
		return PersonManager.listUsersOrPersons(query);
	}
	
	public static List listNewestReceivedPhotos(int n, IUser recipient) {
		String query="select photo_id, max(date_added) as date_added from "+
			"(select sp.photo_id, sr.date_added from share_recipient sr, share_photo sp "+
			"where sr.share_id=sp.share_id and sr.recipient="+recipient.getID()+" and sr.date_inactive is null "+
			"union select ac.photo_id, sr.date_added from share_recipient sr, share_album sa, "+ 
			"album_contents ac where sr.share_id=sa.share_id and ac.album_id=sa.album_id "+ 
			"and sr.recipient="+recipient.getID()+" and sr.date_inactive is null) q group by photo_id order by date_added desc limit "+n;
		return PhotoFactory.listPhotos(query);
	}
	
	
	// TODO -- sorting
	public static List listUsersSharingTo(IUser recipient) {
		String query="select distinct s.created_by from share s, share_recipient sr, mmm_user u "+
        "where s.id=sr.share_id "+
			"and sr.date_inactive is null "+
            "and sr.recipient="+recipient.getID()+ " " +
            "and s.created_by=u.id "+
            "and u.date_inactive is null ";
		return UserManager.listUsers(query);
	}
	
	public static List listSharedAlbums(IUser sender, IUser recipient) {
		return listSharedAlbums(sender, recipient, MediaManager.SORT_DATE_DESC);
	}
	
	
	public static List listSharedAlbums(IUser sender, IUser recipient, int sortOrder) {
//		String query="select distinct sa.album_id from share s, share_recipient sr, share_album sa where s.id=sr.share_id "+
//			"and s.id=sa.share_id and sr.date_inactive is null and sa.date_inactive is null "+
//			"and sr.recipient="+recipient.getID()+" and s.created_by="+sender.getID();
		String query="as the_date from share s, share_recipient sr, share_album sa where s.id=sr.share_id "+ 
			"and s.id=sa.share_id and sr.date_inactive is null and sa.date_inactive is null "+ 
			"and sr.recipient="+recipient.getID()+" and s.created_by="+sender.getID();
		if (sortOrder==MediaManager.SORT_DATE_DESC) {
			query="select sa.album_id as the_id, max(sr.date_added) "+query+" group by the_id order by the_date desc";
		} else if (sortOrder==MediaManager.SORT_DATE_ASC) {
			query="select sa.album_id as the_id, min(sr.date_added) "+query+" group by the_id order by the_date asc";
		} 
		return AlbumFactory.listAlbums(query);
	}
	
	public static List listSharedPhotos(IUser sender, IUser recipient) {
		return listSharedPhotos(sender, recipient, MediaManager.SORT_DATE_DESC);
	}
	
	// directly shared photos albums -- not photos in shared albums -- is this right?
	public static List listSharedPhotos(IUser sender, IUser recipient, int sortOrder) {
//		String query="select distinct sp.photo_id from share s, share_recipient sr, share_photo sp where s.id=sr.share_id "+
//			"and s.id=sp.share_id and sr.date_inactive is null and sp.date_inactive is null "+
//			"and sr.recipient="+recipient.getID()+" and s.created_by="+sender.getID();
		String query="as the_date from share s, share_recipient sr, share_photo sp where s.id=sr.share_id "+
		"and s.id=sp.share_id and sr.date_inactive is null and sp.date_inactive is null "+
		"and sr.recipient="+recipient.getID()+" and s.created_by="+sender.getID();
		if (sortOrder==MediaManager.SORT_DATE_DESC) {
			query="select sp.photo_id as the_id, max(sr.date_added) "+query+" group by the_id order by the_date desc";
		} else if (sortOrder==MediaManager.SORT_DATE_ASC) {
			query="select sp.photo_id as the_id, min(sr.date_added) "+query+" group by the_id order by the_date asc";
		}
		return AlbumFactory.listAlbums(query);
	}
	
	public static List listUsersSharingMedia(IAlbum album, IUser recipient) {
		String query="select distinct s.created_by from share s, share_recipient sr, share_album sa where s.id=sr.share_id "+ 
			"and s.id=sa.share_id and sr.date_inactive is null and sa.date_inactive is null "+ 
			"and sr.recipient="+recipient.getID()+" and sa.album_id="+album.getID();
		return UserManager.listUsers(query);
	}
	
	public static List listUsersSharingMedia(IPhoto photo, IUser recipient) {
		String query="select distinct s.created_by from share s, share_recipient sr, share_photo sp where s.id=sr.share_id "+ 
			"and s.id=sp.share_id and sr.date_inactive is null and sp.date_inactive is null "+ 
			"and sr.recipient="+recipient.getID()+" and sp.photo_id="+photo.getID();
		return UserManager.listUsers(query);
	}
	
	public static List listSharesContainingMedia(IPhoto photo, IUser recipient){
		String query="select distinct s.id from share s, share_recipient sr, share_photo sp where s.id=sr.share_id "+ 
			"and s.id=sp.share_id and sr.date_inactive is null and sp.date_inactive is null "+ 
			"and sr.recipient="+recipient.getID()+" and sp.photo_id="+photo.getID();
		return ShareManager.listShares(query);
	}
	
	// returns a list of IPerson (should be either active contact or active user)
	public static List listFrequentShareRecipients(IUser sender) {
		String query="select sr.recipient, count(*) as c from share s, share_recipient sr "+
			"where s.id=sr.share_id and s.created_by="+sender.getID()+" and recipient in "+
			"(select c.person_id as id from contact c where c.created_by="+sender.getID()+" and c.date_inactive is null "+
			"union select mu.id as id from mmm_user mu where mu.date_inactive is null) group by recipient order by c desc";
		return PersonManager.listUsersOrPersons(query);
	}
	
	public static List getMobileCarriers() {
		String carrierId=null;
		String label=null;
		ArrayList l = new ArrayList();
		String query = "select id, long_label from mobile_carriers";
		ResultSetWrapper rsw = RDBConnection.executeQuery(query);
		ResultSet rs = rsw.getResultSet();
		try {
			while(rs.next()) {
				carrierId=rs.getString("id");
				label=rs.getString("long_label");
				String[] s = {carrierId, label};
				l.add(s);
			} 
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			rsw.closeAll();
		}
		return l;
	}
	
}
