package gcr.mmm2.model;

import gcr.mmm2.util.*;

import java.io.Serializable;

public class UIState implements Serializable
{
    
    // information about the photo sizes
    static final TwoSideBox[] SIZES = {new TwoSideBox(100,75),
                new TwoSideBox(150,112),
                new TwoSideBox(200,150)};
    
    public static final int SMALL_THUMB = 0;
    public static final int MEDIUM_THUMB = 1;
    public static final int LARGE_THUMB = 2;
    
    //View constants for what types of photos should be shown
    public static final int ALL_VISIBLE = MediaManager.ALL_VISIBLE;
    public static final int OWNED_PHOTOS = MediaManager.OWNED_PHOTOS;
    public static final int RECEIVED_PHOTOS = MediaManager.RECEIVED_PHOTOS;
    public static final int PUBLIC_PHOTOS = MediaManager.PUBLIC_PHOTOS;
    public static final int UNRESTRICTED_OWNED_PHOTOS = MediaManager.UNRESTRICTED_OWNED_PHOTOS;
    
    //View constants for what time range to show
    public static final int NEWEST = MediaManager.NEWEST;
    public static final int DAY = MediaManager.TODAY;
    public static final int WEEK = MediaManager.PREV_7;
    public static final int TWO_WEEK = MediaManager.PREV_14;
    public static final int MONTH = MediaManager.LAST_MONTH;
    public static final int YEAR = MediaManager.THIS_YEAR;
    public static final int ALL = MediaManager.ALL;
    public static final int SHOEBOX = MediaManager.SHOEBOX;
    public static final int ALBUM = 100;
    
    
    private int thumbSize = MEDIUM_THUMB;
    private boolean showDetails = true;
    private int timeRange = NEWEST;
    transient private int ownerLimit = OWNED_PHOTOS;
    transient private int albumID = -1;
    transient private String network = "AT&T+Wirel";
    transient private String lac = "";
    transient private String cellID = "";
    transient private int userID = -1;
    transient private int page = 1;
    private int pageSize = 50000;
    private boolean cropThumbs = true;
    transient private String bt_mac="";
    
    public void initTransients()
    {
        timeRange = NEWEST;
        ownerLimit = OWNED_PHOTOS;
        albumID = -1;
        network = "AT&T+Wirel";
        lac = "";
        cellID = "";
        userID = -1;
        page = 1;
        bt_mac="";
        // remove the following line once paging is working
        pageSize = 50000;
    }
    /**
     * @return Returns the albumID.
     */
    public int getAlbumID()
    {
        return albumID;
    }


    /**
     * @param albumID The albumID to set.
     */
    public void setAlbumID(int albumID)
    {
        this.albumID = albumID;
    }


    /**
     * Get the thumbnail TwoSideBox object for specified thumb size
     * @param theSizeEnum
     * @return
     */
    public static TwoSideBox getThumbSize(int theSizeEnum)
    {
        if(theSizeEnum>=SIZES.length||theSizeEnum<0)
            return null;
        return SIZES[theSizeEnum];
    }
    
    
    /**
     * Gives two side box object for current thumbsize
     * @return
     */
    public TwoSideBox getCurrentThumbSize()
    {
        if(thumbSize>=SIZES.length||thumbSize<0)
            return null;
        return SIZES[thumbSize];
    }

    /**
     * @return Returns the showDetails.
     */
    public boolean isShowDetails()
    {
        return showDetails;
    }


    /**
     * @param showDetails The showDetails to set.
     */
    public void setShowDetails(boolean showDetails)
    {
        this.showDetails = showDetails;
    }


    /**
     * @return Returns the thumbSize.
     */
    public int getThumbSize()
    {
        return thumbSize;
    }


    /**
     * @param thumbSize The thumbSize to set.
     */
    public void setThumbSize(int thumbSize)
    {
        this.thumbSize = thumbSize;
    }


    /**
     * @return Returns the timeRange.
     */
    public int getTimeRange()
    {
        return timeRange;
    }


    /**
     * @param timeRange The timeRange to set.
     */
    public void setTimeRange(int timeRange)
    {
        this.timeRange = timeRange;
    }


    /**
     * get if limited by owner
     * @return Returns the ownerLimit.
     */
    public int getOwnerLimit()
    {
        return ownerLimit;
    }


    /**
     * Limit by owner
     * @param ownerLimit The ownerLimit to set.
     */
    public void setOwnerLimit(int ownerLimit)
    {
        this.ownerLimit = ownerLimit;
    }


    /**
     * @return Returns the cellID.
     */
    public String getCellID() {
        return cellID;
    }


    /**
     * @param cellID The cellID to set.
     */
    public void setCellID(String cellID) {
        this.cellID = cellID;
    }


    /**
     * @return Returns the lac.
     */
    public String getLac() {
        return lac;
    }


    /**
     * @param lac The lac to set.
     */
    public void setLac(String lac) {
        this.lac = lac;
    }


    /**
     * @return Returns the network.
     */
    public String getNetwork() {
        return network;
    }


    /**
     * @param network The network to set.
     */
    public void setNetwork(String network) {
        this.network = network;
    }
    
    /**
     * @return Returns the bt_mac.
     */
    public String getBt_mac() {
        return bt_mac;
    }


    /**
     * @param bt_mac to set.
     */
    public void setBt_mac(String bt_mac) {
        this.bt_mac = bt_mac;
    }



    /**
     * @return Returns the cropThumbs.
     */
    public boolean isCropThumbs() {
        return cropThumbs;
    }


    /**
     * @param cropThumbs The cropThumbs to set.
     */
    public void setCropThumbs(boolean cropThumbs) {
        this.cropThumbs = cropThumbs;
    }


    /**
     * @return Returns the userID.
     */
    public int getUserID() {
        return userID;
    }


    /**
     * @param userID The userID to set.
     */
    public void setUserID(int userID) {
        this.userID = userID;
    }
    /**
     * 
     */
    public UIState() {
        super();
        // TODO Auto-generated constructor stub
        initTransients();
    }
    /**
     * @return Returns the page.
     */
    public int getPage() {
        return page;
    }
    /**
     * @param page The page to set.
     */
    public void setPage(int page) {
        this.page = page;
    }
    /**
     * @return Returns the pageSize.
     */
    public int getPageSize() {
        return pageSize;
    }
    /**
     * @param pageSize The pageSize to set.
     */
    public void setPageSize(int pageSize) {
        this.pageSize = pageSize;
    }
    
    
}
