/*
 * Created on Aug 10, 2004
 *
 */
package gcr.mmm2.model;

import gcr.mmm2.rdb.RDBConnection;
import gcr.mmm2.rdb.ResultSetWrapper;
import gcr.mmm2.util.StringUtils;

import java.io.File;
import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

/**
 * @author Simon King
 * 
 */
public class UserImpl implements IUser {

    private int m_id;

    private String m_imageFile;

    private String m_mobileCarrier;

    private String m_password;

    private IPerson m_person;

    private String m_phoneID;

    private String m_phoneNumber;

    private String m_username;

    protected UserImpl(int id) throws ObjectNotFoundException {
        this.m_id = id;
        String query = "select username, password, phone_imsi, phone_num, headshot_image_file "
                + "from mmm_user u left join person_phone p on u.id=p.person_id where u.date_inactive is null and u.id="
                + this.m_id;
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        try {
            if (rs.next()) {
                this.m_username = StringUtils.URLDecode(rs.getString("username") );
                this.m_person = new PersonImpl(this.m_id);
                this.m_password = rs.getString("password");
                this.m_phoneID = rs.getString("phone_imsi");
                this.m_phoneNumber = rs.getString("phone_num");
                this.m_imageFile = rs.getString("headshot_image_file");
            } else {
                throw new ObjectNotFoundException(
                        "Unable to find user where id=" + id);
            }
        } catch (SQLException e) {
            e.printStackTrace();
            throw new ObjectNotFoundException("Unable to initialize user where id="
                    + id, e);
        } finally {
            rsw.closeAll();
        }
    }

    protected UserImpl(String username) throws ObjectNotFoundException {
        this.m_username = username;
        String query = "select u.id, password, phone_imsi, phone_num, mobile_carrier, headshot_image_file "
                + "from mmm_user u left join person_phone p on u.id=p.person_id where u.date_inactive is null and username='"
                + this.m_username + "'";
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        try {
            if (rs.next()) {
                this.m_id = rs.getInt("id");
                this.m_person = new PersonImpl(this.m_id);
                this.m_password = rs.getString("password");
                this.m_phoneID = rs.getString("phone_imsi");
                this.m_phoneNumber = rs.getString("phone_num");
                this.m_mobileCarrier = rs.getString("mobile_carrier");
                this.m_imageFile = rs.getString("headshot_image_file");
            } else {
                throw new ObjectNotFoundException("Unable to find username="
                        + this.m_username);
            }
        } catch (SQLException e) {
            e.printStackTrace();
            throw new ObjectNotFoundException("Unable to find username="
                    + this.m_username, e);
        } finally {
            rsw.closeAll();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IUser#addBlog(java.lang.String, java.lang.String,
     *      java.lang.String, java.lang.String, java.lang.String, int)
     */
    public void addBlog(String title, String login, String password,
            String xmlrpc_url, String blog_id, int blog_type, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify user id " + this.m_id);
        }
        if (title != null && login != null && password != null
                && blog_type > -1) {
            int id = RDBConnection.getNextVal("user_blog_id_seq");
            String query = "insert into user_blog (id, user_id, "
                    + "blog_title, blog_login, blog_password, "
                    + "blog_id, blog_type, xmlrpc_url, created_by, date_added) "
                    + "values (" + id + ", " + this.m_id + ", '" + title
                    + "', '" + login + "', '" + password + "', '" + blog_id
                    + "', " + blog_type + ", '" + xmlrpc_url + "', "
                    + owner.getID() + ", now())";

            RDBConnection.execute(query);
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IUser#addBluetoothID(java.lang.String)
     */
    public void addBluetoothMAC(String btMAC, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify user id " + this.m_id);
        }
        PersonManager.associateBTDevice(this.m_person, btMAC, owner);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IContact#addEmail(java.lang.String)
     */
    public void addEmail(String email, IUser owner) throws PermissionException,
            StringFormatException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify user id " + this.m_id);
        }
        StringUtils.validateEmail(email);
        System.out.println("validated: email");
        if (email != null && !hasEmail(email)) {
            int id = RDBConnection.getNextVal("person_email_id_seq");
            String query = "insert into person_email (id, person_id, email_address, is_primary, created_by, date_added) "
                    + "values ("
                    + id
                    + ", "
                    + this.m_id
                    + ", '"
                    + email
                    + "', FALSE, " + owner.getID() + ", now())";
            System.out.println("Insert email query: " + query);
            RDBConnection.execute(query);
            // now gotta deal with setting primary email
            if (getPrimaryEmail() == null) {
                query = "update person_email set is_primary=TRUE where person_id="
                        + this.m_id
                        + " and created_by="
                        + owner.getID()
                        + " and email_address='"
                        + email
                        + "' and date_inactive is null";
                System.out.println("set primary email query: " + query);
                RDBConnection.execute(query);
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IUser#changePassword(java.lang.String,
     *      java.lang.String)
     */
    public boolean changePassword(String oldPass, String newPass, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify user id " + this.m_id);
        }
        if (checkPassword(oldPass)) {
            updateValue("password", newPass);
            this.m_password = newPass;
            return true;
        }
        return false;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IUser#checkPassword()
     */
    public boolean checkPassword(String testPassword) {
        return ((testPassword != null) && (testPassword.equals(this.m_password)));
    }

    public boolean getEmailNotify() {
        boolean rtnVal = false;
        String query = "select email_notify from person_preferences where person_id="
                + this.m_id;
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        try {
            if (rs.next()) {
                rtnVal = rs.getBoolean("email_notify");
            } else {
                rtnVal = false;
            }
        } catch (SQLException e) {
            e.printStackTrace();
            rtnVal = false;
        } finally {
            rsw.closeAll();
        }
        return rtnVal;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPerson#getFirstName()
     */
    public String getFirstName() {
        return StringUtils.URLDecode(this.m_person.getFirstName() );
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IUser#getHeadshotImagePath()
     */
    public String getHeadshotImagePath() {
        return m_imageFile;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPerson#getID()
     */
    public int getID() {
        return this.m_id;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPerson#getLastName()
     */
    public String getLastName() {
        return StringUtils.URLDecode(this.m_person.getLastName() );
    }

    public String getMobileCarrier() {
        return m_mobileCarrier;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IUser#getPhoneID()
     */
    public String getPhoneID() {
        return m_phoneID;
    }

    public String getPhoneNumber() {
        try {
            return StringUtils.formatPhoneNumber(this.m_phoneNumber);
        } catch (StringFormatException e) {
            e.printStackTrace();
            return "";
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IContact#getPrimaryEmail()
     */
    public EmailAddress getPrimaryEmail() {
        String query = "select id, email_address from person_email where person_id="
                + this.m_id + " and date_inactive is null and is_primary=TRUE";
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        EmailAddress email = null;
        try {
            if (rs.next()) {
                String addr = rs.getString("email_address");
                int id = rs.getInt("id");
                email = new EmailAddress(id, addr, this);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return email;
    }

    public String getRDFNameSpace() {
        String namespace = "";
        String query = "select rdf_namespace from person_preferences a, rdf_namespaces b where a.rdf_namespace_id = b.id and a.person_id="
                + this.m_id;
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        try {
            if (rs.next()) {
                namespace = rs.getString("rdf_namespace");
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return namespace;
    }

    public int getRDFNameSpaceID() {
        int namespace = 0;
        String query = "select rdf_namespace_id from person_preferences a, rdf_namespaces b where a.rdf_namespace_id = b.id and a.person_id="
                + this.m_id;
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        try {
            if (rs.next()) {
                namespace = rs.getInt("rdf_namespace_id");
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return namespace;
    }

    public EmailAddress getSmsEmail() {
        String query = "select phone_num, sms_domain from person_phone p left join mobile_carriers m "
                + "on p.mobile_carrier=m.id where p.person_id=" + this.m_id;
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        EmailAddress email = null;
        try {
            if (rs.next()) {
                String phone_num = rs.getString("phone_num");
                String sms_domain = rs.getString("sms_domain");
                String addr = phone_num + sms_domain;
                email = new EmailAddress(this.m_id, addr, this);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return email;
    }

    public boolean getSmsNotify() {
        boolean rtnVal = false;
        String query = "select sms_notify from person_preferences where person_id="
                + this.m_id;
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        try {
            if (rs.next()) {
                rtnVal = rs.getBoolean("sms_notify");
            } else {
                rtnVal = false;
            }
        } catch (SQLException e) {
            e.printStackTrace();
            rtnVal = false;
        } finally {
            rsw.closeAll();
        }
        return rtnVal;
    }

    public String getUploadDirectory() throws IOException {
        String dir = MediaManager.ORIG_IMAGE_DIR + "/" + this.getID();
        File dirFile = new File(dir);
        if (dirFile.exists()) {
            if (!dirFile.isDirectory()) {
                throw new IOException("Fatal error in getUploadDirectory: '" + dir
                        + "' is a file, not a directory.");
            }
        } else {
            if (!dirFile.mkdirs()) {
                throw new IOException("Unable to create directory '" + dir
                        + "'");
            }
        }
        return dir;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPerson#getURI()
     */
    public String getURI() {
        return this.m_person.getURI();
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IUser#getUsername()
     */
    public String getUsername() {
        return this.m_username;
    }

    private boolean hasEmail(int id) {
        boolean rtnVal = false;
        String query = "select person_id from person_email where person_id="
                + this.m_id + " and id=" + id;
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        try {
            if (rs.next()) {
                rtnVal = true;
            } else {
                rtnVal = false;
            }
        } catch (SQLException e) {
            e.printStackTrace();
            rtnVal = false;
        } finally {
            rsw.closeAll();
        }
        return rtnVal;
    }

    private boolean hasEmail(String email) {
        boolean rtnVal = false;
        if (email == null) {
            return false;
        }
        String query = "select person_id from person_email where person_id="
                + this.m_id + " and upper(email_address) like '"
                + email.toUpperCase() + "%' and date_inactive is null";
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        try {
            if (rs.next()) {
                rtnVal = true;
            } else {
                rtnVal = false;
            }
        } catch (SQLException e) {
            e.printStackTrace();
            rtnVal = false;
        } finally {
            rsw.closeAll();
        }
        return rtnVal;
    }

    private boolean hasPermission(IUser owner) {
        try {
            return (owner.getID() == this.m_id || UserManager.isAdmin(owner));
        } catch (Exception e) {
            return false;
        }
    }

    public List listAlbums() {
        return AlbumFactory.listAlbums(this);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IUser#listBlogs()
     */
    public List listBlogs() {
        String query = "select id from user_blog where user_id=" + this.m_id
                + " and date_inactive is null";
        return UserManager.listBlogs(query);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IUser#listBlueToothIDs()
     */
    public List listBluetoothDevices() {
        // String query="select bt_mac, person_id from person_bluetooth where
        // person_id="+this.m_idtagme+" and date_inactive is null";
        return PersonManager.listBTDevices(this.m_person);
    }

    public List listEmails() {
        String query = "select id, email_address, person_id from person_email where person_id="
                + this.m_id + " and date_inactive is null";
        return PersonManager.listEmails(query);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IUser#removeBlog(int, gcr.mmm2.model.IUser)
     */
    public void removeBlog(int id, IUser owner) throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify user id " + this.m_id);
        }
        String query = "update user_blog set date_inactive=now() where id="
                + id;
        RDBConnection.execute(query);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IUser#removeBluetoothID(java.lang.String)
     */
    public void removeBluetoothMAC(String btMAC, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify user id " + this.m_id);
        }
        PersonManager.removeBluetoothMAC(this.m_person, btMAC);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IContact#removeEmail(java.lang.String)
     */
    public void removeEmail(int id, IUser owner) throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify user id " + this.m_id);
        }
        if (hasEmail(id)) {
            String query = "update person_email set date_inactive=now() where id="
                    + id;
            RDBConnection.execute(query);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IUser#resetPassword()
     */
    public void resetPassword(IUser owner) throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify user id " + this.m_id);
        }
        // TODO make this more clever?
        updateValue("password", this.m_username);
        this.m_password = this.m_username;
    }

    public void setEmailNotify(boolean setting) {
        boolean update = false;
        String query = "select email_notify from person_preferences where person_id="
                + this.m_id;
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        try {
            if (rs.next()) {
                update = true;
            } else {
                update = false;
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }

        if (update) {
            query = "update person_preferences set email_notify=" + setting
                    + " where person_id=" + this.m_id;
        } else {
            int id = RDBConnection.getNextVal("person_preferences_id_seq");
            query = "insert into person_preferences (id, person_id, email_notify) values("
                    + id + ", " + this.m_id + ", " + setting + ")";
        }
        RDBConnection.execute(query);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPerson#setFirstName(java.lang.String)
     */
    public void setFirstName(String firstName) throws PermissionException {
        // PersonManager.setFirstName(this, firstName);
        this.m_person.setFirstName(firstName);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IUser#setHeadshotImagePath(java.lang.String)
     */
    public void setHeadshotImagePath(String imageFilename, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify user id " + this.m_id);
        }
        updateValue("headshot_image_file", imageFilename);
        m_imageFile = imageFilename;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IPerson#setLastName(java.lang.String)
     */
    public void setLastName(String lastName) throws PermissionException {
        // PersonManager.setLastName(this, lastName);
        this.m_person.setLastName(lastName);
    }

    public void setMobileCarrier(String mobileCarrier, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify user id " + this.m_id);
        }
        // TODO: is this affected at all by the fact that the mobile_carrier
        // field should be an int???
        String query = "update person_phone set mobile_carrier=? where person_id=?";
        Object[] params = { mobileCarrier, new Integer(this.m_id) };
        RDBConnection.executePrepared(query, params);
        m_mobileCarrier = mobileCarrier;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IUser#setPhoneID()
     */
    public void setPhoneID(String phoneID, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify user id " + this.m_id);
        }
        
        String query1 = "update person_phone set phone_imsi=null where phone_imsi=?";
        Object[] params1 = { phoneID };
        RDBConnection.executePrepared(query1, params1);
        
        
        String query2 = "update person_phone set phone_imsi=? where person_id=?";
        Object[] params2 = { phoneID, new Integer(this.m_id) };
        RDBConnection.executePrepared(query2, params2);
        m_phoneID = phoneID;
    }

    public void setPhoneNumber(String phoneNumber, IUser owner)
            throws PermissionException, StringFormatException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify user id " + this.m_id);
        }
        String digits = StringUtils.extractPhoneDigits(phoneNumber);
        // updateValue("phone_number", digits);
        // TODO deal someday with multiple phone #s
        String query = "update person_phone set phone_num=? where person_id=?";
        Object[] params = { digits, new Integer(this.m_id) };
        RDBConnection.executePrepared(query, params);
        m_phoneNumber = digits;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.IContact#setPrimaryEmail(java.lang.String)
     */
    public void setPrimaryEmail(int id, IUser owner) throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify user id " + this.m_id);
        }
        if (hasEmail(id)) {
            String query = "update person_email set is_primary=FALSE where person_id="
                    + this.m_id
                    + "and id!="
                    + id
                    + " and date_inactive is null";
            RDBConnection.execute(query);
            query = "update person_email set is_primary=TURE where person_id="
                    + this.m_id + "and id=" + id + " and date_inactive is null";
            RDBConnection.execute(query);
        }
    }

    public void setRDFNameSpaceID(int namespace_id, IUser owner)
            throws PermissionException {
        if (!hasPermission(owner)) {
            throw new PermissionException("User " + owner.getID()
                    + " tried to modify user id " + this.m_id);
        }

        boolean update = false;
        String query = "select rdf_namespace_id from person_preferences where person_id="
                + this.m_id;
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        try {
            if (rs.next()) {
                update = true;
            } else {
                update = false;
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }

        if (update) {
            query = "update person_preferences set rdf_namespace_id="
                    + namespace_id + " where person_id=" + this.m_id;
        } else {
            int id = RDBConnection.getNextVal("person_preferences_id_seq");
            query = "insert into person_preferences (id, person_id, rdf_namespace_id) values("
                    + id + ", " + this.m_id + ", " + namespace_id + ")";
        }
        RDBConnection.execute(query);
    }

    public void setSmsNotify(boolean setting) {
        boolean update = false;
        String query = "select sms_notify from person_preferences where person_id="
                + this.m_id;
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        try {
            if (rs.next()) {
                update = true;
            } else {
                update = false;
            }

        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }

        if (update) {
            query = "update person_preferences set sms_notify=" + setting
                    + " where person_id=" + this.m_id;
        } else {
            int id = RDBConnection.getNextVal("person_preferences_id_seq");
            query = "insert into person_preferences (id, person_id, sms_notify) values("
                    + id + ", " + this.m_id + ", " + setting + ")";
        }
        RDBConnection.execute(query);
    }

    public String toString() {
        return StringUtils.URLDecode(getFirstName() + " " + getLastName() );
    }

    private void updateValue(String colName, String newVal) {
        String query = "update mmm_user set " + colName + "=? where id=?";
        Object[] params = { newVal, new Integer(m_id) };
        RDBConnection.executePrepared(query, params);
    }

}
