/*
 * Created on Aug 10, 2004
 *
 */
package gcr.mmm2.model;

import gcr.mmm2.rdb.RDBConnection;
import gcr.mmm2.rdb.ResultSetWrapper;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Simon King
 * @author Jeff Towle
 *
 */
public class UserManager {
	
	public static final String gcrUsers =  "1,10,11,12,36,37,140,141,142,144,145,1000,1001,1029,1031,1044,1091,1114,1090,1162,1163,1164,1165,1166,1199,1201,1045,13,143,1215";
	
	public static final String researchUsers =  "1086,1082,1045,1090,3888";

	public static IUser getByUsername(String username) {
		return CacheManager.getUserByName(username);
	}
	
	public static IUser getByID(int id) {
			try {
				return CacheManager.getUserByID(id);
			} catch (ObjectNotFoundException e) {
    				System.out.println("USERMANAGEGER:  id " + id + " is not a User.");
				return null;
			}
	}
	
	public static IUser getByIMSI(String imsi) {
		IUser u=null;
//		String query = "select id from mmm_user where phone_id='"+imsi+"'";
		String query="select u.id from mmm_user u, person_phone p where u.date_inactive is null and u.id=p.person_id and p.phone_imsi='"+imsi+"'";
		ResultSetWrapper rsw = RDBConnection.executeQuery(query);
		ResultSet rs = rsw.getResultSet();
		try {
			if (rs.next()) {
				int id = rs.getInt(1);
				u=getByID(id);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			rsw.closeAll();
		}
		return u;
	}
	
	public static IUser createUser(IPerson person, String username, String password, IUser creator) throws PermissionException {
		if (!isAdmin(creator)) {
			throw new PermissionException("Only Admin user (not "+creator.getUsername()+") may create other users.");
		}
		try {
			String query="insert into mmm_user (id, username, password, date_added, created_by) "+
				"values ("+person.getID()+", '"+username+"', '"+password+"', now(), "+creator.getID()+")";
			RDBConnection.execute(query);
			
			//Create entry in person_phone
			int id = RDBConnection.getNextVal("person_phone_id_seq");
			query="insert into person_phone (id, person_id, phone_num, created_by, date_added) "+
				"values ("+id+", "+person.getID()+", '0000000000', "+creator.getID()+", now())";
			System.out.println("Insert person_phone entry query: " + query);
			RDBConnection.execute(query);
			
			return new UserImpl(username);
		} catch (ObjectNotFoundException e) {
			e.printStackTrace();
			return null;
		}
	}
	
	public static boolean isUser(IPerson person) {
		return (getByID(person.getID())!=null);
	}
	
	public static boolean isAdmin(IUser user) {
		return (user.getID()<10); // IDs 0-9 are admin/superuser
	}
	
	// Check if user is in the Research group
	public static boolean isResearchUser(IUser user) {
		String idString = Integer.toString(user.getID());
		return (researchUsers.indexOf(idString) >= 0);
	}

	// set up all users with all other users in their contacts
	public static void populateAddressBooks() {
		// TODO build it
	}
	
	public static List listUsers() {
		String query = "select id from mmm_user where date_inactive is null";
		return listUsers(query);
	}
	
	public static List listNonGCRUsers() {
		String query = "select id from mmm_user where date_inactive is null and id not in ("+UserManager.gcrUsers+");";
		return listUsers(query);
	}
	
	protected static List listUsers(String query) {
		ResultSetWrapper rsw = RDBConnection.executeQuery(query);
		ResultSet rs = rsw.getResultSet();
		ArrayList l = new ArrayList();
		try {
			while (rs.next()) {
				int id = rs.getInt(1);
				IUser u=getByID(id);
				if (u !=null) {
					l.add(u);
				}
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			rsw.closeAll();
		}
		return l;
	}
	
	// returns a list of Blogs
	protected static List listBlogs(String query) {
		ResultSetWrapper rsw = RDBConnection.executeQuery(query);
		ResultSet rs = rsw.getResultSet();
		ArrayList l = new ArrayList();
		try {
			while (rs.next()) {
				int id = rs.getInt(1);
				Blog b = null;
				try {
					b = new Blog(id);
				} catch (ObjectNotFoundException e1) {
					//print stack trace, then do nothing
					e1.printStackTrace();
				}
				if (b !=null) {
					l.add(b);
				}
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			rsw.closeAll();
		}
		return l;
	}
	
	public static Blog getBlogByID(int id) {
		return CacheManager.getBlog(id);
	}
	
	
	public static List listOntologies() {
		String query = "select id, short_name from rdf_namespaces";
		return listOntologies(query);
	}
	
	protected static List listOntologies(String query) {
		ResultSetWrapper rsw = RDBConnection.executeQuery(query);
		ResultSet rs = rsw.getResultSet();
		ArrayList l = new ArrayList();
		try {
			while (rs.next()) {
				String id = Integer.toString(rs.getInt("id"));
				String shortName = rs.getString("short_name");
				String[] s = {id, shortName};
				if (s !=null) {
					l.add(s);
				}
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			rsw.closeAll();
		}
		return l;
	}
	
}
