/*
 * Created on Jul 1, 2004
 *
 */
package gcr.mmm2.rdb;

import gcr.mmm2.util.MMM2Properties;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

/**
 * @author jeff
 */
public class RDBConnection {
	private static RDBConnection m_rdbc;
	private static DataSource m_pool;

	/**
	 * @return
	 */
	public static RDBConnection getInstance() {
		if (m_rdbc == null) {
			m_rdbc = new RDBConnection();
		}

		return m_rdbc;
	}

	/**
	 * @return
	 */
	public Connection getConnection() {
	    String DATASOURCE_NAME = MMM2Properties.getProperty("DATASOURCE_NAME");
		if (m_pool == null) {

			try {

				Context env =
					(Context) new InitialContext().lookup("java:comp/env");

				m_pool = (DataSource) env.lookup(DATASOURCE_NAME);

			} catch (NamingException e) {
				e.printStackTrace();
				return null;
			}
		}
		try {
			return m_pool.getConnection();
		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}

	}

	public static ResultSetWrapper executeQuery(String q) {
		Connection conn = RDBConnection.getInstance().getConnection();
		Statement stmt = null;
		try {
			stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery(q);
			return new ResultSetWrapper(rs, stmt, conn);
		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}
	}

	public static void execute(String q) {
		Connection conn = RDBConnection.getInstance().getConnection();
		Statement stmt = null;
		try {
			stmt = conn.createStatement();
			stmt.execute(q);
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			if (stmt != null) {
				try {
					stmt.close();
				} catch (SQLException e1) {
					e1.printStackTrace();
				}
			}
			if (conn != null) {
				try {
					conn.close();
				} catch (SQLException e1) {
					e1.printStackTrace();
				}
			}
		}
	}
	
	public static void executePrepared(String q, Object[] p) {
		Connection conn = RDBConnection.getInstance().getConnection();
		PreparedStatement pstmt = null;
		try {
			pstmt = conn.prepareStatement(q);
			for (int i=0; i<p.length; i++) {
				pstmt.setObject(i+1, p[i]);
			}
			pstmt.executeUpdate();
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			if (pstmt != null) {
				try {
					pstmt.close();
				} catch (SQLException e1) {
					e1.printStackTrace();
				}
			}
			if (conn != null) {
				try {
					conn.close();
				} catch (SQLException e1) {
					e1.printStackTrace();
				}
			}
		}
	}

	/**
	 * Executes a group of prepared statements in a single transaction.
	 * --jc
	 */
	public static void executePreparedSet(Object[][] qp) {
		Connection conn = RDBConnection.getInstance().getConnection();
		PreparedStatement pstmt = null;
		String q;
		Object[] p;
		
		try{
			// Set auto-commit to false.
			conn.setAutoCommit(false);
	
			// Prepare and execute all statements.
			try {
				for (int i=0; i<qp.length; i++) {
					q = (String)qp[i][0];
					p = (Object[])qp[i][1];
					pstmt = conn.prepareStatement(q);
					for (int j=0; j<p.length; j++) {
						pstmt.setObject(j+1, p[j]);
					}
					pstmt.executeUpdate();
				}
			} catch (SQLException e) {
				e.printStackTrace();
			} 

			// Commit the db changes.
			conn.commit();		
		
			// Reset the connection to auto-commit.
			conn.setAutoCommit(true);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		// Additional cleanup
		if (pstmt != null) {
			try {
				pstmt.close();
			} catch (SQLException e1) {
				e1.printStackTrace();
			}
		}
		if (conn != null) {
			try {
				conn.close();
			} catch (SQLException e1) {
				e1.printStackTrace();
			}
		}
	}

	public static int getID(String table, String whereClause) {
		String query="select id from "+table+" where "+whereClause;
		return getSingleInt(query);
	}
	
	public static int getNextVal(String seq) {
		return getSingleInt("select nextval('"+seq+"')");
	}
	
	public static int getCount(String query){
	    return getSingleInt(query);
	}
	
	
	
	private static int getSingleInt(String query) {
		ResultSetWrapper rsw=executeQuery(query);
		int rtnVal=-1;
		try {
			if (rsw.getResultSet().next()) {
				rtnVal=rsw.getResultSet().getInt(1);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		rsw.closeAll();
		return rtnVal;
	}

}
