/*
 * Created on Jun 23, 2004
 *
 */
package gcr.mmm2.rdf;

import gcr.mmm2.rdb.RDBConnection;
import gcr.mmm2.rdb.ResultSetWrapper;
import gcr.rdf.MMMModelFactory;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntProperty;
import com.hp.hpl.jena.ontology.OntResource;

/**
 * Container for pictures, owned by a specific user
 * 
 * @author jeff
 * 
 */
public class Album extends MMMObject {

    // TODO get IDs to mesh with load by ont resource stuff
    public static final String CLASS_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#Album";

    public final static String PROPERTY_CONTAINS_MEDIA = "http://garage.sims.berkeley.edu/mmm-2/ontology#containsMediaAsset";

    public final static String PROPERTY_HAS_CAPTION = "http://garage.sims.berkeley.edu/mmm-2/ontology#hasCaption";

    private int m_album_id;

    /**
     * @param URI
     * @return Album
     */
    public static Album getAlbumByURI(String URI) {

        Individual ind = MMMModelFactory.getInstance().getOntModel()
                .getIndividual(URI);

        if (ind == null) {
            return null;
        }

        Iterator i = ind.listRDFTypes(false);
        boolean isAlbum = false;
        while (i.hasNext()) {
            String classType = i.next().toString();
            if (CLASS_URI.equals(classType)) {
                isAlbum = true;
            }
        }
        if (!isAlbum || (ind == null)) {
            return null;
        }

        return new Album(ind);
    }

    /**
     * @param id
     * @return Album
     */
    public static Album getAlbumByID(int id) {

        String query = "select album_uri from albums where album_id=" + id;
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();

        String uri = "";
        try {
            if (rs.next()) {
                uri = rs.getString(1);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return getAlbumByURI(uri);
    }

    /**
     * @param owner
     * @param name
     * @param media
     */
    public Album(User owner, String name, Picture[] media) {
        this(owner, name);
        addMedia(media);
    }

    /**
     * @param owner
     * @param name
     * @param pictureURIs
     */
    public Album(User owner, String name, String[] pictureURIs) {
        this(owner, name);
        addMediaByURI(pictureURIs);
    }

    /**
     * @param owner
     * @param name
     */
    public Album(User owner, String name) {
        // TODO: fix this name/caption mess
        this.myClass = MMMModelFactory.getInstance().getOntModel().getOntClass(
                CLASS_URI);
        this.r = MMMModelFactory.getInstance().getOntModel().createIndividual(
                BASE_URI + "album_" + owner.getUserID() + "_"
                        + System.currentTimeMillis(), myClass);
        this.m_album_id = RDBConnection.getNextVal("album_id_seq");
        RDBConnection
                .execute("insert into albums (album_id, username, album_uri, caption) values ("
                        + this.m_album_id
                        + ",'"
                        + owner.getUserID()
                        + "', '"
                        + this.r.getURI() + "', '" + name + "')");
    }

    /**
     * @param r
     */
    protected Album(OntResource r) {
        // doesn't create anybody
        this.r = r;
        this.myClass = MMMModelFactory.getInstance().getOntModel().getOntClass(
                CLASS_URI);
        String query = "select album_id from albums where album_uri='"
                + r.getURI() + "'";
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        int id = -1;
        try {
            if (rs.next()) {
                id = rs.getInt(1);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        this.m_album_id = id;
    }

    /**
     * @param caption
     */
    public void setCaption(String caption) {
        OntProperty p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(PROPERTY_HAS_CAPTION);
        replaceProperty(p, caption);
        RDBConnection.execute("update albums set caption = '" + caption
                + "' where album_id=" + getID());
    }

    /**
     * @param media
     */
    public void addMedia(Picture[] media) {
        for (int i = 0; i < media.length; i++) {
            addProperty(PROPERTY_CONTAINS_MEDIA, media[i].getURI());
            RDBConnection
                    .execute("insert into album_contains (album_id, picture_uri) values ("
                            + this.m_album_id
                            + ", '"
                            + media[i].getURI()
                            + "')");
        }
    }

    /**
     * @param pictureURIs
     */
    public void addMediaByURI(String[] pictureURIs) {
        for (int i = 0; i < pictureURIs.length; i++) {
            addProperty(PROPERTY_CONTAINS_MEDIA, pictureURIs[i]);
            RDBConnection
                    .execute("insert into album_contains (album_id, picture_uri) values ("
                            + this.m_album_id + ", '" + pictureURIs[i] + "')");
        }
    }

    /**
     * @param media
     */
    public void removeMedia(Picture[] media) {
        OntProperty p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(PROPERTY_CONTAINS_MEDIA);
        for (int i = 0; i < media.length; i++) {
            removeProperty(p, media[i].getURI());
            RDBConnection.execute("delete from album_contains where album_id="
                    + this.m_album_id + " AND picture_uri='"
                    + media[i].getURI() + "'");
        }
    }

    /**
     * @param media
     */
    public void removeMedia(Picture media) {
        OntProperty p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(PROPERTY_CONTAINS_MEDIA);
        removeProperty(p, media.getURI());
        RDBConnection
                .execute("delete from album_contains where album_id="
                        + this.m_album_id + " AND picture_uri='"
                        + media.getURI() + "'");
    }

    /**
     * @param pictureURIs
     */
    public void removeMediaByURI(String[] pictureURIs) {
        OntProperty p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(PROPERTY_CONTAINS_MEDIA);
        for (int i = 0; i < pictureURIs.length; i++) {
            removeProperty(p, pictureURIs[i]);
            RDBConnection.execute("delete from album_contains where album_id="
                    + this.m_album_id + " AND picture_uri='" + pictureURIs[i]
                    + "'");
        }
    }

    /**
     * @return Iterator of Picture
     */
    public Iterator listPictures() {
        String query = "select distinct picture_uri from album_contains where album_id="
                + this.m_album_id;
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        ArrayList l = new ArrayList();
        try {
            while (rs.next()) {
                String picURI = rs.getString(1);
                l.add(Picture.getPictureByURI(picURI));
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return l.iterator();
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {

        String query = "select caption from albums where album_id="
                + this.m_album_id;
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        String caption = "";
        try {
            if (rs.next()) {
                caption = rs.getString(1);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return caption;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gcr.mmm2.model.MMMObject#getID()
     */
    public int getID() {
        return this.m_album_id;
    }

    public void removeThis() {
        // TODO: log this!!!
        // cleanse the RDB
        // RDBConnection.execute("delete from albums where album_id='" + getID()
        // + "'");
        // RDBConnection.execute("delete from album_contains where album_id='" +
        // getID() + "'");
        // RDBConnection.execute("delete from shares where album_id='" + getID()
        // + "'");
        // now remove from the model
        super.removeThis();
        // TODO:this doesn't allow us to delete albums!! they have no uri!
    }
}