package gcr.mmm2.rdf;

import gcr.rdf.MMMModelFactory;
import gcr.rdf.ResourceManipulator;

import java.util.Iterator;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntProperty;
import com.hp.hpl.jena.ontology.OntResource;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.rdql.Query;
import com.hp.hpl.jena.rdql.QueryEngine;
import com.hp.hpl.jena.rdql.QueryExecution;
import com.hp.hpl.jena.rdql.QueryResults;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

/*
 * Created on Jun 16, 2004
 *
 */

/**
 * Base ontology object
 * 
 * @author jeff
 * 
 */
public abstract class MMMObject {

    public final static String BASE_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#";

    protected OntResource r;

    protected OntClass myClass;

    /**
     * @return
     */
    public OntResource getOntResource() {
        return r;
    }

    /**
     * @return
     */
    public String getURI() {
        // TODO: worry about anonymous
        return r.toString();
    }

    /**
     * @return
     */
    public StmtIterator getSubjectStatements() {
        return MMMModelFactory.getInstance().getOntModel().listStatements(r,
                (Property) null, (RDFNode) null);
    }

    /**
     * @return
     */
    public StmtIterator getObjectStatements() {
        return MMMModelFactory.getInstance().getOntModel().listStatements(
                (Resource) null, (Property) null, r);
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return r.toString();
    }

    /**
     * @return
     */
    public ExtendedIterator getAllTypes() {
        return r.listRDFTypes(false);
    }

    /**
     * @return
     */
    public OntClass getOntClass() {
        return myClass;
    }

    /**
     * @param propertyURI
     * @param value
     * @return
     */
    public boolean addProperty(String propertyURI, String value) {

        OntProperty p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(propertyURI);
        System.out.println("Property Range: ");

        if (p.isObjectProperty()) {
            Individual ind = MMMModelFactory.getInstance().getOntModel()
                    .getIndividual(value);
            if (ind == null) {
                return false;
            }
            if (ResourceManipulator.isInRange(p, ind)) {
                r.addProperty(p, ind); // could check return to see if it's
                // null
                return true;
            }
            System.out.println("addProperty: bad range: " + p + ", " + ind);
        } else if (p.isDatatypeProperty()) {
            System.out.println("add value: " + value);
            r.addProperty(p, value);
            return true;
        }

        return false;
    }

    public void removeThis() {

        // remove object properties for this subject
        StmtIterator si = MMMModelFactory.getInstance().getOntModel()
                .listStatements(r, (Property) null, (RDFNode) null);
        while (si.hasNext()) {
            Statement s = si.nextStatement();
            MMMModelFactory.getInstance().getOntModel().remove(s);
        }
        // remove String properties for this subject
        si = MMMModelFactory.getInstance().getOntModel().listStatements(r,
                (Property) null, (String) null);
        while (si.hasNext()) {
            Statement s = si.nextStatement();
            MMMModelFactory.getInstance().getOntModel().remove(s);
        }
        // remove statements with object=inputURI
        si = MMMModelFactory.getInstance().getOntModel().listStatements(
                (Resource) null, (Property) null, r);
        while (si.hasNext()) {
            Statement s = si.nextStatement();
            MMMModelFactory.getInstance().getOntModel().remove(s);
        }

    }

    /**
     * @param property
     * @param object
     * @return
     */
    protected boolean replaceProperty(Property property, Resource object) {

        if (property == null || object == null) {
            return false;
        }

        MMMModelFactory.getInstance().getOntModel().remove(
                MMMModelFactory.getInstance().getOntModel().listStatements(r,
                        property, (Resource) null));
        return addProperty(property, object);
    }

    protected void removeProperty(Property property, Resource object) {

        // null object means remove all objects with this property
        // dangerous?
        if (property == null) {
            return;
        }

        MMMModelFactory.getInstance().getOntModel().remove(
                MMMModelFactory.getInstance().getOntModel().listStatements(r,
                        property, object));
    }

    /**
     * @param property
     * @param literalValue
     */
    protected void removeProperty(Property property, String literalValue) {

        // literal value of null means scrub all values with this property
        // dangerous?
        if (property == null) {
            return;
        }

        MMMModelFactory.getInstance().getOntModel().remove(
                MMMModelFactory.getInstance().getOntModel().listStatements(r,
                        property, literalValue));
    }

    /**
     * @param property
     * @param literalValue
     * @return
     */
    protected boolean replaceProperty(Property property, String literalValue) {

        if (property == null || literalValue == null) {
            return false;
        }

        MMMModelFactory.getInstance().getOntModel().remove(
                MMMModelFactory.getInstance().getOntModel().listStatements(r,
                        property, (String) null));
        return addProperty(property, literalValue);
    }

    /**
     * @param property
     * @param literalValue
     */
    protected boolean addProperty(Property property, String literalValue) {
        return (r.addProperty(property, literalValue) != null);
    }

    /**
     * @param property
     * @param object
     */
    protected boolean addProperty(Property property, Resource object) {
        return (r.addProperty(property, object) != null);
    }

    /**
     * @param propertyURI
     * @return
     */
    public Iterator listValuesForProperty(String propertyURI) {
        return ResourceManipulator.listDeclaredInstancePropertyValues(this
                .getURI(), propertyURI);
    }

    /**
     * @param URI
     * @return
     */
    public static MMMObject getObjectByURI(String URI) {
        Individual i = MMMModelFactory.getInstance().getOntModel()
                .getIndividual(URI);
        if (i == null) {
            return null;
        }
        String classURI = i.getRDFType(true).toString();

        // Person
        if (classURI.equals(Person.CLASS_URI)) {
            return Person.getPersonByURI(URI);

            // User
        } else if (classURI.equals(User.CLASS_URI)) {
            return User.getUserByURI(URI);

            // Picture
        } else if (classURI.equals(Picture.CLASS_URI)) {
            return Picture.getPictureByURI(URI);

            // Share
        } else if (classURI.equals(Share.CLASS_URI)) {
            return Share.getShareByURI(URI);

            // Who knows!
        } else {
            return null;
        }
    }

    /**
     * @param queryString
     * @return
     */
    protected QueryResults query(String queryString) {

        Query query = new Query(queryString);

        // Need to set the source if the query does not.
        // This provided this override any query named source.
        query.setSource(MMMModelFactory.getInstance().getOntModel());
        QueryExecution qe = new QueryEngine(query);
        return qe.exec();
    }

    /**
     * @return
     */
    public int getID() {
        // TODO Auto-generated method stub
        return -1;
    }

}
