package gcr.mmm2.rdf;

import gcr.rdf.MMMModelFactory;

import java.util.ArrayList;
import java.util.Iterator;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntResource;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;

/*
 * Created on Jun 16, 2004
 *
 */

/**
 * @author jeff
 * 
 */
public class Person extends MMMObject {

    public static final String CLASS_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#Person";

    public final static String PROPERTY_FIRST_NAME = "http://garage.sims.berkeley.edu/mmm-2/ontology#firstName";

    public final static String PROPERTY_LAST_NAME = "http://garage.sims.berkeley.edu/mmm-2/ontology#lastName";

    public static Person getPersonByURI(String URI) {

        Individual ind = MMMModelFactory.getInstance().getOntModel()
                .getIndividual(URI);
        Iterator i = ind.listRDFTypes(false);
        boolean isPerson = false;
        while (i.hasNext()) {
            String classType = i.next().toString();
            if (CLASS_URI.equals(classType)) {
                isPerson = true;
            }
        }
        if (!isPerson || (ind == null)) {
            return null;
        }

        return new Person(ind);
    }

    public static Iterator listPersons() {
        ArrayList persons = new ArrayList();
        OntClass c = MMMModelFactory.getInstance().getOntModel().getOntClass(
                CLASS_URI);
        Iterator i = c.listInstances();
        while (i.hasNext()) {
            Individual ind = (Individual) i.next();
            persons.add(new Person(ind));
        }
        return persons.iterator();
    }

    protected Person(OntResource r) {
        // doesn't create anybody
        this.r = r;
        myClass = MMMModelFactory.getInstance().getOntModel().getOntClass(
                CLASS_URI);
    }

    public Person(String firstName, String lastName) {

        myClass = MMMModelFactory.getInstance().getOntModel().getOntClass(
                CLASS_URI);

        this.r = MMMModelFactory.getInstance().getOntModel().createIndividual(
                BASE_URI + "person" + firstName + lastName
                        + System.currentTimeMillis(), myClass);

        this.addFirstName(firstName);
        this.addLastName(lastName);

    }

    /**
     * @param firstName
     */
    private void addFirstName(String firstName) {
        addProperty(PROPERTY_FIRST_NAME, firstName);
    }

    /**
     * @param lastName
     */
    private void addLastName(String lastName) {
        addProperty(PROPERTY_LAST_NAME, lastName);
    }

    public String getFirstName() {
        Iterator i = listValuesForProperty(PROPERTY_FIRST_NAME);
        if (i.hasNext()) {
            return i.next().toString();
        }
        return "";
    }

    public String getLastName() {
        Iterator i = listValuesForProperty(PROPERTY_LAST_NAME);
        if (i.hasNext()) {
            return i.next().toString();
        }
        return "";
    }

    public Iterator getShares() {

        ArrayList shares = new ArrayList();

        Property p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(Share.PROPERTY_INITIATED_BY);
        if (p == null) {
            return null;
        }
        StmtIterator i = MMMModelFactory.getInstance().getOntModel()
                .listStatements((Resource) null, p, getURI());

        while (i.hasNext()) {
            Statement s = (Statement) i.next();
            Individual ind = MMMModelFactory.getInstance().getOntModel()
                    .getIndividual(s.getSubject().toString());
            shares.add(new Share(ind));
        }

        return shares.iterator();
    }

    public Iterator getReceivedShares() {
        ArrayList shares = new ArrayList();

        Property p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(Share.PROPERTY_SHARED_WITH);
        if (p == null) {
            return null;
        }
        StmtIterator i = MMMModelFactory.getInstance().getOntModel()
                .listStatements((Resource) null, p, r);

        while (i.hasNext()) {
            Statement s = (Statement) i.next();
            Individual ind = MMMModelFactory.getInstance().getOntModel()
                    .getIndividual(s.getSubject().toString());
            shares.add(new Share(ind));
        }

        return shares.iterator();
    }

    public String toString() {
        return getFirstName() + " " + getLastName();
    }

}
