/*
 * Created on Jun 23, 2004
 *
 */
package gcr.mmm2.rdf;

import gcr.rdf.MMMModelFactory;

import java.util.HashSet;
import java.util.Iterator;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntResource;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;

/**
 * @author jeff
 * 
 */
public class PersonList extends MMMObject {

    public static final String CLASS_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#PersonList";

    public final static String PROPERTY_ENUMERATES_PERSONS = "http://garage.sims.berkeley.edu/mmm-2/ontology#enumeratesPersons";

    /**
     * @param URI
     * @return
     */
    public static PersonList getPersonListByURI(String URI) {

        Individual ind = MMMModelFactory.getInstance().getOntModel()
                .getIndividual(URI);
        Iterator i = ind.listRDFTypes(false);
        boolean isPerson = false;
        while (i.hasNext()) {
            String classType = i.next().toString();
            if (CLASS_URI.equals(classType)) {
                isPerson = true;
            }
        }
        if (!isPerson || (ind == null)) {
            return null;
        }

        return new PersonList(ind);
    }

    /**
     * @param name
     * @param people
     */
    public PersonList(String name, MMMObject[] people) {
        myClass = MMMModelFactory.getInstance().getOntModel().getOntClass(
                CLASS_URI);
        this.r = MMMModelFactory.getInstance().getOntModel().createIndividual(
                BASE_URI + "personList" + name, myClass);
        addPeople(people);
    }

    /**
     * @param name
     * @param peopleURIs
     */
    public PersonList(String name, String[] peopleURIs) {
        myClass = MMMModelFactory.getInstance().getOntModel().getOntClass(
                CLASS_URI);
        this.r = MMMModelFactory.getInstance().getOntModel().createIndividual(
                BASE_URI + "personList" + name, myClass);
        addPeopleByURI(peopleURIs);
    }

    /**
     * @param name
     */
    public PersonList(String name) {
        myClass = MMMModelFactory.getInstance().getOntModel().getOntClass(
                CLASS_URI);
        this.r = MMMModelFactory.getInstance().getOntModel().createIndividual(
                BASE_URI + "personList" + name, myClass);
    }

    /**
     * @param r
     */
    protected PersonList(OntResource r) {
        // doesn't create anybody
        this.r = r;
        myClass = MMMModelFactory.getInstance().getOntModel().getOntClass(
                CLASS_URI);
    }

    /**
     * @param people
     */
    public void addPeople(MMMObject[] people) {
        // TODO add type-checking to people array
        for (int i = 0; i < people.length; i++) {
            addProperty(PROPERTY_ENUMERATES_PERSONS, people[i].getURI());
        }
    }

    /**
     * @param peopleURIs
     */
    public void addPeopleByURI(String[] peopleURIs) {
        // TODO add type-checking to people array
        for (int i = 0; i < peopleURIs.length; i++) {
            addProperty(PROPERTY_ENUMERATES_PERSONS, peopleURIs[i]);
        }
    }

    /**
     * @return
     */
    public Iterator listMembers() {
        HashSet visited = new HashSet();
        HashSet members = new HashSet();
        listMembers(visited, members);
        return members.iterator();
    }

    /**
     * @param visited
     * @param members
     */
    private void listMembers(HashSet visited, HashSet members) {

        Property p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(PROPERTY_ENUMERATES_PERSONS);
        if (p == null) {
            return;
        }

        StmtIterator i = MMMModelFactory.getInstance().getOntModel()
                .listStatements(r, p, (Resource) null);
        System.out.println("got the stuff");

        while (i.hasNext()) {

            Statement s = i.nextStatement();
            Individual ind = MMMModelFactory.getInstance().getOntModel()
                    .getIndividual(s.getObject().toString());

            if ((PersonList.CLASS_URI.equals(ind.getRDFType(true).toString()))
                    && (!visited.contains(ind.toString()))) {

                visited.add(ind.toString());
                PersonList newGuy = new PersonList(ind);
                newGuy.listMembers(visited, members);
            } else if (Person.CLASS_URI.equals(ind.getRDFType(true).toString())) {
                members.add(new Person(ind));
            }
        }// end while
        return;
    }
}
