package gcr.mmm2.rdf;

import gcr.rdf.MMMModelFactory;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntResource;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;

/*
 * Created on Jun 16, 2004
 *
 */

/**
 * @author jeff
 * 
 */
public class Picture extends MMMObject {

    public static String CLASS_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#Picture";

    public final static String PROPERTY_CAPTION = "http://garage.sims.berkeley.edu/mmm-2/ontology#hasCaption";

    public final static String PROPERTY_FILE_PATH = "http://garage.sims.berkeley.edu/mmm-2/ontology#hasImagefile";

    public final static String PROPERTY_PHOTOGRAPHER = "http://garage.sims.berkeley.edu/mmm-2/ontology#photographedBy";

    public final static String PROPERTY_UPLOADED_BY = "http://garage.sims.berkeley.edu/mmm-2/ontology#uploadedBy";

    public final static String PROPERTY_VIEWED_BY = "http://garage.sims.berkeley.edu/mmm-2/ontology#viewedBy";

    public final static String PROPERTY_DEPICTS_PERSON = "http://garage.sims.berkeley.edu/mmm-2/ontology#depictsPerson";

    /**
     * @param URI
     * @return
     */
    public static Picture getPictureByURI(String URI) {

        Individual ind = MMMModelFactory.getInstance().getOntModel()
                .getIndividual(URI);
        Iterator i = ind.listRDFTypes(false);
        boolean isPicture = false;
        while (i.hasNext()) {
            String classType = i.next().toString();
            if (CLASS_URI.equals(classType)) {
                isPicture = true;
            }
        }
        if (!isPicture || (ind == null)) {
            return null;
        }

        return new Picture(ind);
    }

    /**
     * @param r
     */
    protected Picture(OntResource r) {
        // doesn't create anybody
        this.r = r;
        this.myClass = MMMModelFactory.getInstance().getOntModel().getOntClass(
                CLASS_URI);
    }

    /**
     * @param filename
     * @param user
     */
    public Picture(String filename, User user) {

        this.myClass = MMMModelFactory.getInstance().getOntModel().getOntClass(
                CLASS_URI);

        this.r = MMMModelFactory.getInstance().getOntModel().createIndividual(
                BASE_URI + filename, this.myClass);

        setFilename(filename);

        if (user != null)
            setUploadedBy(user);

    }

    /**
     * @param user
     */
    private void setUploadedBy(User user) {
        // TODO: add an addProperty() method that takes resources, not just uris
        addProperty(PROPERTY_UPLOADED_BY, user.getURI());
    }

    /**
     * @param caption
     */
    public void addCaption(String caption) {
        addProperty(PROPERTY_CAPTION, caption);
    }

    /**
     * @param user
     */
    public void setViewedBy(User user) {
        addProperty(PROPERTY_VIEWED_BY, user.getURI());
    }

    /**
     * @param photographer
     */
    public void addPhotographer(Person photographer) {
        // TODO: add methods that take nodes, not strings
        addProperty(PROPERTY_PHOTOGRAPHER, photographer.getURI());
    }

    /**
     * @return
     */
    public String getCaption() {
        Iterator i = listValuesForProperty(PROPERTY_CAPTION);
        if (i.hasNext()) {
            return i.next().toString();
        }
        return "";
    }

    public boolean isViewedBy(User user) {
        // TODO: make next 4 lines their own method in MMMObject or someplace
        Property p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(PROPERTY_FILE_PATH);
        if (p == null) {
            return false;
        }
        return this.r.hasProperty(p, user.getOntResource());
    }

    /**
     * @return
     */
    public Iterator listViewers() {
        System.out.println("looking for share viewers of " + getURI());
        ArrayList viewers = new ArrayList();

        Property p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(PROPERTY_VIEWED_BY);
        if (p == null) {
            return viewers.iterator();
        }
        StmtIterator i = MMMModelFactory.getInstance().getOntModel()
                .listStatements(this.r, p, (Resource) null);
        System.out.println("got the viewers");
        while (i.hasNext()) {
            System.out.println("found a viewer");
            Statement s = i.nextStatement();
            viewers.add(User.getUserByURI(s.getObject().toString()));

        }
        return viewers.iterator();
    }

    /**
     * @return
     */
    public User getUploadedBy() {
        Iterator i = listValuesForProperty(PROPERTY_UPLOADED_BY);
        if (i.hasNext()) {
            return User.getUserByURI(i.next().toString());
        }
        return null;
    }

    /**
     * @return
     */
    public Person getPhotographer() {
        Iterator i = listValuesForProperty(PROPERTY_PHOTOGRAPHER);
        if (i.hasNext()) {
            return Person.getPersonByURI(i.next().toString());
        }
        return null;
    }

    /**
     * @return
     */
    public String getFilename() {
        Iterator i = listValuesForProperty(PROPERTY_FILE_PATH);
        if (i.hasNext()) {
            return i.next().toString();
        }
        return "";
    }

    /**
     * @param base
     * @return
     */
    public String getImageURL(String base) {
        return base + "/images/mmmpicture/" + getFilename();
    }

    /**
     * @param filename
     */
    private void setFilename(String filename) {
        if (getFilename().equals("")) {
            addProperty(PROPERTY_FILE_PATH, filename);
        } else {
            Property p = MMMModelFactory.getInstance().getOntModel()
                    .getOntProperty(PROPERTY_FILE_PATH);
            if (p == null) {
                return;
            }
            StmtIterator i = MMMModelFactory.getInstance().getOntModel()
                    .listStatements(r, p, (String) null);
            while (!i.hasNext()) {
                Statement s = (Statement) i.next();
                s.changeObject(filename);
                // iterate because something may go horribly wrong
            }
        }
    }

    /**
     * @return
     */
    public Iterator getShareRecipients() {
        // TODO simplyfy this once data gets cleaned
        System.out.println("looking for share recipients of " + getURI());
        HashSet recipients = new HashSet();

        Property p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(Share.PROPERTY_SHARES_MEDIA);
        if (p == null) {
            return recipients.iterator();
        }
        StmtIterator i = MMMModelFactory.getInstance().getOntModel()
                .listStatements((Resource) null, p, r);
        System.out.println("got the shares");
        while (i.hasNext()) {
            System.out.println("found a share");

            Statement s = i.nextStatement();
            Individual ind = MMMModelFactory.getInstance().getOntModel()
                    .getIndividual(s.getSubject().toString());

            Iterator j = new Share(ind).getRecipients();
            while (j.hasNext()) {
                System.out.println("found a recipient");
                // recipients.add(MMMPerson.getPersonByURI(j.next().toString()));
                Individual ind2 = MMMModelFactory.getInstance().getOntModel()
                        .getIndividual(j.next().toString());
                if ((PersonList.CLASS_URI.equals(ind2.getRDFType(true)
                        .toString()))) {
                    PersonList pl = new PersonList(ind2);

                    Iterator k = pl.listMembers();
                    while (k.hasNext()) {
                        recipients.add(k.next());
                    }
                } else {
                    recipients.add(new Person(ind2));
                }
            }
        }
        return recipients.iterator();
    }

    public void removeThis() {
        // TODO: log this!!!
        // cleanse the RDB
        // RDBConnection.execute("delete from album_contains where
        // picture_uri='" + getURI() + "')");
        // RDBConnection.execute("delete from shares where picture_uri='" +
        // getURI() + "')");
        // TODO: clear up share recipients

        // now remove from the model
        super.removeThis();
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return getFilename();
    }

}
