package gcr.mmm2.rdf;

import gcr.mmm2.rdb.RDBConnection;
import gcr.rdf.MMMModelFactory;

import java.util.Iterator;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntResource;

/*
 * Created on Jun 16, 2004
 *
 */

/**
 * @author jeff
 */
public class Share extends MMMObject {

    public static final String CLASS_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#Share";

    public final static String PROPERTY_SHARED_WITH = "http://garage.sims.berkeley.edu/mmm-2/ontology#grantedTo";

    public final static String PROPERTY_SHARES_MEDIA = "http://garage.sims.berkeley.edu/mmm-2/ontology#sharesMedia";

    public final static String PROPERTY_INITIATED_BY = "http://garage.sims.berkeley.edu/mmm-2/ontology#initiatedBy";

    public final static String PROPERTY_SHARE_MESSAGE = "http://garage.sims.berkeley.edu/mmm-2/ontology#hasMessage";

    private int m_share_id;

    /**
     * @param URI
     * @return
     */
    public static Share getShareByURI(String URI) {

        Individual ind = MMMModelFactory.getInstance().getOntModel()
                .getIndividual(URI);
        Iterator i = ind.listRDFTypes(false);
        boolean isShare = false;
        while (i.hasNext()) {
            String classType = i.next().toString();
            if (CLASS_URI.equals(classType)) {
                isShare = true;
            }
        }
        if (!isShare || (ind == null)) {
            return null;
        }

        return new Share(ind);
    }

    protected Share(OntResource r) {
        // doesn't create anybody
        this.r = r;
        this.myClass = MMMModelFactory.getInstance().getOntModel().getOntClass(
                CLASS_URI);
    }

    /**
     * @param initiator
     * @param asset
     */
    /**
     * @param initiator
     * @param asset
     */
    public Share(User initiator, MMMObject asset) {

        this.myClass = MMMModelFactory.getInstance().getOntModel().getOntClass(
                CLASS_URI);

        this.r = MMMModelFactory.getInstance().getOntModel().createIndividual(
                BASE_URI + "share" + initiator.getUserID()
                        + System.currentTimeMillis(), this.myClass);

        m_share_id = RDBConnection.getNextVal("share_id_seq");
        RDBConnection
                .execute("insert into shares (share_id, username) values ("
                        + m_share_id + ",'" + initiator.getUserID() + "')");
        setInitiator(initiator);
        setShareObject(asset);

    }

    /**
     * @param initiator
     * @param asset
     * @param message
     */
    public Share(User initiator, Picture asset, String message) {
        this(initiator, asset);
        System.out.println("Share Message: " + message);
        if (message != null) {
            setShareMessage(message);
        }
    }

    /**
     * @param user
     */
    private void setInitiator(User user) {
        addProperty(PROPERTY_INITIATED_BY, user.getURI());
    }

    /**
     * @param asset
     */
    private void setShareObject(MMMObject asset) {
        addProperty(PROPERTY_SHARES_MEDIA, asset.getURI());

        if (asset.myClass.getURI().equals(Album.CLASS_URI))
            RDBConnection.execute("update shares set album_id=" + asset.getID()
                    + " where share_id=" + m_share_id);
        else if (asset.myClass.getURI().equals(Picture.CLASS_URI))
            RDBConnection.execute("update shares set picture_uri='"
                    + asset.getURI() + "' where share_id=" + m_share_id);

    }

    /**
     * @param message
     */
    private void setShareMessage(String message) {
        System.out.println("Set Share message: " + message);
        addProperty(PROPERTY_SHARE_MESSAGE, message);
        RDBConnection.execute("update shares set message='" + message
                + "' where share_id=" + m_share_id);
    }

    /**
     * @return
     */
    public String getShareObject() {
        Iterator i = listValuesForProperty(PROPERTY_SHARES_MEDIA);
        if (i.hasNext()) {
            return i.next().toString();
        }
        return "";
    }

    /**
     * @return
     */
    public String getMessage() {
        Iterator i = listValuesForProperty(PROPERTY_SHARE_MESSAGE);
        if (i.hasNext()) {
            return i.next().toString();
        }
        return "";
    }

    /**
     * @return
     */
    public String getInitiator() {
        Iterator i = listValuesForProperty(PROPERTY_INITIATED_BY);
        if (i.hasNext()) {
            return i.next().toString();
        }
        return "";
    }

    /**
     * @return
     */
    public Iterator getRecipients() {
        return listValuesForProperty(PROPERTY_SHARED_WITH);
    }

    /**
     * @param recipient
     */
    public void addRecipient(Person recipient) {
        addProperty(PROPERTY_SHARED_WITH, recipient.getURI());
        RDBConnection
                .execute("insert into share_recipients (share_recip_id, share_id, person_uri) values (nextval('share_recip_id_seq'),"
                        + m_share_id + ", '" + recipient.getURI() + "')");
    }

    /**
     * @param recipient
     */
    public void addRecipient(PersonList recipient) {
        addProperty(PROPERTY_SHARED_WITH, recipient.getURI());
        Iterator i = recipient.listMembers();
        while (i.hasNext()) {
            Person p = (Person) i.next();
            RDBConnection
                    .execute("insert into share_recipients (share_recip_id, share_id, person_uri) values (nextval('share_recip_id_seq'),"
                            + m_share_id + ", '" + p.getURI() + "')");
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return getInitiator().toString() + " shares "
                + getShareObject().toString();
    }
}
