/*
 * Created on Jun 17, 2004
 *
 */
package gcr.mmm2.rdf;

import gcr.mmm2.rdb.RDBConnection;
import gcr.mmm2.rdb.ResultSetWrapper;
import gcr.rdf.MMMModelFactory;
import gcr.rdf.query.PictureQuery;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntResource;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.rdql.QueryResults;
import com.hp.hpl.jena.rdql.ResultBinding;

/**
 * @author jeff
 * 
 */
public class User extends Person {

    public static final String CLASS_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#User";

    public static final String PROPERTY_PASSWORD = "http://garage.sims.berkeley.edu/mmm-2/ontology#hasPassword";

    public static final String PROPERTY_USER_ID = "http://garage.sims.berkeley.edu/mmm-2/ontology#hasUserID";

    /**
     * @param userID
     * @return
     */
    public static User getByUserID(String userID) {

        Property p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(PROPERTY_USER_ID);
        if (p == null) {
            return null;
        }

        StmtIterator i = MMMModelFactory.getInstance().getOntModel()
                .listStatements((Resource) null, p, userID);
        if (!i.hasNext()) {
            return null;
        }

        Statement s = (Statement) i.next();
        Individual ind = MMMModelFactory.getInstance().getOntModel()
                .getIndividual(s.getSubject().toString());

        return new User(ind);
    }

    /**
     * @param URI
     * @return
     */
    public static User getUserByURI(String URI) {
        Individual ind = MMMModelFactory.getInstance().getOntModel()
                .getIndividual(URI);
        Iterator i = ind.listRDFTypes(false);
        boolean isPerson = false;
        while (i.hasNext()) {
            String classType = i.next().toString();
            if (CLASS_URI.equals(classType)) {
                isPerson = true;
            }
        }
        if (!isPerson || (ind == null)) {
            return null;
        }

        return new User(ind);
    }

    /**
     * @param p
     * @param userID
     * @param password
     */
    public User(Person p, String userID, String password) {
        super(p.getOntResource());

        this.myClass = MMMModelFactory.getInstance().getOntModel().getOntClass(
                CLASS_URI);
        setPassword(password);
        setUserID(userID);
    }

    /**
     * @param r
     */
    private User(OntResource r) {
        // doesn't create anybody
        super(r);
        this.myClass = MMMModelFactory.getInstance().getOntModel().getOntClass(
                CLASS_URI);
    }

    /**
     * @param firstName
     * @param lastName
     * @param userID
     * @param password
     */
    public User(String firstName, String lastName, String userID,
            String password) {
        super(firstName, lastName);

        this.myClass = MMMModelFactory.getInstance().getOntModel().getOntClass(
                CLASS_URI);
        setPassword(password);
        setUserID(userID);
    }

    /**
     * @return
     */
    public String getPassword() {
        Iterator i = listValuesForProperty(PROPERTY_PASSWORD);
        if (i.hasNext()) {
            return i.next().toString();
        }
        return "";
    }

    /**
     * @return
     */
    public String getUserID() {
        Iterator i = listValuesForProperty(PROPERTY_USER_ID);
        if (i.hasNext()) {
            return i.next().toString();
        }
        return "";
    }

    /**
     * @return
     */
    public Iterator getUploadedPictures() {
        System.out.println("looking for pictures uploaded by " + getURI());

        PictureQuery pq = new PictureQuery(this);
        return pq.returnPictureResults().iterator();
    }

    /**
     * @return
     */
    public Iterator getInitiatedShares() {

        String queryString = "SELECT ?a WHERE (?a, <"
                + Share.PROPERTY_INITIATED_BY + ">, <" + getURI() + ">)";

        System.out.println("looking for shares initiated by " + getURI());
        ArrayList shares = new ArrayList();
        QueryResults i = query(queryString);

        while (i.hasNext()) {
            System.out.println("found a share");
            ResultBinding rb = (ResultBinding) i.next();
            shares.add(Share.getShareByURI(rb.get("a").toString()));
        }
        return shares.iterator();
    }

    /**
     * @param password
     */
    public void setPassword(String password) {
        Property p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(PROPERTY_PASSWORD);
        if (p == null) {
            return;
        }
        replaceProperty(p, password);
    }

    /**
     * @param userID
     */
    private void setUserID(String userID) {
        // TODO: cache these properties?
        Property p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(PROPERTY_USER_ID);
        if (p == null) {
            return;
        }
        replaceProperty(p, userID);
    }

    /**
     * @return An iterator of "Picture" objects that have been shared *with*
     *         this user
     */
    public Iterator getReceivedPictures() {

        String query = "select distinct picture_uri from shares s, share_recipients sr where s.share_id=sr.share_id and sr.person_uri='"
                + getURI() + "'";
        ArrayList l = null;
        ResultSetWrapper rsw = null;
        ResultSet rs = null;

        try {
            rsw = RDBConnection.executeQuery(query);
            if (rsw != null) {
                rs = rsw.getResultSet();
                l = new ArrayList();

                while (rs.next()) {
                    String picURI = rs.getString(1);
                    l.add(Picture.getPictureByURI(picURI));
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            if (rsw != null)
                rsw.closeAll();
        }

        if (l != null)
            return l.iterator();
        return null;
    }

    /**
     * @return
     */
    public Iterator listAlbums() {

        String query = "select album_id from albums where username='"
                + getUserID() + "'";
        ArrayList l = null;
        ResultSetWrapper rsw = null;
        ResultSet rs = null;

        try {
            rsw = RDBConnection.executeQuery(query);
            if (rsw != null) {
                rs = rsw.getResultSet();
                l = new ArrayList();

                while (rs.next()) {
                    int albumID = rs.getInt(1);
                    l.add(Album.getAlbumByID(albumID));
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            if (rsw != null)
                rsw.closeAll();
        }

        if (l != null)
            return l.iterator();
        return null;
    }
}