/*
 * Created on Jun 17, 2004
 *
 */
package gcr.mmm2.server;
import gcr.mmm2.model.IUser;
import gcr.mmm2.model.MediaManager;
import gcr.mmm2.model.UIState;
import gcr.mmm2.model.UserManager;

import java.security.Principal;
import java.util.logging.Logger;

import java.io.*;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.caucho.security.BasicPrincipal;
import com.caucho.server.security.AbstractAuthenticator;
//import com.caucho.server.security.NullAuthenticator;

/**
 * @author  simon
 */
public class MMMAuthenticator extends AbstractAuthenticator {

	static protected final Logger log = Logger.getLogger("loginActivity");
	
	/**
	 * 
	 */
	public MMMAuthenticator() {
		super();
	}
	
	public Principal loginImpl(HttpServletRequest request, HttpServletResponse response, ServletContext application, String username, String password) {
		try {
			IUser p=UserManager.getByUsername(username);
			if (p.checkPassword(password)) {
				Principal pr=new BasicPrincipal(p.getUsername());
				request.getSession().setAttribute("user", pr);
				log.info("SID="+request.getSession().getId()+" - login via password for UID="+p.getID());
                // set userid in the session state
                UIState state = (UIState)request.getSession().getAttribute("uiState");
                state.setUserID(p.getID());
                
                // now read in uiSettings for the user
                
                String fileToLoad = MediaManager.ORIG_IMAGE_DIR + "/" + 
                    state.getUserID() + "/settings.ser";
                int uid = state.getUserID();
                
                File f = new File(fileToLoad);
                if(f.exists())
                {
                    try
                    {
                        ObjectInputStream in = new ObjectInputStream(
                            new FileInputStream(fileToLoad));
                        state = (UIState)in.readObject();
                        state.initTransients();
                        
                    }catch(InvalidClassException ice)
                    {
                        System.out.println("Invalid class--class changed since serialized");
                        state = new UIState();
                        state.initTransients();
                        f.delete();
                    }
                    state.setUserID(uid);
                }
                request.getSession().setAttribute("uiState",state);
				return pr;
			}
            log.info("SID="+request.getSession().getId()+" - login via password failed for UID="+p.getID());
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public Principal loginImpl(HttpServletRequest request, String imsi) {
		try {
			IUser p=UserManager.getByIMSI(imsi);
			if(p!=null){
				Principal pr=new BasicPrincipal(p.getUsername());
				request.getSession().setAttribute("user", pr);
				log.info("SID="+request.getSession().getId()+" - login via IMSI for UID="+p.getID());
				return pr;
			}
            log.info("SID="+request.getSession().getId()+" - login via IMSI failed for IMSI="+imsi);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}	
	
	public boolean isUserInRole(HttpServletRequest request, HttpServletResponse response, ServletContext application, Principal user, String role) {
		if ("user".equals(role)) {
			return true;
		} else if ("admin".equals(role)) {
			// user ids < 10 are admin users
			try {
				return (UserManager.getByUsername(user.getName()).getID()<10);
			} catch(Exception e) {
				e.printStackTrace();
			}
		}
		return false;
	}
	
	public Principal getUserPrincipalImpl(HttpServletRequest request, ServletContext application) {
		
		Principal pr = (Principal)request.getSession().getAttribute("user");
		if(pr != null)
			return pr;
		
		//else let's check for IMSI first
		String Imsi = request.getParameter("imsi");
		if(Imsi != null){
			IUser u = UserManager.getByIMSI(Imsi);
			if(u != null){
				pr = new BasicPrincipal(u.getUsername());
				request.getSession().setAttribute("user", pr);
				log.info("SID="+request.getSession().getId()+" - login via IMSI for UID="+u.getID());
			}else{
				log.info("SID="+request.getSession().getId()+" - login via IMSI failed for IMSI="+Imsi);
			}
		}

		return pr;
	}

}
