package gcr.mmm2.server.servlets;

import gcr.mmm2.model.MediaManager;
import gcr.mmm2.model.UserManager;
import gcr.mmm2.util.MMM2Properties;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.logging.Logger;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Lazy, synchronous compression of images down to thumbnail sizes based on the
 * URL requesting the image. Also security for image access.
 * 
 */
public class GetImageServlet extends HttpServlet {

    static final String IMAGE_LOCATION = MMM2Properties
            .getProperty("IMAGE_LOCATION");

    static protected final Logger log = Logger
            .getLogger("imageServletActivity");

    public void doGet(HttpServletRequest request, HttpServletResponse response)
            throws IOException, ServletException {

        // Get the absolute path of the image
        ServletContext sc = getServletContext();

        String[] s = request.getPathInfo().split("/");

        if (s.length != 3) {
            log.info("Could not get path of " + request.getPathInfo());
            response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
        }

        String filename = request.getRealPath(IMAGE_LOCATION + s[2]);

        try {
            filename = MediaManager.getRealPath(s[2]);
        } catch (Exception e) {
            e.printStackTrace();
        }

        String uid = "";
        if (request.getRemoteUser() != null)
            uid = " UID="
                    + UserManager.getByUsername(request.getRemoteUser())
                            .getID();

        log.info("SID=" + request.getSession().getId() + " " + s[2]
                + " requested with code '" + s[1] + "'" + uid);
        if (!filename.endsWith(s[2]))
            log.info("SID=" + request.getSession().getId() + " " + filename
                    + " returned when " + s[2] + " requested" + uid);

        // Get the MIME type of the image
        String mimeType = sc.getMimeType(filename);
        if (mimeType == null) {
            log.info("Could not get MIME type of " + filename);
            response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
            return;
        }

        // very important!
        response.reset();

        // Set content type
        response.setContentType(mimeType);

        // Set content size
        File file = new File(filename);

        if (!file.canRead()) {
            throw new ServletException("Unable to read file:"
                    + file.getAbsolutePath());
        }

        response.setContentLength((int) file.length());

        // Open the file and output streams
        BufferedInputStream in = new BufferedInputStream(new FileInputStream(
                file));
        BufferedOutputStream out = new BufferedOutputStream(response
                .getOutputStream());

        // Copy the contents of the file to the output stream
        byte[] buf = new byte[4096];
        int count = 0;
        while ((count = in.read(buf)) >= 0) {
            out.write(buf, 0, count);
        }

        out.flush();
        out.close();

        in.close();
    }

}
