/*
 * Created on Jul 10, 2004
 *
 */
package gcr.mmm2.server.servlets;

import gcr.mmm2.util.TwoSideBox;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.Calendar;
import java.util.GregorianCalendar;


import javax.imageio.ImageIO;
import javax.swing.ImageIcon;

import photoorganizer.formats.BasicJpeg;

/**
 * Rotates the images. Saves the original. Falls back to use native java methods
 * if the (faster) call to ImageMagick or jpegtran fails.
 * 
 * 
 * @author jeff, scott
 * 
 */
public class JpegRotator {

    public static final int ROTATE_90_LEFT = 0;

    public static final int ROTATE_90_RIGHT = 1;

    public static final int ROTATE_180 = 2;
    
    private static Date nextTryNativeRotate = (new GregorianCalendar(1970, Calendar.JANUARY,1)).getTime();

    private static Date nextTryNativeResize = (new GregorianCalendar(1970, Calendar.JANUARY,1)).getTime();

    public static void rotateImage(String fileName, int direction) {

        String originalFile = fileName + "_original.jpg";
        String newFileName = fileName + "_rotated.jpg";

        switch (direction) {

        case ROTATE_90_LEFT:
            doRotate(fileName, newFileName, "270");
            break;

        case ROTATE_90_RIGHT:
            doRotate(fileName, newFileName, "90");
            break;

        case ROTATE_180:
            doRotate(fileName, newFileName, "180");
            break;
        }

        // first take the input filename and create _rotated.jpg
        // filename for the rotated image

        // get the original file and rename it to _original
        // Windows requires deleting old file first before renaming
        File f = new File(fileName);
        (new File(originalFile)).delete();
        f.renameTo(new File(originalFile));

        // get the rotated file and rename it to original filename
        // Windows requires deleting old file first before renaming
        f = new File(newFileName);
        (new File(fileName)).delete();
        f.renameTo(new File(fileName));
    }

    public static void main(String[] args) {
        rotateImage(args[0], Integer.parseInt(args[1]));
    }

    private static void doRotate(String inFile, String outFile, String degrees) {
        if((new Date()).after(nextTryNativeRotate))
            doNativeRotate(inFile,outFile,degrees);
        else
            doJavaRotate(inFile,outFile,degrees);
        
        if(!new File(outFile).canRead()) {
            System.err.println("ERROR: Rotate failed in JpegRotator");
        }
    }

    public static void doResize(String inFile, String outFile, int width,
            int height) {
        if((new Date()).after(nextTryNativeResize))
            doNativeResize(inFile,outFile,width,height);
        else
            doJavaResize(inFile,outFile,width,height);
        
        if(!new File(outFile).canRead()) {
            System.err.println("ERROR: Resize failed in JpegRotator");
        }
    }
    
    public static void doCropResize(String inFile, String outFile, int width,
            int height){
        if((new Date()).after(nextTryNativeResize))
            doJavaCropResize(inFile,outFile,width,height);
            // doNativeCropResize(inFile,outFile,width,height);
        else
            doJavaCropResize(inFile,outFile,width,height);
        
        if(!new File(outFile).canRead()) {
            System.err.println("ERROR: Resize failed in JpegRotator");
        }
        
    }

    private static void doJavaRotate(String inFile, String outFile,
            String degrees) {
        File f = new File(inFile);
        BasicJpeg image = new BasicJpeg(f);
        if (degrees.equals("90"))
            image.transform(outFile, BasicJpeg.ROT_90, true);
        if (degrees.equals("180"))
            image.transform(outFile, BasicJpeg.ROT_180, true);
        if (degrees.equals("270"))
            image.transform(outFile, BasicJpeg.ROT_270, true);
    }

    private static void doJavaResize(String inFile, String outFile, int width,
            int height) {
        try {
            BufferedImage bi = ImageIO.read(new File(inFile));
            // find correct width and height (scaledSize)
            TwoSideBox tempBox = new TwoSideBox(width, height);
            TwoSideBox scaledSize = tempBox
                    .getSizeForFittingInsideMe(new TwoSideBox(bi.getWidth(), bi
                            .getHeight()));

            BufferedImage thumbImage = new BufferedImage(scaledSize.getWidth(),
                    scaledSize.getHeight(), BufferedImage.TYPE_INT_RGB);
            Graphics2D graphics2D = thumbImage.createGraphics();
            
            graphics2D.drawImage(bi, 0, 0,scaledSize.getWidth(),
                    scaledSize.getHeight(), null);

            /* graphics2D.drawImage(bi, 0, 0, scaledSize.getWidth(), scaledSize
                    .getHeight(), null);
            */

            // image icon blocks?? for sizing
            ImageIcon icon = new ImageIcon(thumbImage);
            ImageIO.write(thumbImage, "jpeg", new File(outFile));
        } catch (NullPointerException npe) {
            return;
        } catch (IOException ioe) {
            return;
        }
    }
    
    private static void doNativeRotate(String inFile, String outFile,
            String degrees) {
        String jpegTran = "jpegtran";
        if (System.getProperty("os.name").indexOf("Windows") > -1) {
            jpegTran = "\\jpegtran\\jpegtran";
        }
        String cmd = jpegTran + " -rotate " + degrees + " -outfile " + outFile
                + " " + inFile;

        if (System.getProperty("os.name").indexOf("Windows") > -1) {
            cmd = cmd.replaceAll("/", "\\\\");
        }
        //System.out.println("Before Call");
        //System.out.println(cmd);

        Runtime r = Runtime.getRuntime();

        try {
            Process p = r.exec(cmd);
            p.waitFor();
        } catch (Exception ex) {
            // set it to an hour from now
            nextTryNativeRotate = new Date((new Date()).getTime() + 60 * 60 * 1000);
            System.out.println("Native Rotate Failed . . . stop using native code for a while.");
            doJavaRotate(inFile, outFile, degrees);
        }
        //System.out.println("After Call");
    }
    
    private static void doNativeResize(String inFile, String outFile, int width,
            int height) {
        String jpegtran = "convert";
        if (System.getProperty("os.name").indexOf("Windows") > -1) {
            jpegtran = "\"\\Program Files\\ImageMagick\\convert.exe\"";
        }

        String cmd = jpegtran + " " + inFile + " -resize " + width + "x"
                + height + " " + outFile;

        if (System.getProperty("os.name").indexOf("Windows") > -1) {
            cmd = cmd.replaceAll("/", "\\\\");
        }
        //System.out.println("Before Resize Call");
        //System.out.println(cmd);

        Runtime r = Runtime.getRuntime();

        try {
            // Call the program
            Process p = r.exec(cmd);

            // Wait for it to complete.
            p.waitFor();

        } catch (Exception ex) {
            System.out.println("Caught exception " + ex.getMessage());
//          set it to an hour from now
            nextTryNativeResize = new Date((new Date()).getTime() + 60 * 60 * 1000);
            System.out.println("EXCEPTION: Native Resize Failed . . . stop using native code for a while.");
            doJavaResize(inFile, outFile, width, height);
        }

        //System.out.println("After Resize Call");
        
    }
    
    private static void doJavaCropResize(String inFile, String outFile, int width,
            int height) {
        try {
            // get the bufferred image from file
            BufferedImage bi = ImageIO.read(new File(inFile));
            // find correct width and height (scaledSize)
            /* 
            TwoSideBox tempBox = new TwoSideBox(width, height);
            TwoSideBox scaledSize = tempBox
                    .getSizeForFittingOutsideMe(new TwoSideBox(bi.getWidth(), bi
                            .getHeight())); */

            BufferedImage thumbImage = (new BufferedImage(width,
                    height, BufferedImage.TYPE_INT_RGB));
            
            // create graphics 2d attached to thumbimage
            Graphics2D graphics2D = thumbImage.createGraphics();
            
            // get side with closest shortest ratio from original
            // and set ratio to cal box size on original image
            double ratio = 0;
            if(((double)bi.getWidth()/(double)width)<
                    ((double)bi.getHeight()/(double)height))
            {
                ratio = (double)bi.getWidth()/(double)width;
            }else{
                ratio = (double)bi.getHeight()/(double)height;
            }
            int sW = (int)Math.round(ratio * width);
            int sH = (int)Math.round(ratio * height);
            int offX = (bi.getWidth()-sW)/2;
            int offY = (bi.getHeight()-sH)/2;
            
            // draw from bufferred image
            graphics2D.drawImage(bi.getSubimage(offX,offY,sW,sH), 0,0, width, height, null);

            // image icon blocks?? for sizing
            ImageIcon icon = new ImageIcon(thumbImage);
            ImageIO.write(thumbImage, "jpeg", new File(outFile));
        } catch (NullPointerException npe) {
            return;
        } catch (IOException ioe) {
            return;
        }
    }
    
    private static void doNativeCropResize(String inFile, String outFile, int width,
            int height) {

        BufferedImage bi;
        // get the bufferred image from file
        try{
            bi = ImageIO.read(new File(inFile));
        }catch(IOException ioe){
            return;
        }
        
        // get side with closest shortest ratio from original
        // and set ratio to cal box size on original image
        double ratio = 0;
        if(((double)bi.getWidth()/(double)width)<
                ((double)bi.getHeight()/(double)height))
        {
            ratio = (double)bi.getWidth()/(double)width;
        }else{
            ratio = (double)bi.getHeight()/(double)height;
        }
        int sW = (int)Math.round(ratio * width);
        int sH = (int)Math.round(ratio * height);
        
        
        String jpegtran = "convert";
        if (System.getProperty("os.name").indexOf("Windows") > -1) {
            jpegtran = "\"\\Program Files\\ImageMagick\\convert.exe\"";
        }

        String cmd = jpegtran + " " + inFile + " -gravity Center -crop " +sW+"x"+sH+"+0+0 " 
                + "-resize " + width + "x"
                + height + " " + outFile;

        if (System.getProperty("os.name").indexOf("Windows") > -1) {
            cmd = cmd.replaceAll("/", "\\\\");
        }
        //System.out.println("Before Resize Call");
        //System.out.println(cmd);

        Runtime r = Runtime.getRuntime();

        try {
            // Call the program
            Process p = r.exec(cmd);

            // Wait for it to complete.
            p.waitFor();

        } catch (Exception ex) {
            System.out.println("Caught exception " + ex.getMessage());
//          set it to an hour from now
            nextTryNativeResize = new Date((new Date()).getTime() + 60 * 60 * 1000);
            System.out.println("EXCEPTION: Native Resize Failed . . . stop using native code for a while.");
            doJavaCropResize(inFile, outFile, width, height);
        }

    }
}
