package gcr.mmm2.server.servlets;

import gcr.mmm2.model.IPhoto;
import gcr.mmm2.model.MediaManager;
import gcr.mmm2.model.PhotoFactory;
import gcr.mmm2.rdb.RDBConnection;
import gcr.mmm2.rdb.ResultSetWrapper;

import java.awt.Canvas;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.Properties;
import java.util.Random;

import javax.imageio.ImageIO;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGImageEncoder;

/**
 * Creates a mosaic image from annotation data. More for fun than actually
 * useful.
 * 
 * @author Benjamin
 * 
 */
public class MakeMosaicServlet extends HttpServlet {

    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 1314198005327198350L;

    // Image notify for no good reason.
    private final Canvas canvas = new Canvas();

    /**
     * @param request
     * @param response
     * @throws ServletException
     * @see javax.servlet.http.HttpServlet#doGet(javax.servlet.http.HttpServletRequest,
     *      javax.servlet.http.HttpServletResponse)
     */
    public void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException {

        response.reset();
        response.setContentType("image/jpg");

        String userName = request.getParameter("name");
        if (userName == null) {
            throw new ServletException("Missing parameter 'name'");
        }

        BufferedImage bi = new BufferedImage(1024, 768,
                BufferedImage.TYPE_INT_RGB);
        Graphics2D biContext = bi.createGraphics();

        String query = "select pip.*, person.first || ' ' || person.last as name, photo.image_file  "
                + "from person_in_photo pip, person, photo  "
                + "where pip.person_id=person.id and photo.id = pip.photo_id "
                + "and person.first || ' ' || person.last='" + userName + "'";
        final Hashtable faces;

        try {
            faces = hashQuery(query, "id", true);

            Random rnd = new Random();

            final Enumeration ids = faces.keys();
            while (ids.hasMoreElements()) {
                String faceid = (String) ids.nextElement();
                Properties face = (Properties) faces.get(faceid);

                IPhoto photo = PhotoFactory.getPhotoByID(Integer.parseInt(face
                        .getProperty("photo_id")));
                String filename = photo.getJPEGFilename();
                filename = MediaManager.getRealPath(filename);

                BufferedImage biface = ImageIO.read(new File(filename));

                int x = rnd.nextInt(900);
                int y = rnd.nextInt(600);

                int left = Integer
                        .parseInt(face.getProperty("left_coordinate"));
                int top = Integer.parseInt(face.getProperty("top_coordinate"));
                int width = Integer.parseInt(face.getProperty("width"));
                int height = Integer.parseInt(face.getProperty("height"));

                biContext.drawImage(biface, x, y, x + (width), y + (height),
                        left, top, left + width, top + height, this.canvas);

            }

            ServletOutputStream sos = response.getOutputStream();
            JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(sos);
            encoder.encode(bi);
        } catch (Exception e1) {
            throw new ServletException(e1 + ":" + query);
        }
    }

    Hashtable hashQuery(final String query, final String keyID,
            final boolean isUnique) throws SQLException {
        final Hashtable results = new Hashtable();
        final ResultSetWrapper rsw = RDBConnection.executeQuery(query);

        ResultSet rs = rsw.getResultSet();
        ResultSetMetaData rsmd = rs.getMetaData();
        int colCount = rsmd.getColumnCount();
        while (rs.next()) {
            final Properties row = new Properties();
            for (int i = 1; i <= colCount; i++) {
                if (rs.getString(i) != null) {
                    row.setProperty(rsmd.getColumnName(i), rs.getString(i));
                }
            }

            final String key = row.getProperty(keyID);
            if (isUnique) {
                results.put(key, row);
            } else {
                if (!results.containsKey(key)) {
                    results.put(key, new LinkedList());
                }
                LinkedList ll = (LinkedList) results.get(key);
                ll.addLast(row);
            }
        }

        rsw.closeAll();
        return results;
    }

}
