package gcr.mmm2.server.servlets.mobile;

import gcr.mmm2.model.ContextManager;
import gcr.mmm2.model.IContext;
import gcr.mmm2.model.IPhoto;
import gcr.mmm2.model.IUser;
import gcr.mmm2.model.PermissionException;
import gcr.mmm2.model.PhotoFactory;
import gcr.mmm2.model.StringFormatException;
import gcr.mmm2.model.UserManager;
import gcr.mmm2.util.MMM2Properties;
import gcr.mmm2.util.StringUtils;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.RandomAccessFile;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.StringTokenizer;
import java.util.logging.Logger;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

public class ContextAppServlet extends HttpServlet {

    static protected final Logger log = Logger
            .getLogger("contextUploadActivity");

    public static String getLogUploadDirectory(IUser user) throws IOException {
        String dir = MMM2Properties.getProperty("UPLOAD_DIR") + "/"
                + user.getID();
        File dirFile = new File(dir);
        if (dirFile.exists()) {
            if (!dirFile.isDirectory()) {
                throw new IOException("Fatal error in getLogUploadDirectory : '" + dir
                        + "' is a file, not a directory.");
            }
        } else {
            if (!dirFile.mkdirs()) {
                throw new IOException("Unable to create directory '" + dir
                        + "'");
            }
        }
        return dir;
    }

    private void addPhoto(IUser user, String filename, String dateTime,
            File metadata) {

        IPhoto p = PhotoFactory.getPhotoByFilename(filename);
        if (p != null) {
            // this photo has already been added to the db by the shareMobile
            // servlet
            log.info("UID=" + user.getID() + " - photo '" + filename
                    + "' already exists");
        } else {
            // parse the xml file and add it to the db
            log.info("UID=" + user.getID() + " - photo '" + filename
                    + "' does not exist. Parsing  ('" + metadata
                    + "') and adding");

            try {
                p = PhotoFactory.createPhoto(filename, user, StringUtils
                        .parseContextDate(dateTime));
                setMetaDataContext(p, user, dateTime, metadata);

            } catch (StringFormatException sfe) {
                sfe.printStackTrace();
                log.info(sfe.getMessage());
            } catch (PermissionException e) {
                e.printStackTrace();
                log.info(e.getMessage());
            }
        }

    }

    /** Called when someone accesses the servlet. */
    public void doGet(HttpServletRequest request, HttpServletResponse response) {
        doPost(request, response);
    }

    public void doPost(HttpServletRequest request, HttpServletResponse response) {

        ServletOutputStream out;
        try {
            out = response.getOutputStream();
        } catch (IOException e) {
            e.printStackTrace();
            return;
        }

        String uploadFilePath = null;
        String imsi = null;
        String fileName = null;
        String qs = request.getQueryString();
        String lprefix = "";

        // split and process the query string tokens
        String[] tokens = qs.split(";");
        String action = tokens[0];
        String meta = tokens[1];
        long offset = Long.parseLong(tokens[2]);
        String filen;
        try {
            filen = URLDecoder.decode(tokens[3], "UTF-8");
        } catch (UnsupportedEncodingException e1) {
            e1.printStackTrace();
            return;
        }
        if (filen == "") {
            response(201, "Error: No filen given", out);
            log.info(lprefix + "- no filename given, returning 201");
            return;
        }
        imsi = filen.substring(0, filen.indexOf('/'));
        fileName = filen.substring(filen.indexOf('/') + 1);

        IUser user = UserManager.getByIMSI(imsi);
        if (user == null) {
            response(500, "uploader not known", out);
            log.info("IMSI=" + imsi
                    + " - Uploader IMSI not known, returning 500");
            return;
        }
        lprefix = lprefix + "UID=" + user.getID() + " ";

        // set up the upload dir
        try {
            if (meta.length() == 0) {
                uploadFilePath = getLogUploadDirectory(user) + "/" + fileName;
            } else {
                uploadFilePath = user.getUploadDirectory() + "/" + fileName;
            }
        } catch (IOException e2) {
            e2.printStackTrace();
            uploadFilePath = MMM2Properties.getProperty("UPLOAD_DIR") + "/"
                    + imsi + "_" + fileName;
        }

        log.info(lprefix + "ACTION=" + action + " FILENAME=" + fileName
                + " OFFSET=" + offset);

        if (action.equals("P") && fileName.equalsIgnoreCase("cellid_names.txt")) {
            updateCellIDNames(uploadFilePath);
        }

        String extension = fileName.substring(fileName.length() - 4, fileName
                .length());
        String filePathNoExtension = uploadFilePath.substring(0, uploadFilePath
                .length() - 4);

        if (action.equals("C") || action.equals("A") || action.equals("P")) {
            InputStream is;
            try {
                is = request.getInputStream();
            } catch (IOException e2) {
                e2.printStackTrace();
                response(201, "Could Not Read Input Stream", out);
                log.info(lprefix + "ACTION=" + action + " FILENAME=" + fileName
                        + " OFFSET=" + offset
                        + " - could not read input stream");
                return;
            }

            ByteArrayOutputStream bos = new ByteArrayOutputStream(50000);
            File file = null;

            int c = 0; // content
            int i = 0; // index
            try {
                while ((c = is.read()) != -1) {
                    bos.write((byte) c);
                    i++;
                }
            } catch (IOException e3) {
                e3.printStackTrace();
                return;
            }
            log.info(lprefix + "ACTION=" + action + " FILENAME=" + fileName
                    + " OFFSET=" + offset + " - read " + i + " bytes");

            // rewrite the byte array to a (new)file
            byte[] fileBytes = bos.toByteArray();
            if (fileBytes.length > 0) {

                if (action.equals("P")) { // PACKET
                    file = new File(filePathNoExtension + ".xml");
                    offset = 0; // reset the offset because the xml gets its own
                    // file
                } else {
                    file = new File(uploadFilePath);
                }

                /*
                 * Open a Random Access file, so we can seek and write data
                 */
                try {
                    RandomAccessFile ra = new RandomAccessFile(file, "rws");
                    if (ra.length() < offset) {
                        ra.setLength(offset);
                    }
                    ra.seek(offset);
                    ra.write(fileBytes);
                    ra.close();
                } catch (IOException ioe) {
                    ioe.printStackTrace();
                    return;
                }

                /*
                 * The XML Packet has been received, so the media upload is
                 * completed Rename the media file and put it in the MMM
                 * relational DB
                 */
                if (action.equals("P")) {

                    String dateTime = getDateTime(file);
                    String newFileName = user.getID() + "M" + dateTime
                            + extension;

                    String renamedFilePath;
                    try {
                        renamedFilePath = user.getUploadDirectory() + "/"
                                + newFileName;
                    } catch (IOException e4) {
                        e4.printStackTrace();
                        renamedFilePath = MMM2Properties
                                .getProperty("UPLOAD_DIR")
                                + "/"
                                + user.getID()
                                + "_"
                                + imsi
                                + "_"
                                + newFileName;
                    }

                    File origFile = new File(uploadFilePath);
                    File newFile = new File(renamedFilePath);

                    boolean success = origFile.renameTo(newFile);
                    if (!success) {
                        log.info(lprefix + "ACTION=" + action + " FILENAME="
                                + fileName + " OFFSET=" + offset
                                + " - file was not successfully renamed");
                    } else {
                        log.info(lprefix + "ACTION=" + action + " FILENAME="
                                + fileName + " OFFSET=" + offset
                                + " - file renamed to " + renamedFilePath);
                    }

                    addPhoto(user, newFileName, dateTime, file);
                }
            }
        }
        response(200, "OK", out);
    }// end doPost()

    private String getDateTime(File metadata) {

        String dateTime = "";

        try {
            DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory
                    .newInstance();
            DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();

            Document doc = docBuilder.parse(metadata);

            doc.getDocumentElement().normalize();

            NodeList events = doc.getElementsByTagName("event");

            for (int s = 0; s < events.getLength(); s++) {

                Node eventNode = events.item(s);
                if (eventNode.getNodeType() == Node.ELEMENT_NODE) {

                    NodeList eventItems = eventNode.getChildNodes();

                    if (eventItems.item(1).getNodeName() == "location") {
                        dateTime = eventItems.item(0).getFirstChild()
                                .getNodeValue();
                    }
                }// end of if clause
            }// end of for loop with s var

        } catch (SAXParseException err) {
            err.printStackTrace();

        } catch (SAXException e) {
            e.printStackTrace();

        } catch (Throwable t) {
            t.printStackTrace();
        }

        return dateTime;
    }

    public void response(int status, String msg, ServletOutputStream out) {
        try {
            out.print("Content-Type: text/html\n");
            out.print("Status: " + status + "\n");
            out.print("\n");
            out.print(msg);
            out.print("\n");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void setMetaDataContext(IPhoto p, IUser user, String dateTime,
            File metadata) throws PermissionException, StringFormatException {

        IContext context = ContextManager.createContext(StringUtils
                .parseContextDate(dateTime));
        String lprefix = "[CAServlet] UID=" + user.getID() + " PhotoID="
                + p.getID() + " ";

        try {

            // String dateTime = "";
            String tag = "";
            String description = "";
            String network = "";
            String lac = "";
            String cellID = "";

            DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory
                    .newInstance();
            DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();

            Document doc = docBuilder.parse(metadata);
            doc.getDocumentElement().normalize();

            Node tagNode = doc.getElementsByTagName("tag").item(0)
                    .getFirstChild();
            if (tagNode != null)
                tag = tagNode.getNodeValue();
            Node descriptionNode = doc.getElementsByTagName("description")
                    .item(0).getFirstChild();
            if (descriptionNode != null)
                description = descriptionNode.getNodeValue();

            p.setTag(tag, user);
            p.setCaption(description, user);

            NodeList events = doc.getElementsByTagName("event");

            for (int s = 0; s < events.getLength(); s++) {

                Node eventNode = events.item(s);
                if (eventNode.getNodeType() == Node.ELEMENT_NODE) {

                    NodeList eventItems = eventNode.getChildNodes();

                    if (eventItems.item(1).getNodeName() == "location") {
                        dateTime = eventItems.item(0).getFirstChild()
                                .getNodeValue();
                        if (eventItems.item(1).getFirstChild().getChildNodes() != null) {
                            network = eventItems.item(1).getFirstChild()
                                    .getChildNodes().item(0).getFirstChild()
                                    .getNodeValue();
                            lac = eventItems.item(1).getFirstChild()
                                    .getChildNodes().item(1).getFirstChild()
                                    .getNodeValue();
                            cellID = eventItems.item(1).getFirstChild()
                                    .getChildNodes().item(2).getFirstChild()
                                    .getNodeValue();
                        }

                        log.info(lprefix + "DT=" + dateTime + " NET=" + network
                                + " LAC=" + lac + " CellID=" + cellID);

                        context.setCellLocation(cellID, lac, network);
                    }
                    if (eventItems.item(1).getNodeName() == "bluetooth") {
                        if (eventItems.item(1).getChildNodes().item(0)
                                .getFirstChild() != null) {
                            String btDeviceList = eventItems.item(1)
                                    .getChildNodes().item(0).getFirstChild()
                                    .getNodeValue();
                            StringTokenizer st = new StringTokenizer(
                                    btDeviceList, "]");
                            while (st.hasMoreTokens()) {
                                String token = st.nextToken().trim();
                                if (token.indexOf("[") > 0) {
                                    int bracket = token.indexOf("[");
                                    String btMac = token.substring(0, bracket)
                                            .trim();
                                    String btName = token
                                            .substring(bracket + 1).trim();
                                    context.addBluetoothDevice(btMac, btName);
                                }
                            }
                            log.info(lprefix + "BT_PROX=" + btDeviceList);
                        }
                    }
                    System.out.println(eventItems.item(1).getNodeName());
                    if (eventItems.item(1).getNodeName() == "gps") {
                        String gpsData = null;
                        try {
                            gpsData = eventItems.item(1).getFirstChild()
                                    .getNodeValue();
                        } catch (NullPointerException npe) {
                            // gpsData = null
                        }

                        if (gpsData == null) {
                            log.info(lprefix + "- No GPS Data");
                        } else {
                            log.info(lprefix + "GPS=" + gpsData);
                            StringTokenizer st = new StringTokenizer(gpsData,
                                    ",");
                            st.nextToken(); // NMEA GPS message format
                            String gps_utc_time = st.nextToken();
                            String status = st.nextToken(); // status

                            if (status.equals("A")) { // status: A = valid, V
                                // = invalid
                                String gps_lat = st.nextToken();
                                String gps_ns_indicator = st.nextToken();

                                String gps_lat_dd = gps_lat.substring(0, 2);
                                if (gps_ns_indicator.equals("S"))
                                    gps_lat_dd = "-" + gps_lat_dd;
                                float gps_lat_mm = Float.parseFloat(gps_lat
                                        .substring(2, 4));
                                float gps_lat_ss = 60 * Float
                                        .parseFloat(gps_lat.substring(4));
                                float total_lat_seconds = (gps_lat_mm * 60)
                                        + gps_lat_ss;

                                String gps_lat_decimal;
                                if (gps_ns_indicator.equals("S"))
                                    gps_lat_decimal = String.valueOf(Float
                                            .parseFloat(gps_lat_dd)
                                            - (total_lat_seconds / 3600));
                                else
                                    gps_lat_decimal = String.valueOf(Float
                                            .parseFloat(gps_lat_dd)
                                            + (total_lat_seconds / 3600));

                                String gps_long = st.nextToken();
                                String gps_ew_indicator = st.nextToken();

                                String gps_long_ddd = gps_long.substring(0, 3);
                                if (gps_ew_indicator.equals("W"))
                                    gps_long_ddd = "-" + gps_long_ddd;
                                float gps_long_mm = Float.parseFloat(gps_long
                                        .substring(3, 5));
                                float gps_long_ss = 60 * Float
                                        .parseFloat(gps_long.substring(5));
                                float total_long_seconds = (gps_long_mm * 60)
                                        + gps_long_ss;

                                String gps_speed = st.nextToken();
                                String gps_direction = st.nextToken();
                                String gps_utc_date = st.nextToken();

                                String gps_long_decimal;
                                if (gps_ew_indicator.equals("W"))
                                    gps_long_decimal = String.valueOf(Float
                                            .parseFloat(gps_long_ddd)
                                            - (total_long_seconds / 3600));
                                else
                                    gps_long_decimal = String.valueOf(Float
                                            .parseFloat(gps_long_ddd)
                                            + (total_long_seconds / 3600));

                                context.setGpsLocation(gps_lat_decimal,
                                        gps_long_decimal, gps_speed,
                                        gps_direction);
                            }
                        }
                    }
                }// end of if clause

            }// end of for loop with s var

        } catch (SAXParseException err) {
            err.printStackTrace();
            System.err.println(err.getMessage());
        } catch (SAXException se) {
            se.printStackTrace();
            System.err.println(se.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            System.err.println(e.getMessage());
        }

        p.setContext(context, user);
    }

    private void updateCellIDNames(String cellIDFile) {

        // TODO: provide way to do batch update of CellID names in IContext
    }

}