package gcr.mmm2.server.servlets.mobile;

import gcr.mmm2.model.ContextManager;
import gcr.mmm2.model.IContext;
import gcr.mmm2.model.IPhoto;
import gcr.mmm2.model.IUser;
import gcr.mmm2.model.PhotoFactory;
import gcr.mmm2.model.UserManager;
import gcr.mmm2.util.StringUtils;

import java.util.StringTokenizer;
import java.util.logging.Logger;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class ShareMobileServlet extends HttpServlet {

    static protected final Logger log = Logger
            .getLogger("contextUploadActivity");

    public void doPost(HttpServletRequest request, HttpServletResponse response) {

        try {
            String lprefix = "[SMServlet] ";

            response.setContentType("text/html");
            // ServletOutputStream out = response.getOutputStream();

            /*
             * Retrive User By IMSI
             */
            String imsi = request.getParameter("user");
            IUser user = UserManager.getByIMSI(imsi);
            if (user == null) {
                log.warning("Failed to get user by IMSI:" + imsi);
                throw new ServletException("Unable to get user by IMSI");
            }
            lprefix = lprefix + "UID=" + user.getID() + " ";

            /*
             * Get datetime from request string
             */
            String dateTime = request.getParameter("datetime");
            if (dateTime == null) {
                log.warning("Failed to get dateTime");
                throw new ServletException("Unable to get parameter datetime");
            }
            
            IContext context = ContextManager.createContext(StringUtils
                    .parseContextDate(dateTime));

            String filename = user.getID() + "M" + dateTime + ".jpg";
            IPhoto p = PhotoFactory.getPhotoByFilename(filename);
            if (p == null) {
                p = PhotoFactory.createPhoto(filename, user, StringUtils
                        .parseContextDate(dateTime));
            }
            lprefix = lprefix + "PhotoID=" + p.getID() + " ";

            /*
             * Get Photo Metadata
             */
            String tag = request.getParameter("tag");
            if (tag == null)
                tag = "";

            String description = request.getParameter("desc");
            if (description == null)
                description = "";

            p.setTag(tag, user);
            p.setCaption(description, user);
            log.info(lprefix + "TAG='" + tag + "' DESC='" + description + "'");

            /*
             * Get Cell/Location Metadata
             */
            String network = request.getParameter("network");
            if (network == null)
                network = "";

            String lac = request.getParameter("lac");
            if (lac == null)
                lac = "";

            String cellID = request.getParameter("cellid");
            if (cellID == null)
                cellID = "";

            context.setCellLocation(cellID, lac, network);
            log.info(lprefix + "DT=" + dateTime + " NET=" + network + " LAC="
                    + lac + " CellID=" + cellID);

            /*
             * Get Bluetooth Proximity Data
             */
            String btDeviceList = request.getParameter("btlist");
            if (btDeviceList == null) {
                btDeviceList = "";
            }

            // bt stuff
            StringTokenizer st = new StringTokenizer(btDeviceList, "]");
            while (st.hasMoreTokens()) {
                String token = st.nextToken().trim();
                if (token.indexOf("[") > 0) {
                    int bracket = token.indexOf("[");
                    String btMac = token.substring(0, bracket).trim();
                    String btName = token.substring(bracket + 1).trim();
                    context.addBluetoothDevice(btMac, btName);
                }
            }
            log.info(lprefix + "BT_PROX=" + btDeviceList);

            /*
             * Get GPS Data
             */
            String gpsData = request.getParameter("gpsdata");
            if (gpsData == null || ("").equals(gpsData)) {
                log.info(lprefix + "- No GPS Data");
            } else {
                log.info(lprefix + "GPS=" + gpsData);
                st = new StringTokenizer(gpsData, ",");
                st.nextToken(); // NMEA GPS message format
                String gps_utc_time = st.nextToken();
                String status = st.nextToken(); // status

                if (status.equals("A")) { // status: A = valid, V = invalid
                    String gps_lat = st.nextToken();
                    String gps_ns_indicator = st.nextToken();

                    String gps_lat_dd = gps_lat.substring(0, 2);
                    if (gps_ns_indicator.equals("S"))
                        gps_lat_dd = "-" + gps_lat_dd;
                    float gps_lat_mm = Float
                            .parseFloat(gps_lat.substring(2, 4));
                    float gps_lat_ss = 60 * Float.parseFloat(gps_lat
                            .substring(4));
                    float total_lat_seconds = (gps_lat_mm * 60) + gps_lat_ss;

                    String gps_lat_decimal;
                    if (gps_ns_indicator.equals("S"))
                        gps_lat_decimal = String.valueOf(Float
                                .parseFloat(gps_lat_dd)
                                - (total_lat_seconds / 3600));
                    else
                        gps_lat_decimal = String.valueOf(Float
                                .parseFloat(gps_lat_dd)
                                + (total_lat_seconds / 3600));

                    String gps_long = st.nextToken();
                    String gps_ew_indicator = st.nextToken();

                    String gps_long_ddd = gps_long.substring(0, 3);
                    if (gps_ew_indicator.equals("W"))
                        gps_long_ddd = "-" + gps_long_ddd;
                    float gps_long_mm = Float.parseFloat(gps_long.substring(3,
                            5));
                    float gps_long_ss = 60 * Float.parseFloat(gps_long
                            .substring(5));
                    float total_long_seconds = (gps_long_mm * 60) + gps_long_ss;

                    String gps_speed = st.nextToken();
                    String gps_direction = st.nextToken();
                    String gps_utc_date = st.nextToken();

                    String gps_long_decimal;
                    if (gps_ew_indicator.equals("W"))
                        gps_long_decimal = String.valueOf(Float
                                .parseFloat(gps_long_ddd)
                                - (total_long_seconds / 3600));
                    else
                        gps_long_decimal = String.valueOf(Float
                                .parseFloat(gps_long_ddd)
                                + (total_long_seconds / 3600));

                    context.setGpsLocation(gps_lat_decimal, gps_long_decimal,
                            gps_speed, gps_direction);
                }
            }
            p.setContext(context, user);

            response.sendRedirect("./mobile/share_mobile.jsp?imsi=" + imsi
                    + "&photoID=" + p.getID());

        } catch (Exception e) {
            System.out.println(e.getMessage());
            e.printStackTrace();
        }
    }

    /** Called when someone accesses the servlet. */
    public void doGet(HttpServletRequest request, HttpServletResponse response) {
        doPost(request, response);
    }

}
