/*
 * Created on Jul 27, 2005
 *
 */
package gcr.mmm2.tag;

import java.util.LinkedHashSet;
import java.util.Set;
import java.util.TreeSet;

/**
 * Abstract taggable interface, allowing string based getting and setting of
 * metadata.
 * 
 * @author Benjamin
 * 
 */
public abstract class Taggable {

    /**
     * A grab-bag type of tags.
     */
    public static final int OTHER_FREE_TAG = 1;

    public static String getTagDescription(int type) {
        return null;
    }

    public static String[] getTagValues(int type, Taggable[] items) {
        Set tags = new TreeSet();
        for (int i = 0; i < items.length; i++) {
            String[] tagValues = items[i].getTag(type);
            for (int j = 0; tagValues != null && j < tagValues.length; j++) {
                if (tagValues[j]!=null && !"".equals(tagValues[j])) {
                    tags.add(tagValues[j]);
                }
            }
        }
        
        return (String[]) tags.toArray(new String[0]);
    }

    public static int[] getTypes() {
        return null;
    }

    /**
     * 
     * @param type
     * @return if the given type can have more than one value
     */
    public static boolean isListType(int type) {
        return false;
    }

    /**
     * 
     * @param type
     * @return if the given type can be set (as opposed to contextual info)
     */
    public static boolean isWritable(int type) {
        return false;
    }

    public static void validType(int type) {
        for (int i = 0; i < getTypes().length; i++)
            if (getTypes()[i] == type)
                return;
        throw new IllegalArgumentException("Invalid tag type:" + type);
    }

    /**
     * Adds a tag to a tag list or sets a value of a single tag
     * 
     * @param type
     * @param value
     * @throws IllegalArgumentException
     *             if not a recognised type OR if trying to set a fixed tag
     *             type.
     */
    public abstract void addOrSetTag(int type, String value)
            throws IllegalArgumentException;

    /**
     * 
     * @return list of tag types with valid non null values.
     */
    public abstract int[] getNonEmptyTagTypes();

    /**
     * Gets tags. If it is a single value type, only returns an array of 1.
     * 
     * @param type
     *            LIST type allowed
     * @return value of tags
     * @throws IllegalArgumentException
     *             if not a recognised type.
     */
    public abstract String[] getTag(int type) throws IllegalArgumentException;

    public int getTagOverlap(int type, Taggable other) {
        String[] myValues = this.getTag(type);
        String[] otherValues = other.getTag(type);

        if (myValues == null || otherValues == null)
            return 0;

        int matches = 0;
        for (int i = 0; i < myValues.length; i++) {
            for (int j = i; j < otherValues.length; j++) {

                if (myValues[i] != null && otherValues[j] != null
                        && myValues[i].equals(otherValues[j]))
                    matches++;
            }
        }
        return matches;
    }

    public static Taggable[] filterOnTag(int type, String value, Taggable[] list) {
        Set out = new LinkedHashSet();
        for(int i=0; i<list.length; i++) {
            String[] tags = list[i].getTag(type);
            for(int j=0; tags!=null && j<tags.length; j++ ) {
                //System.out.println(value + "=?"+ tags[j]);
                if(value.equals(tags[j])) {
                    out.add(list[i]);
                }
            }
        }
        return (Taggable[])out.toArray(new Taggable[0]);
    }
    
    /**
     * Unsets a single value tag or removes an entry from a tag list
     * 
     * @param type
     * @param value
     *            ignored if removing from a single value list
     * @throws IllegalArgumentException
     *             if not a recognised type or not a writable type.
     */
    public abstract void removeTag(int type, String value)
            throws IllegalArgumentException;
}
