package gcr.mmm2.util;

import java.awt.Point;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.*;

public class RankedNearness {
    
    private Point origin;
    private Rectangle photoSize;
    private int photoPadding;
    private Rectangle screenSize;
    private Rectangle boxSize;
    private ArrayList sorted;
    
    public RankedNearness(Point origin, Rectangle photoSize,
            int photoPadding, Rectangle screenSize)
    {
        this.origin = origin;
        this.photoSize = photoSize;
        this.photoPadding = photoPadding;
        this.screenSize = screenSize;
        this.boxSize = new Rectangle((int)(Math.round(photoSize.getWidth())+photoPadding),
                (int)(Math.round(photoSize.getHeight())+photoPadding));
        
        int effectiveScreenWidth = (int)(screenSize.getWidth() - boxSize.getWidth());
        int effectiveScreenHeight = (int)(screenSize.getHeight() - boxSize.getHeight());
        
        sorted = new ArrayList();
        for(int y=0;y<effectiveScreenHeight;y+=boxSize.getHeight())
        {
            for(int x=0;x<effectiveScreenWidth;x+=boxSize.getWidth())
            {
                CompDistPoint cdp = new CompDistPoint(origin,new Point(x,y),boxSize,photoSize);
                sorted.add(cdp);
            }
        }
        Collections.sort(sorted);
    }

    /**
     * @return Returns the origin.
     */
    public Point getOrigin() {
        return origin;
    }

    /**
     * @return Returns the photoPadding.
     */
    public int getPhotoPadding() {
        return photoPadding;
    }

    /**
     * @return Returns the photoSize.
     */
    public Rectangle getPhotoSize() {
        return photoSize;
    }
    
    /**
     * @return Returns the box size (with padding).
     */
    public Rectangle getBoxSize() {
        int width = (int)(photoSize.getWidth() + photoPadding);
        int height = (int)(photoSize.getHeight() + photoPadding);
        return new Rectangle(width,height);
    }

    /**
     * @return Returns the screenSize.
     */
    public Rectangle getScreenSize() {
        return screenSize;
    }
    
    public int size()
    {
        return sorted.size();
    }
    
    public CompDistPoint get(int i)
    {
        return (CompDistPoint)sorted.get(i);
    }

}
