/*
 * Created on Nov 16, 2004
 *
 */
package gcr.mmm2.util;

import gcr.mmm2.model.IPhoto;
import gcr.mmm2.model.IUser;
import gcr.mmm2.model.MediaManager;
import gcr.mmm2.model.PhotoFactory;
import gcr.mmm2.model.StringFormatException;
import gcr.mmm2.model.UserManager;
import gcr.mmm2.rdb.RDBConnection;
import gcr.mmm2.rdb.ResultSetWrapper;

import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

/**
 * @author Jeff Towle
 * 
 */
public class StatCalculator {

    public static final Locale locale = Locale.US;

    public static final Format daySliceFormatter = new SimpleDateFormat("yyyy-MM-dd",
            locale);

    // public static String gcrMembers =
    // "1,10,11,12,36,37,140,141,142,144,145,1000,1001,1029,1031,1044,1091,1114,1090,1162,1163,1164,1165,1166,1199,1201,1045,13,143,1215";
    // public static int[] gcrMembersList =
    // {1,10,11,12,36,37,140,141,142,144,145,1000,1001,1029,1031,1044,1091,1114,1090,1162,1163,1164,1165,1166,1199,1201,1045,13,143,1215};

    public static int getPhotoCount(boolean includeGCR) {
        String query = "select count(id) from photo";
        if (!includeGCR)
            query += " where created_by not in (" + UserManager.gcrUsers + ");";
        else
            query += ";";
        return getCount(query);
    }

    public static int getDeletedPhotoCount(boolean includeGCR) {
        String query = "select count(id) from photo where date_inactive is not null";
        if (!includeGCR)
            query += " and created_by not in (" + UserManager.gcrUsers + ");";
        else
            query += ";";
        return getCount(query);
    }

    public static int getAlbumCount(boolean includeGCR) {
        String query = "select count(id) from album";
        if (!includeGCR)
            query += " where created_by not in (" + UserManager.gcrUsers + ");";
        else
            query += ";";
        return getCount(query);
    }

    public static int getShareCount(boolean includeGCR) {
        String query = "select count(id) from share";
        if (!includeGCR)
            query += " where created_by not in (" + UserManager.gcrUsers + ");";
        else
            query += ";";
        return getCount(query);
    }

    public static int getUnsuccessfulUploadCount(boolean includeGCR) {
        int failed = 0;
        String query = "select id from photo";
        if (!includeGCR)
            query += " where created_by not in (" + UserManager.gcrUsers + ");";
        else
            query += ";";
        List photos = listPhotos(query);
        Iterator p_it = photos.iterator();
        while (p_it.hasNext()) {
            IPhoto p = (IPhoto) p_it.next();
            try {
                String filename = MediaManager.getRealPath(p.getJPEGFilename());
                if (!filename.endsWith(p.getJPEGFilename())) {
                    failed++;
                }
            } catch (StringFormatException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }

        }

        return failed;
    }

    /*
     * User-Specific Functions
     */
    public static int getPhotoCountByUser(IUser u, Date d) {
        String query = "select count(id) from photo where created_by="
                + u.getID();
        if (d != null) {
            query += " and date_added ~ '" + daySliceFormatter.format(d) + "'";
        }
        query += ";";
        return getCount(query);
    }

    public static int getDeletedPhotoCountByUser(IUser u, Date d) {
        String query = "select count(id) from photo where created_by="
                + u.getID() + " and date_inactive";
        if (d != null) {
            query += " ~ '" + daySliceFormatter.format(d) + "'";
        } else {
            query += " is not null";
        }
        return getCount(query);
    }

    public static int getShareCountByUser(IUser u, Date d) {
        String query = "select count(id) from share where created_by="
                + u.getID();
        if (d != null) {
            query += " and date_added ~ '" + daySliceFormatter.format(d) + "'";
        }
        query += ";";
        return getCount(query);
    }

    public static int getSharedPhotoCountByUser(IUser u, Date d) {
        String query = "select count(sp.id) from share_photo sp, share s where s.created_by="
                + u.getID() + " and s.id=sp.share_id";
        if (d != null) {
            query += " and sp.date_added ~ '" + daySliceFormatter.format(d)
                    + "'";
        }
        query += ";";
        return getCount(query);
    }

    public static int getSharedAlbumCountByUser(IUser u, Date d) {
        String query = "select count(sa.id) from share_album sa, share s where s.created_by="
                + u.getID() + " and s.id=sa.share_id";
        if (d != null) {
            query += " and sa.date_added ~ '" + daySliceFormatter.format(d)
                    + "'";
        }
        query += ";";
        return getCount(query);
    }

    public static int getReceivedShareCountByUser(IUser u, Date d) {
        String query = "select count(id) from share_recipient where recipient="
                + u.getID();
        if (d != null) {
            query += " and date_added ~ '" + daySliceFormatter.format(d) + "'";
        }
        query += ";";
        return getCount(query);
    }

    public static int getAlbumCountByUser(IUser u, Date d) {
        String query = "select count(id) from album where created_by="
                + u.getID();
        if (d != null) {
            query += " and date_added ~ '" + daySliceFormatter.format(d) + "'";
        }
        query += ";";
        return getCount(query);
    }

    public static int getContactListCountByUser(IUser u, Date d) {
        String query = "select count(id) from contact_list where created_by="
                + u.getID();
        if (d != null) {
            query += " and date_added ~ '" + daySliceFormatter.format(d) + "'";
        }
        query += ";";
        return getCount(query);
    }

    public static int getContactCountByUser(IUser u, Date d) {
        String query = "select count(id) from contact where created_by="
                + u.getID();
        if (d != null) {
            query += " and date_added ~ '" + daySliceFormatter.format(d) + "'";
        }
        query += ";";
        return getCount(query);
    }

    public static int getUnsuccessfulUploadCountByUser(IUser u, Date d) {
        int failed = 0;
        String query = "select id from photo where created_by=" + u.getID();

        if (d != null) {
            query += " and date_added ~ '" + daySliceFormatter.format(d) + "'";
        }
        query += ";";
        List photos = listPhotos(query);
        Iterator p_it = photos.iterator();
        while (p_it.hasNext()) {
            IPhoto p = (IPhoto) p_it.next();
            try {
                String filename = MediaManager.getRealPath(p.getJPEGFilename());
                if (!filename.endsWith(p.getJPEGFilename())) {
                    failed++;
                }
            } catch (StringFormatException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }

        }

        return failed;
    }

    /*
     * Aggregate Functions
     */

    public static int getUploadCountForDate(Date d) {
        String query = "select count(id) from photo where date_added ~ '"
                + daySliceFormatter.format(d) + "' and created_by not in ("
                + UserManager.gcrUsers + ");";
        return getCount(query);
    }

    public static int getCaptureCountForDate(Date d) {
        String query = "select count(id) from photo where capture_date ~ '"
                + daySliceFormatter.format(d) + "' and created_by not in ("
                + UserManager.gcrUsers + ");";
        return getCount(query);
    }

    public static int getUploaderCountForDate(Date d) {
        String query = "select count(distinct created_by) from photo where date_added ~'"
                + daySliceFormatter.format(d)
                + "' and created_by not in ("
                + UserManager.gcrUsers + ");";
        return getCount(query);
    }

    private static int getCount(String query) {
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        int count = -1;
        try {
            while (rs.next()) {
                count = rs.getInt(1);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return count;
    }

    private static List listPhotos(String query) {
        ResultSetWrapper rsw = RDBConnection.executeQuery(query);
        ResultSet rs = rsw.getResultSet();
        ArrayList l = new ArrayList();
        try {
            while (rs.next()) {
                int id = rs.getInt(1);
                IPhoto p = PhotoFactory.getPhotoByID(id);
                if (p != null) {
                    l.add(p);
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            rsw.closeAll();
        }
        return l;
    }

}
