/*
 * Created on Aug 18, 2004
 */
package gcr.mmm2.util;

import gcr.mmm2.model.StringFormatException;
import gcr.rdf.MMMModelFactory;
import gcr.rdf.PropertyDescriptor;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.text.DateFormat;
import java.text.Format;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.http.HttpServletRequest;

import com.oreilly.servlet.Base64Decoder;
import com.oreilly.servlet.Base64Encoder;

/**
 * @author Simon King
 * @author Jeff Towle
 * 
 */
public class StringUtils {
    private static Pattern emailPattern = Pattern
            .compile("^[a-zA-Z]([\\w\\.-])*([a-zA-Z0-9])*@[a-zA-Z0-9][\\w\\.-]*[a-zA-Z0-9]\\.[a-zA-Z][a-zA-Z\\.]*[a-zA-Z]$");

    private static Pattern validPhoneNonDigitsPattern = Pattern
            .compile("[ \\(\\)-]");

    private static Pattern tenDigitsPattern = Pattern.compile("^\\d{10}?$");

    public static final SimpleDateFormat sdf = new SimpleDateFormat(
            "yyyyMMdd'T'HHmmss");

    public static final SimpleDateFormat xmlsdf = new SimpleDateFormat(
            "yyyy-MM-dd'T'HH:mm:ss");

    public static String URLEncode(String toEncode) {
        if (toEncode != null) {
            toEncode = toEncode.replaceAll("%", "&#37;");
            System.out.println(toEncode);
            try {
                return URLEncoder.encode(toEncode, "UTF-8");
            } catch (UnsupportedEncodingException e) {
                // This should never happen
                e.printStackTrace();
            }
        }
        return null;
    }

    public static String URLDecode(String toDecode) {
        if (toDecode != null) {
            String ourString = new String(toDecode);
            try {
                return URLDecoder.decode(ourString, "UTF-8");
            } catch (UnsupportedEncodingException e) {
                // This should never happen
                // e.printStackTrace();
            }
        }
        return null;
    }

    /*
     * showSafeHTML(); URLDecodes previously URLEncode'd strings and removes
     * everything within <,>'s this should keep any nasty HTML and/or Javascript
     * from making it through this function.
     * 
     * Used heavily on photo_details.jsp, include_photolist.jsp,
     * my_albums_menu.jsp, received_albums.jsp
     */
    public static String showSafeHTML(String unsafeString) {
        if (unsafeString != null) {
            String safeString = unsafeString;
            safeString = safeString.replaceAll("<.*?>", ""); // kill tag
                                                                // contents
            // safeString = safeString.replaceAll("&", "&amp;"); //convert
            // ampersands
            safeString = safeString.replaceAll(">", "&gt;"); // convert angle
                                                                // brackets
            safeString = safeString.replaceAll("<", "&lt;");
            safeString = safeString.replaceAll("\"", "&quot;");
            safeString = safeString.replaceAll("!", "&#33;"); // some bangs
                                                                // make stuff
                                                                // weird
            safeString = safeString.replaceAll("javascript:.*?;", ""); // kill
                                                                        // javascript
            return safeString;
        } else {
            return null;
        }
    }

    public static boolean isValidEmailAddress(String email) {
        // TODO -- make this a little less generous -- it accepts too many
        // addresses
        Matcher m = emailPattern.matcher(email);
        return m.matches();
    }

    public static void validateEmail(String email) throws StringFormatException {
        if (!isValidEmailAddress(email)) {
            throw new StringFormatException(email
                    + " is not a valid email address.");
        }
    }

    public static String extractPhoneDigits(String phoneNum)
            throws StringFormatException {
        Matcher m1 = validPhoneNonDigitsPattern.matcher(phoneNum);
        String digits = m1.replaceAll("");
        Matcher m2 = tenDigitsPattern.matcher(digits);
        if (!m2.matches()) {
            throw new StringFormatException(phoneNum
                    + " is not a valid phone number");
        }
        return digits;
    }

    public static String formatPhoneNumber(String digits)
            throws StringFormatException {

        if (digits == null) {
            return "";
        }

        Matcher m = tenDigitsPattern.matcher(digits);
        if (!m.matches()) {
            throw new StringFormatException(digits
                    + " is not a 10-digit sequence");
        }
        return "(" + digits.substring(0, 3) + ")" + digits.substring(3, 6)
                + "-" + digits.substring(6, 10);
    }

    /**
     * @param dateString
     *            yyyyMMdd'T'HHmmss
     * @return Date object
     * @throws StringFormatException
     */
    public static Date parseContextDate(String dateString)
            throws StringFormatException {
        try {
            return sdf.parse(dateString);
        } catch (ParseException e) {
            e.printStackTrace();
            throw new StringFormatException("Invalid date format: "
                    + dateString);
        }

    }

    /**
     * @param dateString
     *            yyyy-MM-dd'T'HH:mm:ss
     * @return Date object
     * @throws StringFormatException
     */
    public static Date parseXMLDate(String dateString)
            throws StringFormatException {
        try {
            return xmlsdf.parse(dateString);
        } catch (ParseException e) {
            e.printStackTrace();
            throw new StringFormatException("Invalid date format: "
                    + dateString);
        }

    }

    public static String getXMLDateTime(Date d) {
        return xmlsdf.format(d);
    }

    // public static String getMapAsHTML(Map m) {
    // StringBuffer b=new StringBuffer();
    // b.append("<table border=1><tr><td>");
    // Iterator i=m.keySet().iterator();
    // while (i.hasNext()) {
    // String key=(String)i.next();
    // String val=null;
    // try {
    // val=(String)m.get(key);
    // } catch (ClassCastException e) {
    // Map m2=(Map)m.get(key);
    // val=getMapAsHTML(m2);
    // }
    // b.append(key+" : "+val+"<br>");
    // } // end while
    // b.append("</td></tr></table>");
    // return b.toString();
    // }

    // if asRow==true return elements (including nulls) as
    // <td>elem1</td><td>elem2...
    // if false return a full table with table tags
    public static String getMapAsHTML2(Map m, boolean asRow,
            boolean inclCreatedBy) {
        StringBuffer b = new StringBuffer();
        String classURI = (String) m.get("CLASS_URI");
        if (classURI == null) {
            return "";
        }
        List propList = (List) MMMModelFactory.getInstance().getClassMap().get(
                classURI);
        if (asRow) {
            // b.append("<td>");
        } else {
            b.append("<table border=1><tr><td>");
        }
        String createdBy = (String) m.get("CREATED_BY");
        if ((createdBy != null) && (inclCreatedBy)) {
            if (asRow) {
                b.append("<td>" + createdBy + "</td>");
            } else {
                b.append("<b>Created By</b>: " + createdBy);
                b.append("<br>");
            }
        }
        Iterator i = propList.iterator();
        while (i.hasNext()) {
            PropertyDescriptor pd = (PropertyDescriptor) i.next();
            String key = pd.getURI();
            Object val = m.get(key);
            if (val != null) {
                String sVal = null;
                try {
                    sVal = (String) m.get(key);
                } catch (ClassCastException e) {
                    Map m2 = (Map) m.get(key);
                    sVal = getMapAsHTML2(m2, false, (inclCreatedBy && (!asRow))); // inner
                                                                                    // cells
                                                                                    // return
                                                                                    // full
                                                                                    // table
                }
                if (asRow) {
                    b.append("<td>" + sVal + "</td>");
                } else {
                    b.append("<b>" + pd.getLabel() + ": </b>" + sVal + "<br>");
                }
            } else if (asRow) { // include empty cells in row representation
                b.append("<td>&nbsp;</td>");
            }
        } // end while
        if (asRow) {
            // b.append("</td>");
        } else {
            b.append("</td></tr></table>");
        }
        return b.toString();
    }

    // if false return a full table with table tags
    public static String getMapAsIndentedHTML(Map m, boolean inclCreatedBy) {
        StringBuffer b = new StringBuffer();
        String classURI = (String) m.get("CLASS_URI");
        if (classURI == null) {
            return "";
        }
        List propList = (List) MMMModelFactory.getInstance().getClassMap().get(
                classURI);

        b.append("<div class='property_map_container'>");

        Iterator i = propList.iterator();
        while (i.hasNext()) {
            PropertyDescriptor pd = (PropertyDescriptor) i.next();
            String key = pd.getURI();
            Object val = m.get(key);
            if (val != null) {
                String sVal = null;
                try {
                    sVal = (String) m.get(key);
                } catch (ClassCastException e) {
                    Map m2 = (Map) m.get(key);
                    sVal = "<div class='nested_property_map'>"
                            + getMapAsIndentedHTML(m2, inclCreatedBy)
                            + "</div>\n"; // inner cells return full table
                }
                b.append("<div class='property_name'>" + pd.getLabel()
                        + ": </div>\n");
                b.append("<div class='property_value'>" + sVal + "</div>\n");

            }
        } // end while

        String createdBy = (String) m.get("CREATED_BY");
        if ((createdBy != null) && (inclCreatedBy)) {
            b.append("<div class='created_by'>Created by: " + createdBy
                    + "</div>\n");
        }

        b.append("</div>");
        return b.toString();
    }

    // public static String getListAsHTML(List l) {
    // StringBuffer b=new StringBuffer();
    // b.append("<table border=1><tr><td>");
    // Iterator i=l.iterator();
    // while (i.hasNext()) {
    // Object[] item=(Object[])i.next();
    // System.out.println(item[0].getClass());
    // String label=null;
    // try {
    // PropertyDescriptor pd=(PropertyDescriptor)item[0];
    // label=pd.getLabel();
    // } catch (ClassCastException e) {
    // label=(String)item[0];
    // }
    // String val=null;
    // try {
    // val=(String)item[1];
    // } catch (ClassCastException e) {
    // List l2=(List)item[1];
    // val=getListAsHTML(l2);
    // }
    // b.append(label+" : "+val+"<br>");
    // } // end while
    // b.append("</td></tr></table>");
    // return b.toString();
    // }

    public static String stripBaseURI(String uri) {
        // if (uri.startsWith(MMMModelFactory.BASE_URI)) {
        // return uri.substring(MMMModelFactory.BASE_URI.length());
        // }
        return collapseURI(uri);
    }

    public static String expandURI(String uri) {

        if (uri.startsWith("mmmbase:")) {
            return "http://garage.sims.berkeley.edu/mmm-2/ontology#"
                    + uri.substring("mmmbase:".length());
        }

        HashMap h = (HashMap) MMMModelFactory.getInstance().getOntologyMap();
        Iterator i = h.keySet().iterator();
        while (i.hasNext()) {
            String id = (String) i.next();
            String[] s = (String[]) h.get(id);
            String namespace = s[0];
            String file_name = s[1];
            id = id + ":";
            if (uri.startsWith(id)) {
                // System.out.println("Expanding URI to: "+
                // namespace+"#"+uri.substring(id.length()));
                return namespace + "#" + uri.substring(id.length());
            }
        }
        return null;
    }

    public static String collapseURI(String uri) {
        if (uri.startsWith("http://garage.sims.berkeley.edu/mmm-2/ontology#")) {
            return "mmmbase:"
                    + uri
                            .substring("http://garage.sims.berkeley.edu/mmm-2/ontology#"
                                    .length());
        }
        HashMap h = (HashMap) MMMModelFactory.getInstance().getOntologyMap();
        Iterator i = h.keySet().iterator();
        while (i.hasNext()) {
            String id = (String) i.next();
            String[] s = (String[]) h.get(id);
            String namespace = s[0];
            String file_name = s[1];
            namespace = namespace + "#";
            if (uri.startsWith(namespace)) {
                // System.out.println("Collapsing URI to: " +
                // id+":"+uri.substring(namespace.length()));
                return id + ":" + uri.substring(namespace.length());
            }
        }
        return null;
    }

    /**
     * @param date
     * @return
     */
    public static String formatDetailsDate(Date date) {
        return DateFormat.getDateInstance(DateFormat.MEDIUM).format(date);

        /*
         * Format formatter;
         *  // The year formatter = new SimpleDateFormat("yy"); // 02 formatter =
         * new SimpleDateFormat("yyyy"); // 2002
         *  // The month formatter = new SimpleDateFormat("M"); // 1 formatter =
         * new SimpleDateFormat("MM"); // 01 formatter = new
         * SimpleDateFormat("MMM"); // Jan formatter = new
         * SimpleDateFormat("MMMM"); // January
         *  // The day formatter = new SimpleDateFormat("d"); // 9 formatter =
         * new SimpleDateFormat("dd"); // 09
         *  // The day in week formatter = new SimpleDateFormat("E"); // Wed
         * formatter = new SimpleDateFormat("EEEE"); // Wednesday
         *  // Get today's date Date date = new Date();
         *  // Some examples formatter = new SimpleDateFormat("MM/dd/yy");
         * String s = formatter.format(date); // 01/09/02
         * 
         * formatter = new SimpleDateFormat("dd-MMM-yy"); s =
         * formatter.format(date); // 29-Jan-02
         *  // Examples with date and time; see also // e316 Formatting the Time
         * Using a Custom Format formatter = new
         * SimpleDateFormat("yyyy.MM.dd.HH.mm.ss"); s = formatter.format(date); //
         * 2002.01.29.08.36.33
         * 
         * formatter = new SimpleDateFormat("E, dd MMM yyyy HH:mm:ss Z"); s =
         * formatter.format(date); // Tue, 09 Jan 2002 22:14:02 -0500
         * 
         */
    }

    /**
     * @param date
     * @return
     */
    public static String formatDetailsTime(Date date) {

        /*
         * // The hour (1-12) formatter = new SimpleDateFormat("h"); // 8
         * formatter = new SimpleDateFormat("hh"); // 08
         *  // The hour (0-23) formatter = new SimpleDateFormat("H"); // 8
         * formatter = new SimpleDateFormat("HH"); // 08
         *  // The minutes formatter = new SimpleDateFormat("m"); // 7 formatter =
         * new SimpleDateFormat("mm"); // 07
         *  // The seconds formatter = new SimpleDateFormat("s"); // 3 formatter =
         * new SimpleDateFormat("ss"); // 03
         *  // The am/pm marker formatter = new SimpleDateFormat("a"); // AM
         *  // The time zone formatter = new SimpleDateFormat("z"); // PST
         * formatter = new SimpleDateFormat("zzzz"); // Pacific Standard Time
         * formatter = new SimpleDateFormat("Z"); // -0800
         */

        Locale locale = Locale.US;
        Format formatter = new SimpleDateFormat("h:mm a", locale);
        return formatter.format(date);

        // DEFAULT DATE FORMAT
        // return DateFormat.getTimeInstance(DateFormat.LONG,
        // locale).format(date);
    }

    /**
     * @param date
     * @return
     */
    public static String formatThumbDateTime(Date date) {
        Locale locale = Locale.US;
        Format formatter = new SimpleDateFormat("MM.dd.yy h:mm a", locale);
        return formatter.format(date);
    }

    /**
     * @param date
     * @return Month, day, &lt;br&gt;, day of week
     */
    public static String formatMiniDate(Date date) {
        Locale locale = Locale.US;
        Format formatter = new SimpleDateFormat("MM.dd.yy'<br>'EEE", locale);
        return formatter.format(date);
    }

    /**
     * Turns a string "sideways" by inserting <br>
     * between every letter.
     * 
     * @param theString
     * @return string with BR tags
     */
    public static String makeSideways(String theString) {
        StringBuffer strBuf = new StringBuffer(theString);

        int i = 1;
        while (i < strBuf.length()) {
            strBuf.insert(i, "<br />");
            i += 5;
        }

        return strBuf.toString();
    }

    /**
     * This is to escape the quote characters for making some JS output
     * 
     * @param theString
     * @return
     */
    public static String javaScriptEscape(String theString) {

        theString = theString.replaceAll("\\\"", "\\\\u0022");
        theString = theString.replaceAll("\\\'", "\\\\u0027");
        theString = theString.replaceAll("\\\r", "<br>");
        theString = theString.replaceAll("\\\n", "<br>");

        return theString;

    }

    public static String formatThumbDate(Date theDate) {
        String outDate = formatThumbDateTime(theDate);
        if (outDate.indexOf(" ") > 0) {
            outDate = outDate.substring(0, outDate.indexOf(" ")).trim();
        }
        return outDate;
    }

    /*
     * xorB64encodeString: take String input, XOR it byte-wise against String
     * salt, Base64 the result and return it as string -Ivan
     */
    public static String xorB64encodeString(String input, String salt) {
        String inputText = input;
        String key = salt;
        byte[] keyArray = key.getBytes();
        // byte[] inputTextArray = inputText.getBytes();
        byte[] outputArray = inputText.getBytes();

        // encrypt
        for (int i = 0; i < inputText.length(); i++) {
            outputArray[i] ^= keyArray[i % key.length()];
        }

        return new String(Base64Encoder.encode(outputArray));
    }

    /*
     * xorB64decodeString: take Base64 encoded String input, decode it to a
     * String XOR it against String salt, then return the result as String
     */
    public static String xorB64decodeString(String input, String salt) {
        String inputText = new String(Base64Decoder.decode(input));
        String key = salt;
        byte[] keyArray = key.getBytes();
        // byte[] inputTextArray = inputText.getBytes();
        byte[] outputArray = inputText.getBytes();

        // decrypt
        for (int i = 0; i < inputText.length(); i++) {
            outputArray[i] ^= keyArray[i % key.length()];
        }

        return new String(outputArray);
    }

    public static String displaySelect(String name, Map values, HttpServletRequest request) {
        final StringBuffer buff = new StringBuffer();
        final String param = request.getParameter(name);
        buff.append("<select name=\"").append(name).append("\">");
        buff.append("<option></option>");
        Object keys[] = values.keySet().toArray();
        
        for (int i = 0; i < keys.length ; i++) {
            final Object value = values.get(keys[i]);
            buff.append("<option ");
            if(keys[i].equals(param)) {
                buff.append(" selected ");
            }
            buff.append("value=\"").append(keys[i]).append("\">")
                    .append(value).append("</option>\n");
        }
        buff.append("</select>");
        return buff.toString();
    }
}
