package gcr.rdf;

import gcr.mmm2.model.IPerson;
import gcr.mmm2.model.ObjectSorter;
import gcr.mmm2.model.PersonManager;
import gcr.mmm2.rdb.RDBConnection;
import gcr.mmm2.rdf.MMMObject;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntProperty;
import com.hp.hpl.jena.ontology.OntResource;
import com.hp.hpl.jena.ontology.UnionClass;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

/*
 * Created on Jun 4, 2004
 */

/**
 * @author jeff
 */

public class ResourceManipulator {

    // public final static String PICTURE_CLASS_URI=
    // "http://garage.sims.berkeley.edu/mmm-2/ontology#Picture";
    // public final static String PROPERTY_FILE_PATH =
    // "http://garage.sims.berkeley.edu/mmm-2/ontology#hasImagefile";
    // public final static String PROPERTY_FIRST_NAME =
    // "http://garage.sims.berkeley.edu/mmm-2/ontology#firstName";
    // public final static String PROPERTY_LAST_NAME =
    // "http://garage.sims.berkeley.edu/mmm-2/ontology#lastName";
    public final static String PROPERTY_ISA = "http://garage.sims.berkeley.edu/mmm-2/ontology#isA";

    public final static String PROPERTY_HAS_LABEL = "http://garage.sims.berkeley.edu/mmm-2/ontology#label";

    // public final static String PROPERTY_SHARED_WITH =
    // "http://garage.sims.berkeley.edu/mmm-2/ontology#grantedTo";
    // public final static String PROPERTY_SHARES_MEDIA =
    // "http://garage.sims.berkeley.edu/mmm-2/ontology#sharesMedia";
    // public final static String PROPERTY_INITIATED_BY =
    // "http://garage.sims.berkeley.edu/mmm-2/ontology#initiatedBy";
    public final static String PERSON_ID_CLASS_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#mmmPersonID";

    public final static String PHOTO_ID_CLASS_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#photoID";

    public final static String ALBUM_ID_CLASS_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#albumID";

    public final static String ANNOTATIONBASE_CLASS_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#annotationBase";

    public final static String PROPERTY_RDFS_LABEL_URI = "http://www.w3.org/2000/01/rdf-schema#label";

    public final static String PROPERTY_INCLUDE_CONCISE_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#includeInConciseView";

    public final static String PROPERTY_SORT_ORDER_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#sortOrder";

    public final static String PROPERTY_CREATOR_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#creator";

    public final static String PROPERTY_SUBJECT_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#subject";

    public final static String PROPERTY_EDIT_SUBJECT_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#editSubject";

    public final static String PROPERTY_HIDDEN_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#hidden";

    public final static String PROPERTY_TIMESTAMP_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#timestamp";

    public final static String PROPERTY_HAS_ASSOC_OBJ_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#hasAssociatedObject";

    public final static String TAXONOMYITEM_CLASS_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#taxonomyItem";

    public final static String INSTANTIABLE_CLASS_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#Instantiable";

    // public final static String
    // PERSON_ID_CLASS_URI="http://garage.sims.berkeley.edu/mmm-2/ontology#mmmPersonID";
    public final static String MMMBASE_CLASS_URI = "http://garage.sims.berkeley.edu/mmm-2/ontology#mmmBase";

    public final static int DATATYPE_PROPERTY = 1;

    public final static int TAXONOMY_CLASS_PROPERTY = 2;

    public final static int INSTANTIABLE_INSTANCE_PROPERTY = 3;

    public final static int PERSON_ID_PROPERTY = 4;

    public final static int MMMBASE_INSTANCE_PROPERTY = 5;

    public static ExtendedIterator listClassMembers(String classURI) {

        OntClass c = MMMModelFactory.getInstance().getOntModel().getOntClass(
                classURI);
        if (c == null) {
            return null;
        }
        return c.listInstances();
    }

    // public static String createList(String[] members) {
    // ArrayList nodes=new ArrayList();
    // for (int i=0; i<members.length; i++) {
    // Individual
    // ind=MMMModelFactory.getInstance().getOntModel().getIndividual(members[i]);
    // if (ind!=null) {
    // nodes.add(ind);
    // }
    // }
    // return
    // MMMModelFactory.getInstance().getOntModel().createList(nodes.iterator()).toString();
    // }

    // TODO : fix addProperty to use RDFNode/Individual
    public static boolean createInstance(String subjectURI,
            String predicateURI, String objectURI) {
        if (ResourceManipulator.individualExists(subjectURI)) {
            return false;
        }
        OntProperty p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(predicateURI);
        if (p == null) {
            System.out.println("createInstance p==null");
            return false;
        }
        if (p.isObjectProperty()) {
            Individual ind = MMMModelFactory.getInstance().getOntModel()
                    .getIndividual(objectURI);
            if (ind == null) {
                System.out.println("createInstance ind==null");
                return false;
            }
            if (!ResourceManipulator.isInRange(p, ind)) {
                System.out.println("createInstance bad range: " + p + ", "
                        + ind);
                return false;
            }
            Resource r = MMMModelFactory.getInstance().getOntModel()
                    .createResource(subjectURI);
            r.addProperty(p, ind);
        } else {
            // do validation on literal
            Resource r = MMMModelFactory.getInstance().getOntModel()
                    .createResource(subjectURI);
            r.addProperty(p, objectURI);
        }
        return true;
    }

    // start new stuff -- SPK 09/20
    // return a int (see constants at top) representin the range of an
    // annotation property
    public static int getAnnotationPropertyType(String propURI) {
        OntProperty p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(propURI);
        if (p == null) {
            System.out.println("getAnnotationPropertyType p=null");
            return -1;
        }
        if (p.isDatatypeProperty()) {
            return DATATYPE_PROPERTY;
        }
        // else it's an object property
        String[] classURIs = { TAXONOMYITEM_CLASS_URI, INSTANTIABLE_CLASS_URI,
                PERSON_ID_CLASS_URI, MMMBASE_CLASS_URI };
        for (int i = 0; i < classURIs.length; i++) {
            OntClass c = MMMModelFactory.getInstance().getOntModel()
                    .getOntClass(classURIs[i]);
            if (p.hasRange(c)) {
                return i + 2;
            } else { // check to see if range(p) is a subclass of c
                ExtendedIterator ei = p.listRange();
                while (ei.hasNext()) {
                    OntResource r = (OntResource) ei.next();
                    // System.out.println("getAnnotationPropertyType -
                    // "+c.getURI()+"\t"+r.getURI());
                    if (c.hasSubClass(r)) {
                        ei.close();
                        return i + 2;
                    }
                }
            }
        }
        return -2;
    }

    // TODO -- fix this ugly hack
    // this version doesn't do range checking
    public static boolean addProperty2(String subjectURI, String propertyURI,
            String value) {

        if (!individualExists(subjectURI)) {
            return false;
        }

        Resource r = MMMModelFactory.getInstance().getOntModel()
                .createResource(subjectURI); // create == get

        // all good
        OntProperty p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(propertyURI);
        System.out.println("Property Range: ");
        // PictureAnnotator.printIterator(p.listRange());
        if (p.isObjectProperty()) {
            Individual ind = MMMModelFactory.getInstance().getOntModel()
                    .getIndividual(value);
            if (ind == null) {
                return false;
            }
            // if (isInRange(p, ind)) {
            // r.addProperty(p, value);
            r.addProperty(p, ind); // could check return to see if it's null
            return true;
            // }else{
            // System.out.println("addProperty: bad range: " + p + ", " + ind);
            // }
        } else if (p.isDatatypeProperty()) {
            r.addProperty(p, value);
            return true;
        }
        // done
        return false;
    }

    // set the subject's editSubject property to target and set target to be
    // hidden
    // return the subjectURI of the target (so that it can be used as the
    // subjectURI for this subject)
    public static String addAnnotationEditTarget(Individual subject,
            String targetURI) {
        // set the edit target
        if (addAnnotationProperty(subject, PROPERTY_EDIT_SUBJECT_URI, targetURI)) {
            // if it worked, set the target to be hidden
            return setHidden(targetURI);
        }
        return null;
    }

    // return the subject URI (base, not edit subject URI) property of
    // subjectURI
    public static String getBaseSubjectURI(String subjectURI) {
        Individual ind = MMMModelFactory.getInstance().getOntModel()
                .getIndividual(subjectURI);
        if (ind == null) {
            return null;
        }
        OntProperty p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(PROPERTY_SUBJECT_URI);
        String rtn = ind.getPropertyValue(p).toString();
        return rtn;
    }

    // return the subjectURI of this subject on success, null on failure
    public static String setHidden(String subjectURI) {
        Individual ind = MMMModelFactory.getInstance().getOntModel()
                .getIndividual(subjectURI);
        if (ind == null) {
            return null;
        }
        OntProperty p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(PROPERTY_HIDDEN_URI);
        ind.addProperty(p, true);
        p = MMMModelFactory.getInstance().getOntModel().getOntProperty(
                PROPERTY_SUBJECT_URI);
        String rtn = ind.getPropertyValue(p).toString();
        return rtn;
    }

    // this is used for "deleting" annotations. Hides the top annotation, but
    // unhides the one below it (if any)
    public static boolean setHiddenAndUnhide(String subjectURI) {
        Individual ind = MMMModelFactory.getInstance().getOntModel()
                .getIndividual(subjectURI);
        if (ind == null) {
            return false;
        }
        OntProperty pHidden = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(PROPERTY_HIDDEN_URI);
        ind.addProperty(pHidden, true);
        OntProperty pEdit = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(PROPERTY_EDIT_SUBJECT_URI);
        RDFNode r = ind.getPropertyValue(pEdit);
        if (r != null) {
            ind = MMMModelFactory.getInstance().getOntModel().getIndividual(
                    r.toString());
            if (isAnnotationObject(ind)) {
                ind.removeAll(pHidden);
            }
        }
        return true;
    }

    // TODO should we pass an individual into this? or a map of properties?
    public static boolean addAnnotationProperty(Individual subject,
            String propURI, String propValue) {
        // all good
        OntProperty p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(propURI);
        // System.out.println("Property Range: ");
        // PictureAnnotator.printIterator(p.listRange());
        if (p.isObjectProperty()) {
            Individual ind = MMMModelFactory.getInstance().getOntModel()
                    .getIndividual(propValue);
            if (ind == null) {
                return false;
            }
            // range touble -- class/instance confusion leave out range check
            // for now (maybe owl reasoner)
            // if (isInRange(p, ind)) {
            // r.addProperty(p, value);
            subject.addProperty(p, ind); // could check return to see if it's
            // null
            return true;
            // }else{
            // System.out.println("addProperty: bad range: " + p + ", " + ind);
            // }
        } else if (p.isDatatypeProperty()) {
            subject.addProperty(p, propValue);
            return true;
        }
        // done
        return false;
    }

    private static List convertNodesToStringList(Iterator i) {
        ArrayList l = new ArrayList();
        while (i.hasNext()) {
            OntResource r = (OntResource) i.next();
            l.add(convertNodeToString(r));
        }
        return l;
    }

    private static String[] convertNodeToString(OntResource r) {
        String label = null;
        Property pLabel = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(PROPERTY_RDFS_LABEL_URI);
        if (pLabel == null) {
            System.out.println("convertNodesToStringList pLabel==null");
            return new String[2];
        }
        try {
            label = r.getPropertyValue(pLabel).toString();
        } catch (NullPointerException ignored) {
        }
        if (label == null) {
            // label="no label";
        }
        String[] temp = { r.getURI(), label };
        return temp;
    }

    public static String getInstanceClassLabel(String indURI) {
        Individual ind = MMMModelFactory.getInstance().getOntModel()
                .getIndividual(indURI);
        if (ind == null) {
            return null;
        } else {
            return getClassLabel(ind.getRDFType(true).getURI());
        }
    }

    public static String getInstanceClass(String indURI) {
        Individual ind = MMMModelFactory.getInstance().getOntModel()
                .getIndividual(indURI);
        if (ind == null) {
            return null;
        } else {
            return getInstanceClass(ind);
        }
    }

    public static String getInstanceClass(Individual ind) {
        return ind.getRDFType(true).getURI();
    }

    public static String getClassLabel(String classURI) {
        OntClass c = MMMModelFactory.getInstance().getOntModel().getOntClass(
                classURI);
        if (c == null) {
            System.out.println("getClassLabel c==null");
            return "unknown class";
        }
        return convertNodeToString(c)[1];
    }

    public static String getResourceLabel(String rsrcURI) {
        OntResource r = MMMModelFactory.getInstance().getOntModel()
                .getOntClass(rsrcURI);
        if (r == null) {
            r = MMMModelFactory.getInstance().getOntModel().getIndividual(
                    rsrcURI);
        }
        if (r == null) {
            r = MMMModelFactory.getInstance().getOntModel().getOntProperty(
                    rsrcURI);
        }
        if (r == null) {
            return "unknown URI";
        } else {
            return convertNodeToString(r)[1];
        }
    }

    public static List getTaxonomyChildren(String classURI) {
        return listSubClasses(classURI, true);
    }

    // return a list of taxonomy items {uri, label} from ancestor to child, not
    // including child
    public static List getTaxonomyPath(String ancestorURI, String childURI) {
        ArrayList l = new ArrayList();
        OntClass a = MMMModelFactory.getInstance().getOntModel().getOntClass(
                ancestorURI);
        OntClass c = MMMModelFactory.getInstance().getOntModel().getOntClass(
                childURI);
        if ((c == null) || (a == null)) {
            System.out.println("getTaxonomyPath c or a==null");
            return l;
        }
        l.add(a);
        l = getTaxonomyPath(l, c);
        return convertNodesToStringList(l.iterator());
    }

    public static boolean isAnnotationObject(Individual ind) {
        if (ind == null) {
            return false;
        }
        return ind.hasRDFType(ANNOTATIONBASE_CLASS_URI);
    }

    public static boolean isAnnotationObject(String indURI) {
        if (indURI == null) {
            return false;
        }
        Individual ind = MMMModelFactory.getInstance().getOntModel()
                .getIndividual(indURI);
        if (ind == null) {
            return false;
        }
        return isAnnotationObject(ind);
    }

    private static ArrayList getTaxonomyPath(ArrayList path, OntClass node) {
        OntClass a = (OntClass) path.get(path.size() - 1);
        if (!node.hasSuperClass(a)) { // no path from end of list to child
            // node
            return new ArrayList(); // return empty list
        }
        if (node.hasSuperClass(a, true)) { // end of list is direct parent
            return path; // return path as is (without child node)
        } else { // list ends in ancestor, but not direct parent
            ExtendedIterator ei = a.listSubClasses(true); // get a's children
            while (ei.hasNext()) {
                OntClass c = (OntClass) ei.next();
                if (node.hasSuperClass(c)) { // c is ancestor of node
                    path.add(c);
                    ei.close();
                    return getTaxonomyPath(path, node);
                }
            } // end while
        }
        // shouldn't get here
        System.out.println("getTaxonomyPath bad situation " + a.getURI() + "\t"
                + node.getURI());
        return new ArrayList();
    }

    public static boolean hasTaxonomyChildren(String classURI) {
        return (listSubClasses(classURI, false).size() > 0);
    }

    // return ancestors of childURI (not including child or taxonomy root)
    public static List getTaxonomyPath(String childURI) {
        // System.out.println("getTaxonomyPath - "+childURI);
        ArrayList l = new ArrayList(getTaxonomyPath(TAXONOMYITEM_CLASS_URI,
                childURI));
        l.remove(0); // remove taxonomy root
        return l;
    }

    public static Individual createIndividual(String indURI, String classURI) {
        if (ResourceManipulator.individualExists(indURI)) {
            return null;
        }
        OntClass c = MMMModelFactory.getInstance().getOntModel().getOntClass(
                classURI);
        if (c == null) {
            System.out.println("createIndividual c==null");
            return null;
        }
        Individual i = MMMModelFactory.getInstance().getOntModel()
                .createIndividual(indURI, c);
        return i;
    }

    public static Individual createIndividual(String classURI) {
        // TODO -- handle this ugly hack a bit better
        String newURI = MMMModelFactory.BASE_URI + "annotation/"
                + RDBConnection.getNextVal("rdf_uri_seq");
        return createIndividual(newURI, classURI);
    }

    public static List listAnnotationClasses() {
        return listSubClasses(ANNOTATIONBASE_CLASS_URI, false);
    }

    // // returns List of Object[2] -- index 0 is String label, index 1 is
    // String value or nested List
    // // if concise set, only show some properties (those flagged as
    // includeInConciseView=true)
    // public static List getObjectAsList(String indURI, boolean concise) {
    // List rtnList=new ArrayList();
    // Individual
    // ind=MMMModelFactory.getInstance().getOntModel().getIndividual(indURI);
    // if (ind==null) {
    // return rtnList;
    // }
    // // start the list out with creator/timestamp
    // String creatorURI=null;
    // Iterator i2=listDeclaredInstancePropertyValues(indURI,
    // PROPERTY_CREATOR_URI);
    // if (i2.hasNext()) {
    // creatorURI=i2.next().toString();
    // }
    // String timestamp=null;
    // i2=listDeclaredInstancePropertyValues(indURI, PROPERTY_TIMESTAMP_URI);
    // if (i2.hasNext()) {
    // timestamp=i2.next().toString();
    // }
    // String createdBy=getPersonLabel(creatorURI)+" at "+timestamp;
    // String[] temp={"created by", createdBy};
    // if (!concise) {
    // rtnList.add(temp);
    // }
    // // now get all other sorted class properties
    // List propList =
    // (List)MMMModelFactory.getInstance().getClassMap().get(ind.getRDFType(true).getURI());
    // Iterator i=propList.iterator();
    // while (i.hasNext()) {
    // PropertyDescriptor pd=(PropertyDescriptor)i.next();
    // if (concise && !pd.isInConcise()) {
    // continue;
    // }
    // OntProperty
    // p=MMMModelFactory.getInstance().getOntModel().getOntProperty(pd.getURI());
    // // TODO -- think about listPropertyValues to allow for multiples...
    // System.out.println("getObjectAsList p- "+p+", "+ind);
    // RDFNode propVal=ind.getPropertyValue(p);
    // String objURI=null;
    // if (propVal==null) {
    // continue;
    // } else {
    // objURI=propVal.toString();
    // }
    // Object propValue=null;
    // switch (pd.getType()) {
    // case DATATYPE_PROPERTY :
    // propValue=objURI;
    // break;
    // case TAXONOMY_CLASS_PROPERTY :
    // propValue=getResourceLabel(objURI);
    // //TODO -- or maybe this should be the full breadcrumbs...
    // break;
    // case INSTANTIABLE_INSTANCE_PROPERTY :
    // // TODO fill in here
    // break;
    // case PERSON_ID_PROPERTY :
    // propValue=getPersonLabel(objURI);
    // break;
    // case MMMBASE_INSTANCE_PROPERTY :
    // // gotta get another map...
    // propValue=getObjectAsList(objURI,concise);
    // break;
    // default :
    // propValue=objURI;
    // }
    // Object[] item={pd.getLabel(),propValue};
    // rtnList.add(item);
    // }
    // return rtnList;
    // }

    public static AnnotationWrapper getWrappedRDFObject2(String indURI,
            boolean concise) {
        Map rtnMap = new HashMap();
        List rtnList = new ArrayList();
        Individual ind = MMMModelFactory.getInstance().getOntModel()
                .getIndividual(indURI);
        if (ind == null) {
            return new AnnotationWrapper(rtnList, null, indURI, null, null,
                    null, null, rtnMap);
        }
        // start the list out with creator/timestamp
        String creatorURI = null;
        Iterator i2 = listDeclaredInstancePropertyValues(indURI,
                PROPERTY_CREATOR_URI);
        if (i2.hasNext()) {
            creatorURI = i2.next().toString();
        }
        String subjectURI = null;
        i2 = listDeclaredInstancePropertyValues(indURI, PROPERTY_SUBJECT_URI);
        if (i2.hasNext()) {
            subjectURI = i2.next().toString();
        }
        String editSubjURI = null;
        i2 = listDeclaredInstancePropertyValues(indURI,
                PROPERTY_EDIT_SUBJECT_URI);
        if (i2.hasNext()) {
            editSubjURI = i2.next().toString();
        }
        String hidden = null;
        i2 = listDeclaredInstancePropertyValues(indURI, PROPERTY_HIDDEN_URI);
        if (i2.hasNext()) {
            hidden = i2.next().toString();
        }
        String timestamp = null;
        i2 = listDeclaredInstancePropertyValues(indURI, PROPERTY_TIMESTAMP_URI);
        if (i2.hasNext()) {
            timestamp = i2.next().toString();
        }
        rtnMap = getObjectAsPropMap2(indURI, concise);
        return new AnnotationWrapper(rtnList, timestamp, indURI, subjectURI,
                creatorURI, editSubjURI, hidden, rtnMap);
    }

    public static Map getObjectAsPropMap2(String indURI, boolean concise) {
        HashMap rtnMap = new HashMap();
        Individual ind = MMMModelFactory.getInstance().getOntModel()
                .getIndividual(indURI);
        if (ind == null) {
            return rtnMap;
        }
        // start the list out with creator/timestamp
        String creatorURI = null;
        Iterator i2 = listDeclaredInstancePropertyValues(indURI,
                PROPERTY_CREATOR_URI);
        if (i2.hasNext()) {
            creatorURI = i2.next().toString();
        }
        String subjectURI = null;
        i2 = listDeclaredInstancePropertyValues(indURI, PROPERTY_SUBJECT_URI);
        if (i2.hasNext()) {
            subjectURI = i2.next().toString();
        }
        String editSubjURI = null;
        i2 = listDeclaredInstancePropertyValues(indURI,
                PROPERTY_EDIT_SUBJECT_URI);
        if (i2.hasNext()) {
            editSubjURI = i2.next().toString();
        }
        String hidden = null;
        i2 = listDeclaredInstancePropertyValues(indURI, PROPERTY_HIDDEN_URI);
        if (i2.hasNext()) {
            hidden = i2.next().toString();
        }
        String timestamp = null;
        i2 = listDeclaredInstancePropertyValues(indURI, PROPERTY_TIMESTAMP_URI);
        if (i2.hasNext()) {
            timestamp = i2.next().toString();
        }
        String createdBy = getPersonLabel(creatorURI) + " at " + timestamp;
        // String[] temp={"created by", createdBy};
        if (!concise) {
            rtnMap.put("CREATED_BY", createdBy);
        }
        // now get all other sorted class properties
        String classURI = ind.getRDFType(true).getURI();
        rtnMap.put("CLASS_URI", classURI);
        List propList = (List) MMMModelFactory.getInstance().getClassMap().get(
                classURI);
        Iterator i = propList.iterator();
        while (i.hasNext()) {
            PropertyDescriptor pd = (PropertyDescriptor) i.next();
            if (concise && !pd.isInConcise()) {
                continue;
            }
            OntProperty p = MMMModelFactory.getInstance().getOntModel()
                    .getOntProperty(pd.getURI());
            // TODO -- think about listPropertyValues to allow for multiples...
            // System.out.println("getObjectAsList p- "+p+", "+ind);
            RDFNode propVal = ind.getPropertyValue(p);
            String objURI = null;
            if (propVal == null) {
                continue;
            } else {
                objURI = propVal.toString();
            }
            Object propValue = null;
            switch (pd.getType()) {
            case DATATYPE_PROPERTY:
                propValue = objURI;
                break;
            case TAXONOMY_CLASS_PROPERTY:
                propValue = getResourceLabel(objURI);
                // TODO -- or maybe this should be the full breadcrumbs...
                break;
            case INSTANTIABLE_INSTANCE_PROPERTY:
                // TODO fill in here
                break;
            case PERSON_ID_PROPERTY:
                propValue = getPersonLabel(objURI);
                break;
            case MMMBASE_INSTANCE_PROPERTY:
                // gotta get another map...
                propValue = getObjectAsPropMap2(objURI, concise);
                break;
            default:
                propValue = objURI;
            }
            Object[] item = { pd, propValue };
            rtnMap.put(pd.getURI(), propValue);
        }
        return rtnMap;
    }

    // public static AnnotationWrapper getWrappedRDFObject(String indURI,
    // boolean concise) {
    // List rtnList=new ArrayList();
    // Individual
    // ind=MMMModelFactory.getInstance().getOntModel().getIndividual(indURI);
    // if (ind==null) {
    // return new AnnotationWrapper(rtnList, null, indURI, null, null, null,
    // null, null);
    // }
    // // start the list out with creator/timestamp
    // String creatorURI=null;
    // Iterator i2=listDeclaredInstancePropertyValues(indURI,
    // PROPERTY_CREATOR_URI);
    // if (i2.hasNext()) {
    // creatorURI=i2.next().toString();
    // }
    // String subjectURI=null;
    // i2=listDeclaredInstancePropertyValues(indURI, PROPERTY_SUBJECT_URI);
    // if (i2.hasNext()) {
    // subjectURI=i2.next().toString();
    // }
    // String editSubjURI=null;
    // i2=listDeclaredInstancePropertyValues(indURI, PROPERTY_EDIT_SUBJECT_URI);
    // if (i2.hasNext()) {
    // editSubjURI=i2.next().toString();
    // }
    // String hidden=null;
    // i2=listDeclaredInstancePropertyValues(indURI, PROPERTY_HIDDEN_URI);
    // if (i2.hasNext()) {
    // hidden=i2.next().toString();
    // }
    // String timestamp=null;
    // i2=listDeclaredInstancePropertyValues(indURI, PROPERTY_TIMESTAMP_URI);
    // if (i2.hasNext()) {
    // timestamp=i2.next().toString();
    // }
    // String createdBy=getPersonLabel(creatorURI)+" at "+timestamp;
    // String[] temp={"created by", createdBy};
    // if (!concise) {
    // rtnList.add(temp);
    // }
    // // now get all other sorted class properties
    // List propList =
    // (List)MMMModelFactory.getInstance().getClassMap().get(ind.getRDFType(true).getURI());
    // Iterator i=propList.iterator();
    // while (i.hasNext()) {
    // PropertyDescriptor pd=(PropertyDescriptor)i.next();
    // if (concise && !pd.isInConcise()) {
    // continue;
    // }
    // OntProperty
    // p=MMMModelFactory.getInstance().getOntModel().getOntProperty(pd.getURI());
    // // TODO -- think about listPropertyValues to allow for multiples...
    // System.out.println("getObjectAsList p- "+p+", "+ind);
    // RDFNode propVal=ind.getPropertyValue(p);
    // String objURI=null;
    // if (propVal==null) {
    // continue;
    // } else {
    // objURI=propVal.toString();
    // }
    // Object propValue=null;
    // switch (pd.getType()) {
    // case DATATYPE_PROPERTY :
    // propValue=objURI;
    // break;
    // case TAXONOMY_CLASS_PROPERTY :
    // propValue=getResourceLabel(objURI);
    // //TODO -- or maybe this should be the full breadcrumbs...
    // break;
    // case INSTANTIABLE_INSTANCE_PROPERTY :
    // // TODO fill in here
    // break;
    // case PERSON_ID_PROPERTY :
    // propValue=getPersonLabel(objURI);
    // break;
    // case MMMBASE_INSTANCE_PROPERTY :
    // // gotta get another map...
    // propValue=getObjectAsList(objURI,concise);
    // break;
    // default :
    // propValue=objURI;
    // }
    // Object[] item={pd,propValue};
    // rtnList.add(item);
    // }
    // return new AnnotationWrapper(rtnList, timestamp, indURI, subjectURI,
    // creatorURI, editSubjURI, hidden, null);
    // }

    // public static Map getObjectAsPropMap(String indURI) {
    // HashMap hm= new HashMap();
    // Individual
    // ind=MMMModelFactory.getInstance().getOntModel().getIndividual(indURI);
    // if (ind !=null) {
    // StmtIterator i =
    // MMMModelFactory.getInstance().getOntModel().listStatements(ind,
    // (Property)null, (RDFNode)null);
    // while (i.hasNext()){
    // Statement s=(Statement)i.next();
    // String propURI=s.getPredicate().getURI();
    // String objURI=s.getObject().toString();
    // OntProperty
    // p=MMMModelFactory.getInstance().getOntModel().getOntProperty(propURI);
    // String propLabel=null;
    // Object propValue=null;
    // if (p==null) {
    // System.out.println("getObjectAsPropMap p=null");
    // } else {
    // propLabel=convertNodeToString(p)[1];
    // }
    // System.out.println("getObjectAsPropMap,
    // type="+getAnnotationPropertyType(propURI));
    // switch (getAnnotationPropertyType(propURI)) {
    // case DATATYPE_PROPERTY :
    // propValue=objURI;
    // break;
    // case TAXONOMY_CLASS_PROPERTY :
    // propValue=getResourceLabel(objURI);
    // //TODO -- or maybe this should be the full breadcrumbs...
    // break;
    // case INSTANTIABLE_INSTANCE_PROPERTY :
    // // TODO fill in here
    // break;
    // case PERSON_ID_PROPERTY :
    // propValue=getPersonLabel(objURI);
    // break;
    // case MMMBASE_INSTANCE_PROPERTY :
    // // gotta get another map...
    // propValue=getObjectAsPropMap(objURI);
    // break;
    // default :
    // propValue=objURI;
    // }
    // if (propLabel!=null) {
    // hm.put(propLabel, propValue);
    // }
    // }
    // String creatorURI=null;
    // Iterator i2=listDeclaredInstancePropertyValues(indURI,
    // PROPERTY_CREATOR_URI);
    // if (i2.hasNext()) {
    // creatorURI=i2.next().toString();
    // }
    // String timestamp=null;
    // i2=listDeclaredInstancePropertyValues(indURI, PROPERTY_TIMESTAMP_URI);
    // if (i2.hasNext()) {
    // timestamp=i2.next().toString();
    // }
    // String createdBy=getPersonLabel(creatorURI)+" at "+timestamp;
    // hm.put("created by", createdBy);
    // }
    // return hm;
    // }

    public static String getPersonLabel(String personURI) {
        // TODO think about storing person labels in RDF?
        String temp = personURI.substring(personURI.lastIndexOf("/") + 1);
        int personID = -1;
        try {
            personID = Integer.parseInt(temp);
        } catch (NumberFormatException nfe) {
            // photoID is set to -1, redirect will be called next
        }
        // System.out.println("getPersonLabel, id="+personID);
        IPerson p = PersonManager.getByID(personID);
        if (p != null) {
            return p.toString();
        }
        return null;
    }

    //	
    // // return list of annotation object URIs
    // public static List listAnnotations(String subjURI) {
    // ArrayList l=new ArrayList();
    // Individual
    // ind=MMMModelFactory.getInstance().getOntModel().getIndividual(subjURI);
    // Property
    // p=(Property)MMMModelFactory.getInstance().getOntModel().getOntProperty(PROPERTY_SUBJECT_URI);
    // if ((ind !=null)&&(p!=null)) {
    // StmtIterator i =
    // MMMModelFactory.getInstance().getOntModel().listStatements((Resource)null,
    // p, ind);
    // while (i.hasNext()){
    // Statement s=(Statement)i.next();
    // l.add(s.getSubject().getURI());
    // }
    // }
    // return l;
    // }

    // public static List listAnnotationWrappers(String subjURI, boolean
    // concise) {
    // ArrayList l=new ArrayList();
    // Individual
    // ind=MMMModelFactory.getInstance().getOntModel().getIndividual(subjURI);
    // Property
    // p=(Property)MMMModelFactory.getInstance().getOntModel().getOntProperty(PROPERTY_SUBJECT_URI);
    // if ((ind !=null)&&(p!=null)) {
    // StmtIterator i =
    // MMMModelFactory.getInstance().getOntModel().listStatements((Resource)null,
    // p, ind);
    // while (i.hasNext()){
    // Statement s=(Statement)i.next();
    // l.add(getWrappedRDFObject(s.getSubject().getURI(), concise));
    // }
    // }
    // Collections.sort(l, ObjectSorter.annotationWrapperByDate());
    // Collections.sort(l, ObjectSorter.annotationWrapperByType());
    // return l;
    // }

    public static List listAnnotationWrappers2(String subjURI, boolean concise) {
        ArrayList l = new ArrayList();
        Individual ind = MMMModelFactory.getInstance().getOntModel()
                .getIndividual(subjURI);
        Property p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(PROPERTY_SUBJECT_URI);
        if ((ind != null) && (p != null)) {
            StmtIterator i = MMMModelFactory.getInstance().getOntModel()
                    .listStatements((Resource) null, p, ind);
            while (i.hasNext()) {
                Statement s = (Statement) i.next();
                l.add(getWrappedRDFObject2(s.getSubject().getURI(), concise));
            }
        }
        Collections.sort(l, ObjectSorter.annotationWrapperByDate());
        Collections.sort(l, ObjectSorter.annotationWrapperByType());
        return l;
    }

    public static List listAnnotationHistory(String subjURI, boolean concise) {
        ArrayList l = new ArrayList();
        Individual ind = MMMModelFactory.getInstance().getOntModel()
                .getIndividual(subjURI);
        Property p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(PROPERTY_EDIT_SUBJECT_URI);
        if ((ind != null) && (p != null)) {
            l.add(getWrappedRDFObject2(ind.getURI(), concise));
            RDFNode prev = ind.getPropertyValue(p);
            Individual prevInd = null;
            if (prev != null) {
                prevInd = MMMModelFactory.getInstance().getOntModel()
                        .getIndividual(prev.toString());
            }
            while (isAnnotationObject(prevInd)) {
                l.add(getWrappedRDFObject2(prevInd.getURI(), concise));
                prev = prevInd.getPropertyValue(p);
                if (prev != null) {
                    prevInd = MMMModelFactory.getInstance().getOntModel()
                            .getIndividual(prev.toString());
                } else {
                    prevInd = null;
                }
            }
        }
        return l;
    }

    // return list of mmmBase object URIs
    // list of URIs that are associated with subjURI and subclasses of classURI
    public static List listAssociatedObjects(String subjURI, String classURI) {
        ArrayList l = new ArrayList();
        Individual ind = MMMModelFactory.getInstance().getOntModel()
                .getIndividual(subjURI);
        Property p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(PROPERTY_HAS_ASSOC_OBJ_URI);
        OntClass c = MMMModelFactory.getInstance().getOntModel().getOntClass(
                classURI);
        if ((ind != null) && (p != null)) {
            StmtIterator i = MMMModelFactory.getInstance().getOntModel()
                    .listStatements(ind, p, (RDFNode) null);
            while (i.hasNext()) {
                Statement s = (Statement) i.next();
                Individual candidate = MMMModelFactory.getInstance()
                        .getOntModel().getIndividual(s.getObject().toString());
                if (candidate != null && candidate.hasRDFType(c)) {
                    l.add(candidate.getURI());
                }
            }
        }
        return l;
    }

    // returns list of String[2] -- index 0=classURI, index 1=label
    public static List listSubClasses(String classURI, boolean direct) {
        ArrayList l = new ArrayList();
        OntClass c = MMMModelFactory.getInstance().getOntModel().getOntClass(
                classURI);
        if (c == null) {
            System.out.println("listSubclasses c==null");
            return l;
        }
        return convertNodesToStringList(c.listSubClasses(direct));
    }

    public static String[] getAnnotationPropertyRange(String propURI) {
        // String[] rtnVal=new String[2]; // could return this instead of null
        // on error
        OntProperty p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(propURI);
        if (p == null) {
            System.out.println("getAnnotationPropertyRange p=null");
            return null;
        }
        return convertNodeToString(p.getRange());
    }

    // return list of String[] as above
    // filters out subject, creator, timestamp properties, which should be set
    // automatically
    // public static List listAnnotationProperties(String classURI) {
    // ArrayList l=new ArrayList();
    // OntClass c =
    // MMMModelFactory.getInstance().getOntModel().getOntClass(classURI);
    // if (c==null) {System.out.println("listSubclasses c==null"); return l;}
    // Iterator i=c.listDeclaredProperties();
    // while (i.hasNext()) {
    // OntProperty p=(OntProperty)i.next();
    // //if (true) {
    // if (!(PROPERTY_SUBJECT_URI.equals(p.getURI()) ||
    // PROPERTY_CREATOR_URI.equals(p.getURI())
    // || PROPERTY_TIMESTAMP_URI.equals(p.getURI()) ||
    // PROPERTY_HAS_ASSOC_OBJ_URI.equals(p.getURI()))) {
    // l.add(convertNodeToString(p));
    // }
    // }
    // return l;
    // }

    public static List listAnnotationProperties(String classURI) {
        ArrayList l = new ArrayList();
        List l2 = (List) MMMModelFactory.getInstance().getClassMap().get(
                classURI);
        Iterator i = l2.iterator();
        while (i.hasNext()) {
            PropertyDescriptor pd = (PropertyDescriptor) i.next();
            String[] temp = { pd.getURI(), pd.getLabel() };
            l.add(temp);
        }
        return l;
    }

    // end new stuff

    // public static String createPicture(String filePath) {
    // String newURI = MMMObject.BASE_URI + filePath;
    // if (createInstance(newURI,Picture.PROPERTY_FILE_PATH,filePath)) {
    // return newURI;
    // } else {
    // return null;
    // }
    // }
    //	
    // public static String createAlbum(String label) {
    // String newURI=MMMObject.BASE_URI + "album" +label;
    // if (createInstance(newURI,PROPERTY_HAS_LABEL,label)) {
    // return newURI;
    // }else {
    // return null;
    // }
    // }

    // public static String createShare(String initiatedBy, String sharesMedia)
    // {
    // String newURI=MMMObject.BASE_URI + "share" +initiatedBy+sharesMedia;
    // if (createInstance(newURI,Share.PROPERTY_INITIATED_BY,
    // MMMObject.BASE_URI+initiatedBy)) {
    // addProperty(newURI, Share.PROPERTY_SHARES_MEDIA,
    // MMMObject.BASE_URI+sharesMedia);
    // return newURI;
    // }else {
    // return null;
    // }
    // }
    //	
    // public static String createPerson(String fName, String lName) {
    // String newURI=MMMObject.BASE_URI + "person" + fName + lName;
    // if (createInstance(newURI,Person.PROPERTY_FIRST_NAME,fName)) {
    // addProperty(newURI, Person.PROPERTY_LAST_NAME, lName);
    // return newURI;
    // }else {
    // return null;
    // }
    // }
    //	
    // public static String createNonPerson(String label, String thingType) {
    // String newURI=MMMObject.BASE_URI + "np" + label;
    // if (createInstance(newURI, PROPERTY_ISA, MMMObject.BASE_URI + thingType))
    // {
    // addProperty(newURI, PROPERTY_HAS_LABEL, label);
    // return newURI;
    // }else {
    // System.out.println("Non Person Creation Failed");
    // return null;
    // }
    // }

    public static ExtendedIterator listProperties(String classURI) {
        OntClass c = MMMModelFactory.getInstance().getOntModel().getOntClass(
                classURI);
        if (c == null) {
            return null;
        }

        // Iterator i = c.listDeclaredProperties();
        //		
        // while(i.hasNext()){
        // OntProperty p = (OntProperty)i.next();
        // System.out.println("\t" + p.getNameSpace() + ": " + p + ", " +
        // p.getLabel(null));
        // }

        return c.listDeclaredProperties();
    }

    public static HashMap getFilteredProperties(String classURI,
            String nameSpace, boolean includeSuperProps) {
        OntClass c = MMMModelFactory.getInstance().getOntModel().getOntClass(
                classURI);
        if (c == null) {
            return null;
        } // or should we return empty HashMap?
        Iterator i = c.listDeclaredProperties();
        HashMap hm = new HashMap();
        while (i.hasNext()) {
            OntProperty p = (OntProperty) i.next();
            // System.out.println("\t" + p.getNameSpace() + ": " + p + ", " +
            // p.getLabel(null));
            if (nameSpace.equals(p.getNameSpace())) {
                if ((!p.listSubProperties().hasNext()) || (includeSuperProps)) {
                    hm.put(p.toString(), p.getLabel(null));
                }
            }
        }
        return hm;
    }

    public static HashMap getFilteredProperties(String classURI) {
        return getFilteredProperties(classURI, MMMObject.BASE_URI, false);
    }

    // public static Iterator listDeclaredInstanceProperties(String pictureURI){
    //		
    // if(!individualExists(pictureURI)){return null;}
    //		
    // Resource pictureResource = m.createResource(pictureURI);
    //		
    // ExtendedIterator ei = listProperties(PICTURE_CLASS_URI);
    //		
    // ArrayList list = new ArrayList();
    //		
    // while(ei.hasNext()){
    // Property p = (Property) ei.next();
    //			
    // if(pictureResource.hasProperty(p)){
    // list.add(p);
    // }
    // }
    // return list.iterator();
    // }

    public static StmtIterator listInstancePropertiesValues(String pictureURI) {
        if (!individualExists(pictureURI)) {
            return null;
        }
        return MMMModelFactory.getInstance().getOntModel().createResource(
                pictureURI).listProperties();
    }

    public static Iterator listDeclaredInstancePropertyValues(
            String subjectURI, String propertyURI) {
        Resource r = MMMModelFactory.getInstance().getOntModel().getIndividual(
                subjectURI);
        Property p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(propertyURI);
        List l = new ArrayList();
        if ((r != null) && (p != null)) {
            StmtIterator i = MMMModelFactory.getInstance().getOntModel()
                    .listStatements(r, p, (RDFNode) null);
            while (i.hasNext()) {
                Statement s = (Statement) i.next();
                l.add(s.getObject());
            }
        }
        return l.iterator();
    }

    public static StmtIterator getPropertyValues(String pictureURI,
            String propertyURI) {

        if (!individualExists(pictureURI)) {
            return null;
        }

        Resource pictureResource = MMMModelFactory.getInstance().getOntModel()
                .createResource(pictureURI);
        Property property = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(propertyURI);

        return pictureResource.listProperties(property);
    }

    public static boolean isPropertyLiteral(String propertyURI) {
        OntProperty property = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(propertyURI);
        if (property == null) {
            return false;
        }
        return property.isDatatypeProperty();
    }

    public static Iterator getPropertyObjectRange(String propertyURI) {
        if (isPropertyLiteral(propertyURI)) {
            return null;
        }
        OntProperty p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(propertyURI);
        if (p == null) {
            return null;
        }
        HashSet hs = new HashSet();

        ExtendedIterator ei = p.listRange();
        while (ei.hasNext()) {
            OntResource r = (OntResource) ei.next();
            OntClass currentClass = r.asClass();
            if (currentClass.isUnionClass()) {
                UnionClass uc = currentClass.asUnionClass();
                addOperandClasses(hs, uc);
            } else {
                hs.add(currentClass);
            }
        }

        return hs.iterator();
    }

    public static void addOperandClasses(HashSet hs, UnionClass uc) {
        ExtendedIterator ei = uc.listOperands();
        while (ei.hasNext()) {
            OntClass currentClass = (OntClass) ei.next();
            if (currentClass.isUnionClass()) {
                addOperandClasses(hs, currentClass.asUnionClass());
            } else {
                hs.add(currentClass);
            }
        }
    }

    public static Collection expandOntClasses(ExtendedIterator classList) {

        HashSet hs = new HashSet();

        while (classList.hasNext()) {
            Resource current = (Resource) classList.next();
            OntClass currentClass = (OntClass) current.as(OntClass.class);
            if (currentClass.isUnionClass()) {
                addOperandClasses(hs, currentClass.asUnionClass());
            } else {
                hs.add(currentClass);
            }
        }

        return hs;
    }

    public static Iterator getIndividualsInRange(String propURI) {
        Iterator ci = getPropertyObjectRange(propURI);
        List l = new ArrayList();
        while (ci.hasNext()) {
            OntClass c = (OntClass) ci.next();
            // System.out.println("Class: " + c);
            Iterator ii = getIndividualsOfClass(c);
            while (ii.hasNext()) {
                l.add(ii.next());
            }
        }
        return l.iterator();
    }

    public static Iterator getIndividualsOfClass(OntClass c) {

        if (c.isUnionClass()) {
            System.out.println("THIS IS A UNION CLASS");
            System.out.println(c.getSubClass());
            UnionClass uc = c.asUnionClass();
            ExtendedIterator ei = uc.listOperands();

            // PictureAnnotator.printIterator(ei);

            HashSet hs = new HashSet();

            while (ei.hasNext()) {
                OntClass currentClass = (OntClass) ei.next();

                Iterator i = getIndividualsOfClass(currentClass);
                while (i.hasNext()) {
                    hs.add(i.next());
                }
            }

            return hs.iterator();
        }

        if ((c == null)) {
            return null;
        }
        return c.listInstances();

    }

    public static boolean isInRange(OntProperty p, OntResource r) {

        Iterator i = expandOntClasses(r.listRDFTypes(false)).iterator();

        HashSet ranges = new HashSet(expandOntClasses(p.listRange()));

        while (i.hasNext()) {
            Resource current = (Resource) i.next();
            OntClass c = (OntClass) current.as(OntClass.class);
            System.out.println("OntClass: " + c);
            if (ranges.contains(c))
                return true;
            if (p.hasRange(c))
                return true;// for union class compatibility
        }
        return false;

    }

    public static boolean addProperty(String subjectURI, String propertyURI,
            String value) {

        if (!individualExists(subjectURI)) {
            return false;
        }

        Resource r = MMMModelFactory.getInstance().getOntModel()
                .createResource(subjectURI); // create == get

        // all good
        OntProperty p = MMMModelFactory.getInstance().getOntModel()
                .getOntProperty(propertyURI);
        // System.out.println("Property Range: ");
        // PictureAnnotator.printIterator(p.listRange());
        if (p.isObjectProperty()) {
            Individual ind = MMMModelFactory.getInstance().getOntModel()
                    .getIndividual(value);
            if (ind == null) {
                return false;
            }
            if (isInRange(p, ind)) {
                // r.addProperty(p, value);
                r.addProperty(p, ind); // could check return to see if it's
                // null
                return true;
            }
            // System.out.println("addProperty: bad range: " + p + ", " + ind);

        } else if (p.isDatatypeProperty()) {
            r.addProperty(p, value);
            return true;
        }
        // done
        return false;
    }

    public static boolean individualExists(String uri) {
        return (MMMModelFactory.getInstance().getOntModel().getIndividual(uri) != null);
    }

    public static void delete(String uri) {
        Individual ind = MMMModelFactory.getInstance().getOntModel()
                .getIndividual(uri);
        if (ind != null) {
            // remove object properties for this subject
            StmtIterator si = MMMModelFactory.getInstance().getOntModel()
                    .listStatements(ind, (Property) null, (RDFNode) null);
            while (si.hasNext()) {
                Statement s = si.nextStatement();
                MMMModelFactory.getInstance().getOntModel().remove(s);
            }
            // remove String properties for this subject
            si = MMMModelFactory.getInstance().getOntModel().listStatements(
                    ind, (Property) null, (String) null);
            while (si.hasNext()) {
                Statement s = si.nextStatement();
                MMMModelFactory.getInstance().getOntModel().remove(s);
            }
            // remove statements with object=inputURI
            si = MMMModelFactory.getInstance().getOntModel().listStatements(
                    (Resource) null, (Property) null, ind);
            while (si.hasNext()) {
                Statement s = si.nextStatement();
                MMMModelFactory.getInstance().getOntModel().remove(s);
            }
        }
    }

}
